@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -39,6 +39,7 @@
 #if defined WITH_AVRO
 #include "plugin_cmn_avro.h"
 #endif
+#include "bgp_lg.h"
 
 /* Global variables */
 thread_pool_t *bgp_pool;
@@ -57,10 +58,19 @@ struct bgp_misc_structs inter_domain_misc_dbs[FUNC_TYPE_MAX], *bgp_misc_db;
 struct bgp_xconnects bgp_xcs_map;
 
 /* Functions */
-void nfacctd_bgp_wrapper()
+void bgp_daemon_wrapper()
 {
   /* initialize variables */
-  if (!config.nfacctd_bgp_port) config.nfacctd_bgp_port = BGP_TCP_PORT;
+  if (!config.bgp_daemon_port) config.bgp_daemon_port = BGP_TCP_PORT;
+
+#if defined WITH_ZMQ
+  if (config.bgp_lg) bgp_lg_wrapper();
+#else
+  if (config.bgp_lg) {
+    Log(LOG_ERR, "ERROR ( %s/core/lg ): 'bgp_daemon_lg' requires --enable-zmq. Exiting.\n", config.name);
+    exit_gracefully(1);
+  }
+#endif
 
   /* initialize threads pool */
   bgp_pool = allocate_thread_pool(1);
@@ -72,21 +82,21 @@ void nfacctd_bgp_wrapper()
   send_to_pool(bgp_pool, skinny_bgp_daemon, NULL);
 }
 
-void skinny_bgp_daemon()
+int skinny_bgp_daemon()
 {
   skinny_bgp_daemon_online();
+
+  return SUCCESS;
 }
 
 void skinny_bgp_daemon_online()
 {
   int ret, rc, peers_idx, allowed, yes=1;
-#if (defined IPV6_BINDV6ONLY)
-  int no=0;
-#endif
   int peers_idx_rr = 0, peers_xconnect_idx_rr = 0, max_peers_idx = 0;
   struct plugin_requests req;
   struct host_addr addr;
   struct bgp_peer *peer;
+  struct bgp_peer_buf *peer_buf;
   char bgp_reply_pkt[BGP_BUFFER_SIZE], *bgp_reply_pkt_ptr;
   char bgp_peer_str[INET6_ADDRSTRLEN], bgp_xconnect_peer_str[BGP_XCONNECT_STRLEN];
   struct sockaddr_storage server, client;
@@ -120,54 +130,49 @@ void skinny_bgp_daemon_online()
   bgp_attr_init(config.bgp_table_attr_hash_buckets, bgp_routing_db);
 
   /* socket creation for BGP server: IPv4 only */
-  if (!config.nfacctd_bgp_ip) {
+  if (!config.bgp_daemon_ip) {
     struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&server;
 
     sa6->sin6_family = AF_INET6;
-    sa6->sin6_port = htons(config.nfacctd_bgp_port);
+    sa6->sin6_port = htons(config.bgp_daemon_port);
     slen = sizeof(struct sockaddr_in6);
   }
   else {
-    trim_spaces(config.nfacctd_bgp_ip);
-    ret = str_to_addr(config.nfacctd_bgp_ip, &addr);
+    trim_spaces(config.bgp_daemon_ip);
+    ret = str_to_addr(config.bgp_daemon_ip, &addr);
     if (!ret) {
       Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_daemon_ip' value is not a valid IPv4/IPv6 address. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
-    slen = addr_to_sa((struct sockaddr *)&server, &addr, config.nfacctd_bgp_port);
+    slen = addr_to_sa((struct sockaddr *)&server, &addr, config.bgp_daemon_port);
   }
 
-  if (!config.nfacctd_bgp_max_peers) config.nfacctd_bgp_max_peers = MAX_BGP_PEERS_DEFAULT;
-  Log(LOG_INFO, "INFO ( %s/%s ): maximum BGP peers allowed: %d\n", config.name, bgp_misc_db->log_str, config.nfacctd_bgp_max_peers);
+  if (!config.bgp_daemon_max_peers) config.bgp_daemon_max_peers = MAX_BGP_PEERS_DEFAULT;
+  Log(LOG_INFO, "INFO ( %s/%s ): maximum BGP peers allowed: %d\n", config.name, bgp_misc_db->log_str, config.bgp_daemon_max_peers);
 
-  peers = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer));
+  peers = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer));
   if (!peers) {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
     exit_gracefully(1);
   }
-  memset(peers, 0, config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer));
+  memset(peers, 0, config.bgp_daemon_max_peers*sizeof(struct bgp_peer));
 
   if (config.bgp_lg) {
-    if (config.acct_type != ACCT_PMBGP) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): bgp_daemon_lg feature not supported for this daemon. Exiting ...\n", config.name, config.type);
-      exit_gracefully(1);
-    }
-
-    peers_cache = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_cache_bucket));
+    peers_cache = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_cache_bucket));
     if (!peers_cache) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers cache structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_peer_cache_init(peers_cache, config.nfacctd_bgp_max_peers);
+    bgp_peer_cache_init(peers_cache, config.bgp_daemon_max_peers);
 
-    peers_port_cache = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_cache_bucket));
+    peers_port_cache = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_cache_bucket));
     if (!peers_port_cache) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers cache structure (2). Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_peer_cache_init(peers_port_cache, config.nfacctd_bgp_max_peers);
+    bgp_peer_cache_init(peers_port_cache, config.bgp_daemon_max_peers);
   }
   else {
     peers_cache = NULL;
@@ -176,6 +181,7 @@ void skinny_bgp_daemon_online()
 
   if (config.bgp_xconnect_map) {
     int bgp_xcs_allocated = FALSE;
+    int bgp_xcs_size = config.maps_entries ? config.maps_entries : MAX_PRETAG_MAP_ENTRIES;
 
     if (config.acct_type != ACCT_PMBGP) {
       Log(LOG_ERR, "ERROR ( %s/%s ): bgp_daemon_xconnect_map feature not supported for this daemon. Exiting ...\n", config.name, config.type);
@@ -186,14 +192,15 @@ void skinny_bgp_daemon_online()
     memset(&req, 0, sizeof(req));
 
     /* Setting up the pool */
-    bgp_xcs_map.pool = malloc((config.nfacctd_bgp_max_peers + 1) * sizeof(struct bgp_xconnect));
+    bgp_xcs_map.pool = malloc(bgp_xcs_size * sizeof(struct bgp_xconnect));
     if (!bgp_xcs_map.pool) {
       Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate BGP xconnect pool. Exiting ...\n", config.name, config.type);
       exit_gracefully(1);
     }
-    else memset(bgp_xcs_map.pool, 0, (config.nfacctd_bgp_max_peers + 1) * sizeof(struct bgp_xconnect));
+    else memset(bgp_xcs_map.pool, 0, bgp_xcs_size * sizeof(struct bgp_xconnect));
 
     req.key_value_table = (void *) &bgp_xcs_map;
+    req.map_entries = bgp_xcs_size;
     load_id_file(MAP_BGP_XCS, config.bgp_xconnect_map, NULL, &req, &bgp_xcs_allocated);
   }
   else {
@@ -227,25 +234,25 @@ void skinny_bgp_daemon_online()
 #endif
   }
 
-  if (config.nfacctd_bgp_msglog_file || config.nfacctd_bgp_msglog_amqp_routing_key || config.nfacctd_bgp_msglog_kafka_topic) {
-    if (config.nfacctd_bgp_msglog_file) bgp_misc_db->msglog_backend_methods++;
-    if (config.nfacctd_bgp_msglog_amqp_routing_key) bgp_misc_db->msglog_backend_methods++;
-    if (config.nfacctd_bgp_msglog_kafka_topic) bgp_misc_db->msglog_backend_methods++;
+  if (config.bgp_daemon_msglog_file || config.bgp_daemon_msglog_amqp_routing_key || config.bgp_daemon_msglog_kafka_topic) {
+    if (config.bgp_daemon_msglog_file) bgp_misc_db->msglog_backend_methods++;
+    if (config.bgp_daemon_msglog_amqp_routing_key) bgp_misc_db->msglog_backend_methods++;
+    if (config.bgp_daemon_msglog_kafka_topic) bgp_misc_db->msglog_backend_methods++;
 
     if (bgp_misc_db->msglog_backend_methods > 1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): bgp_daemon_msglog_file, bgp_daemon_msglog_amqp_routing_key and bgp_daemon_msglog_kafka_topic are mutually exclusive. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_misc_db->peers_log = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_log));
+    bgp_misc_db->peers_log = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_log));
     if (!bgp_misc_db->peers_log) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers log structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
-    memset(bgp_misc_db->peers_log, 0, config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_log));
+    memset(bgp_misc_db->peers_log, 0, config.bgp_daemon_max_peers*sizeof(struct bgp_peer_log));
     bgp_peer_log_seq_init(&bgp_misc_db->log_seq);
 
-    if (config.nfacctd_bgp_msglog_amqp_routing_key) {
+    if (config.bgp_daemon_msglog_amqp_routing_key) {
 #ifdef WITH_RABBITMQ
       bgp_daemon_msglog_init_amqp_host();
       p_amqp_connect_to_publish(&bgp_daemon_msglog_amqp_host);
@@ -254,7 +261,7 @@ void skinny_bgp_daemon_online()
 #endif
     }
 
-    if (config.nfacctd_bgp_msglog_kafka_topic) {
+    if (config.bgp_daemon_msglog_kafka_topic) {
 #ifdef WITH_KAFKA
       bgp_daemon_msglog_init_kafka_host();
 #else
@@ -292,12 +299,12 @@ void skinny_bgp_daemon_online()
   config.bgp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
   if (config.bgp_sock < 0) {
     /* retry with IPv4 */
-    if (!config.nfacctd_bgp_ip) {
+    if (!config.bgp_daemon_ip) {
       struct sockaddr_in *sa4 = (struct sockaddr_in *)&server;
 
       sa4->sin_family = AF_INET;
       sa4->sin_addr.s_addr = htonl(0);
-      sa4->sin_port = htons(config.nfacctd_bgp_port);
+      sa4->sin_port = htons(config.bgp_daemon_port);
       slen = sizeof(struct sockaddr_in);
 
       config.bgp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
@@ -308,37 +315,42 @@ void skinny_bgp_daemon_online()
       exit_gracefully(1);
     }
   }
-  if (config.nfacctd_bgp_ipprec) {
-    int opt = config.nfacctd_bgp_ipprec << 5;
+
+  setsockopt(config.bgp_sock, SOL_SOCKET, SO_KEEPALIVE, (void *)&yes, sizeof(yes));
+
+  if (config.bgp_daemon_ipprec) {
+    int opt = config.bgp_daemon_ipprec << 5;
 
     rc = setsockopt(config.bgp_sock, IPPROTO_IP, IP_TOS, &opt, (socklen_t) sizeof(opt));
     if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IP_TOS (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
   }
 
-#if (defined LINUX) && (defined HAVE_SO_REUSEPORT)
-  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR|SO_REUSEPORT, (char *)&yes, (socklen_t) sizeof(yes));
-  if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR|SO_REUSEPORT (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
-#else
+#if (defined HAVE_SO_REUSEPORT)
+  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEPORT, (char *)&yes, (socklen_t) sizeof(yes));
+  if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEPORT (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
+#endif
+
   rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, (socklen_t) sizeof(yes));
   if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
-#endif
 
-#if (defined IPV6_BINDV6ONLY)
-  rc = setsockopt(config.bgp_sock, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *) &no, (socklen_t) sizeof(no));
-  if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IPV6_BINDV6ONLY (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
-#endif
+  if (config.bgp_daemon_ipv6_only) {
+    int yes=1;
 
-  if (config.nfacctd_bgp_pipe_size) {
-    socklen_t l = sizeof(config.nfacctd_bgp_pipe_size);
+    rc = setsockopt(config.bgp_sock, IPPROTO_IPV6, IPV6_V6ONLY, (char *) &yes, (socklen_t) sizeof(yes));
+    if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IPV6_V6ONLY (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
+  }
+
+  if (config.bgp_daemon_pipe_size) {
+    socklen_t l = sizeof(config.bgp_daemon_pipe_size);
     int saved = 0, obtained = 0;
 
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &saved, &l);
-    Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &config.nfacctd_bgp_pipe_size, (socklen_t) sizeof(config.nfacctd_bgp_pipe_size));
+    Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &config.bgp_daemon_pipe_size, (socklen_t) sizeof(config.bgp_daemon_pipe_size));
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &obtained, &l);
 
     Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &saved, l);
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &obtained, &l);
-    Log(LOG_INFO, "INFO ( %s/%s ): bgp_daemon_pipe_size: obtained=%d target=%d.\n", config.name, bgp_misc_db->log_str, obtained, config.nfacctd_bgp_pipe_size);
+    Log(LOG_INFO, "INFO ( %s/%s ): bgp_daemon_pipe_size: obtained=%d target=%d.\n", config.name, bgp_misc_db->log_str, obtained, config.bgp_daemon_pipe_size);
   }
 
   rc = bind(config.bgp_sock, (struct sockaddr *) &server, slen);
@@ -346,8 +358,8 @@ void skinny_bgp_daemon_online()
     char null_ip_address[] = "0.0.0.0";
     char *ip_address;
 
-    ip_address = config.nfacctd_bgp_ip ? config.nfacctd_bgp_ip : null_ip_address;
-    Log(LOG_ERR, "ERROR ( %s/%s ): bind() to ip=%s port=%d/tcp failed (errno: %d).\n", config.name, bgp_misc_db->log_str, ip_address, config.nfacctd_bgp_port, errno);
+    ip_address = config.bgp_daemon_ip ? config.bgp_daemon_ip : null_ip_address;
+    Log(LOG_ERR, "ERROR ( %s/%s ): bind() to ip=%s port=%d/tcp failed (errno: %d).\n", config.name, bgp_misc_db->log_str, ip_address, config.bgp_daemon_port, errno);
     exit_gracefully(1);
   }
 
@@ -373,12 +385,12 @@ void skinny_bgp_daemon_online()
   }
 
   /* Preparing ACL, if any */
-  if (config.nfacctd_bgp_allow_file) load_allow_file(config.nfacctd_bgp_allow_file, &allow);
+  if (config.bgp_daemon_allow_file) load_allow_file(config.bgp_daemon_allow_file, &allow);
 
   /* Preparing MD5 keys, if any */
-  if (config.nfacctd_bgp_md5_file) {
+  if (config.bgp_daemon_md5_file) {
     bgp_md5_file_init(&bgp_md5);
-    bgp_md5_file_load(config.nfacctd_bgp_md5_file, &bgp_md5);
+    bgp_md5_file_load(config.bgp_daemon_md5_file, &bgp_md5);
     if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5);
   }
 
@@ -390,73 +402,63 @@ void skinny_bgp_daemon_online()
   }
 
   /* BGP peers batching checks */
-  if ((config.nfacctd_bgp_batch && !config.nfacctd_bgp_batch_interval) ||
-      (config.nfacctd_bgp_batch_interval && !config.nfacctd_bgp_batch)) {
+  if ((config.bgp_daemon_batch && !config.bgp_daemon_batch_interval) ||
+      (config.bgp_daemon_batch_interval && !config.bgp_daemon_batch)) {
     Log(LOG_WARNING, "WARN ( %s/%s ): 'bgp_daemon_batch_interval' and 'bgp_daemon_batch' both set to zero.\n", config.name, bgp_misc_db->log_str);
-    config.nfacctd_bgp_batch = 0;
-    config.nfacctd_bgp_batch_interval = 0;
+    config.bgp_daemon_batch = 0;
+    config.bgp_daemon_batch_interval = 0;
   }
-  else bgp_batch_init(&bp_batch, config.nfacctd_bgp_batch, config.nfacctd_bgp_batch_interval);
+  else bgp_batch_init(&bp_batch, config.bgp_daemon_batch, config.bgp_daemon_batch_interval);
 
   if (bgp_misc_db->msglog_backend_methods) {
 #ifdef WITH_JANSSON
-    if (!config.nfacctd_bgp_msglog_output) config.nfacctd_bgp_msglog_output = PRINT_OUTPUT_JSON;
+    if (!config.bgp_daemon_msglog_output) config.bgp_daemon_msglog_output = PRINT_OUTPUT_JSON;
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bgp_daemon_msglog_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bgp_misc_db->log_str);
 #endif
 
 #ifdef WITH_AVRO
-    if ((config.nfacctd_bgp_msglog_output == PRINT_OUTPUT_AVRO_BIN) ||
-	(config.nfacctd_bgp_msglog_output == PRINT_OUTPUT_AVRO_JSON)) {
+    if ((config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_BIN) ||
+	(config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_JSON)) {
       assert(BGP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
 
-      bgp_misc_db->msglog_avro_schema[0] = avro_schema_build_bgp(BGP_LOGDUMP_ET_LOG, "bgp_msglog");
-      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT] = avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_loginit");
-      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE] = avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_logclose");
+      bgp_misc_db->msglog_avro_schema[0] = p_avro_schema_build_bgp(BGP_LOGDUMP_ET_LOG, "bgp_msglog");
+      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT] = p_avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_loginit");
+      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE] = p_avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_logclose");
 
-      if (config.nfacctd_bgp_msglog_avro_schema_file) {
-	char avro_schema_file[SRVBUFLEN];
+      if (config.bgp_daemon_msglog_avro_schema_file) {
+	char p_avro_schema_file[SRVBUFLEN];
 
-	if (strlen(config.nfacctd_bgp_msglog_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	if (strlen(config.bgp_daemon_msglog_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
 	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_daemon_msglog_avro_schema_file' too long. Exiting.\n", config.name, bgp_misc_db->log_str);
 	  exit_gracefully(1);
 	}
 
-	write_avro_schema_to_file_with_suffix(config.nfacctd_bgp_msglog_avro_schema_file, "-bgp_msglog",
-					      avro_schema_file, bgp_misc_db->msglog_avro_schema[0]);
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_msglog",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[0]);
 
-	write_avro_schema_to_file_with_suffix(config.nfacctd_bgp_msglog_avro_schema_file, "-bgp_loginit",
-					      avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT]);
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_loginit",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT]);
 
-	write_avro_schema_to_file_with_suffix(config.nfacctd_bgp_msglog_avro_schema_file, "-bgp_logclose",
-					      avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE]);
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_logclose",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE]);
       }
 
-      if (config.nfacctd_bgp_msglog_kafka_avro_schema_registry) {
+      if (config.bgp_daemon_msglog_kafka_avro_schema_registry) {
 #ifdef WITH_SERDES
-        if (strchr(config.nfacctd_bgp_msglog_kafka_topic, '$')) {
+        if (strchr(config.bgp_daemon_msglog_kafka_topic, '$')) {
           Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'bgp_daemon_msglog_kafka_topic' is not compatible with 'bgp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
 	      config.name, bgp_misc_db->log_str);
 	  exit_gracefully(1);
         }
 
-	if (config.nfacctd_bgp_msglog_output == PRINT_OUTPUT_AVRO_JSON) {
+	if (config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_JSON) {
           Log(LOG_ERR, "ERROR ( %s/%s ): 'avro_json' output is not compatible with 'bgp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
 	      config.name, bgp_misc_db->log_str);
 	  exit_gracefully(1);
 	}
 
-	bgp_daemon_msglog_kafka_host.sd_schema[0] = compose_avro_schema_registry_name_2(config.nfacctd_bgp_msglog_kafka_topic, FALSE,
-										        bgp_misc_db->msglog_avro_schema[0], "bgp", "msglog",
-										        config.nfacctd_bgp_msglog_kafka_avro_schema_registry);
-
-	bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGINIT] = compose_avro_schema_registry_name_2(config.nfacctd_bgp_msglog_kafka_topic, FALSE,
-										        bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT], "bgp", "loginit",
-										        config.nfacctd_bgp_msglog_kafka_avro_schema_registry);
-
-	bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGCLOSE] = compose_avro_schema_registry_name_2(config.nfacctd_bgp_msglog_kafka_topic, FALSE,
-										        bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE], "bgp", "logclose",
-										        config.nfacctd_bgp_msglog_kafka_avro_schema_registry);
+	bgp_daemon_msglog_prepare_sd_schemas();
 #endif
       }
     }
@@ -464,8 +466,14 @@ void skinny_bgp_daemon_online()
   }
 
   if (bgp_misc_db->dump_backend_methods) {
+    if (!config.bgp_table_dump_workers) {
+      config.bgp_table_dump_workers = 1;
+    }
+
 #ifdef WITH_JANSSON
-    if (!config.bgp_table_dump_output) config.bgp_table_dump_output = PRINT_OUTPUT_JSON;
+    if (!config.bgp_table_dump_output) {
+      config.bgp_table_dump_output = PRINT_OUTPUT_JSON;
+    }
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bgp_table_dump_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bgp_misc_db->log_str);
 #endif
@@ -475,12 +483,12 @@ void skinny_bgp_daemon_online()
 	(config.bgp_table_dump_output == PRINT_OUTPUT_AVRO_JSON)) {
       assert(BGP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
 
-      bgp_misc_db->dump_avro_schema[0] = avro_schema_build_bgp(BGP_LOGDUMP_ET_DUMP, "bgp_dump");
-      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT] = avro_schema_build_bgp_dump_init(BGP_LOGDUMP_ET_DUMP, "bgp_dumpinit");
-      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE] = avro_schema_build_bgp_dump_close(BGP_LOGDUMP_ET_DUMP, "bgp_dumpclose");
+      bgp_misc_db->dump_avro_schema[0] = p_avro_schema_build_bgp(BGP_LOGDUMP_ET_DUMP, "bgp_dump");
+      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT] = p_avro_schema_build_bgp_dump_init(BGP_LOGDUMP_ET_DUMP, "bgp_dumpinit");
+      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE] = p_avro_schema_build_bgp_dump_close(BGP_LOGDUMP_ET_DUMP, "bgp_dumpclose");
 
       if (config.bgp_table_dump_avro_schema_file) {
-	char avro_schema_file[SRVBUFLEN];
+	char p_avro_schema_file[SRVBUFLEN];
 
 	if (strlen(config.bgp_table_dump_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
 	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_table_dump_avro_schema_file' too long. Exiting.\n", config.name, bgp_misc_db->log_str);
@@ -488,13 +496,13 @@ void skinny_bgp_daemon_online()
 	}
 
 	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dump",
-					      avro_schema_file, bgp_misc_db->dump_avro_schema[0]);
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[0]);
 
 	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dumpinit",
-					      avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT]);
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT]);
 
 	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dumpclose",
-					      avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE]);
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE]);
       }
     }
 #endif
@@ -519,9 +527,6 @@ void skinny_bgp_daemon_online()
       bgp_misc_db->dump_backend_methods = FALSE;
       Log(LOG_WARNING, "WARN ( %s/%s ): Invalid 'bgp_table_dump_refresh_time'.\n", config.name, bgp_misc_db->log_str);
     }
-
-    if (config.bgp_table_dump_amqp_routing_key) bgp_table_dump_init_amqp_host();
-    if (config.bgp_table_dump_kafka_topic) bgp_table_dump_init_kafka_host();
   }
 
 #ifdef WITH_AVRO
@@ -533,7 +538,7 @@ void skinny_bgp_daemon_online()
   else memset(bgp_misc_db->avro_buf, 0, LARGEBUFLEN);
 #endif
 
-  if (config.nfacctd_bgp_msglog_kafka_avro_schema_registry || config.bgp_table_dump_kafka_avro_schema_registry) {
+  if (config.bgp_daemon_msglog_kafka_avro_schema_registry || config.bgp_table_dump_kafka_avro_schema_registry) {
 #ifndef WITH_SERDES
     Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_*_kafka_avro_schema_registry' require --enable-serdes. Exiting.\n", config.name, bgp_misc_db->log_str);
     exit_gracefully(1);
@@ -567,7 +572,7 @@ void skinny_bgp_daemon_online()
       select_fd = config.bgp_sock;
       max_peers_idx = -1; /* .. since valid indexes include 0 */
 
-      for (peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
         if (select_fd < peers[peers_idx].fd) select_fd = peers[peers_idx].fd; 
 
         if (config.bgp_xconnect_map) {
@@ -603,13 +608,13 @@ void skinny_bgp_daemon_online()
 
     /* signals handling */
     if (reload_map_bgp_thread) {
-      if (config.nfacctd_bgp_allow_file) load_allow_file(config.nfacctd_bgp_allow_file, &allow);
+      if (config.bgp_daemon_allow_file) load_allow_file(config.bgp_daemon_allow_file, &allow);
 
-      if (config.nfacctd_bgp_md5_file) {
+      if (config.bgp_daemon_md5_file) {
 	bgp_md5_file_unload(&bgp_md5);
 	if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5); // process unload
 
-	bgp_md5_file_load(config.nfacctd_bgp_md5_file, &bgp_md5);
+	bgp_md5_file_load(config.bgp_daemon_md5_file, &bgp_md5);
 	if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5); // process load
       }
 
@@ -628,7 +633,7 @@ void skinny_bgp_daemon_online()
     }
 
     if (reload_log_bgp_thread) {
-      for (peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
 	if (bgp_misc_db->peers_log[peers_idx].fd) {
 	  fclose(bgp_misc_db->peers_log[peers_idx].fd);
 	  bgp_misc_db->peers_log[peers_idx].fd = open_output_file(bgp_misc_db->peers_log[peers_idx].filename, "a", FALSE);
@@ -667,14 +672,14 @@ void skinny_bgp_daemon_online()
 	  if (bgp_peer_log_seq_has_ro_bit(&bgp_misc_db->log_seq))
 	    bgp_peer_log_seq_init(&bgp_misc_db->log_seq);
 
-	  bgp_handle_dump_event();
+	  bgp_handle_dump_event(max_peers_idx);
 
 	  dump_refresh_deadline += config.bgp_table_dump_refresh_time;
 	}
       }
 
 #ifdef WITH_RABBITMQ
-      if (config.nfacctd_bgp_msglog_amqp_routing_key) { 
+      if (config.bgp_daemon_msglog_amqp_routing_key) { 
         time_t last_fail = P_broker_timers_get_last_fail(&bgp_daemon_msglog_amqp_host.btimers);
 
 	if (last_fail && ((last_fail + P_broker_timers_get_retry_interval(&bgp_daemon_msglog_amqp_host.btimers)) <= bgp_misc_db->log_tstamp.tv_sec)) {
@@ -685,11 +690,17 @@ void skinny_bgp_daemon_online()
 #endif
 
 #ifdef WITH_KAFKA
-      if (config.nfacctd_bgp_msglog_kafka_topic) {
+      if (config.bgp_daemon_msglog_kafka_topic) {
         time_t last_fail = P_broker_timers_get_last_fail(&bgp_daemon_msglog_kafka_host.btimers);
 
         if (last_fail && ((last_fail + P_broker_timers_get_retry_interval(&bgp_daemon_msglog_kafka_host.btimers)) <= bgp_misc_db->log_tstamp.tv_sec))
           bgp_daemon_msglog_init_kafka_host();
+
+	if (config.bgp_daemon_msglog_kafka_avro_schema_registry) {
+#ifdef WITH_SERDES
+	  bgp_daemon_msglog_prepare_sd_schemas();
+#endif
+	}
       }
 #endif
     }
@@ -715,11 +726,16 @@ void skinny_bgp_daemon_online()
       else allowed = TRUE;
 
       if (!allowed) {
+	char disallowed_str[INET6_ADDRSTRLEN];
+
+	sa_to_str(disallowed_str, sizeof(disallowed_str), (struct sockaddr *) &client);
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] peer '%s' not allowed. close()\n", config.name, bgp_misc_db->log_str, config.bgp_daemon_allow_file, disallowed_str);
+
         close(fd);
         goto read_data;
       }
 
-      for (peer = NULL, peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peer = NULL, peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
         if (!peers[peers_idx].fd) {
 	  /*
 	     Admitted if:
@@ -757,8 +773,11 @@ void skinny_bgp_daemon_online()
 
       if (!peer) {
 	/* We briefly accept the new connection to be able to drop it */
-        Log(LOG_ERR, "ERROR ( %s/%s ): Insufficient number of BGP peers has been configured by 'bgp_daemon_max_peers' (%d).\n",
-			config.name, bgp_misc_db->log_str, config.nfacctd_bgp_max_peers);
+	if (!log_notification_isset(&log_notifications.bgp_peers_limit, now)) {
+	  log_notification_set(&log_notifications.bgp_peers_limit, now, FALSE);
+          Log(LOG_WARNING, "WARN ( %s/%s ): Insufficient number of BGP peers has been configured by 'bgp_daemon_max_peers' (%d).\n",
+			config.name, bgp_misc_db->log_str, config.bgp_daemon_max_peers);
+	}
 
 	close(fd);
 	goto read_data;
@@ -767,31 +786,23 @@ void skinny_bgp_daemon_online()
       peer->fd = fd;
       peer->idx = peers_idx; 
       FD_SET(peer->fd, &bkp_read_descs);
-      peer->addr.family = ((struct sockaddr *)&client)->sa_family;
-      if (peer->addr.family == AF_INET) {
-	peer->addr.address.ipv4.s_addr = ((struct sockaddr_in *)&client)->sin_addr.s_addr;
-	peer->tcp_port = ntohs(((struct sockaddr_in *)&client)->sin_port);
-      }
-      else if (peer->addr.family == AF_INET6) {
-	memcpy(&peer->addr.address.ipv6, &((struct sockaddr_in6 *)&client)->sin6_addr, 16);
-	peer->tcp_port = ntohs(((struct sockaddr_in6 *)&client)->sin6_port);
-      }
+      sa_to_addr((struct sockaddr *) &client, &peer->addr, &peer->tcp_port);
 
       if (peers_cache && peers_port_cache) {
 	u_int32_t bucket;
 
-	bucket = addr_hash(&peer->addr, config.nfacctd_bgp_max_peers);
+	bucket = addr_hash(&peer->addr, config.bgp_daemon_max_peers);
 	bgp_peer_cache_insert(peers_cache, bucket, peer);
 
-	bucket = addr_port_hash(&peer->addr, peer->tcp_port, config.nfacctd_bgp_max_peers);
+	bucket = addr_port_hash(&peer->addr, peer->tcp_port, config.bgp_daemon_max_peers);
 	bgp_peer_cache_insert(peers_port_cache, bucket, peer);
       }
 
       if (bgp_misc_db->msglog_backend_methods)
-	bgp_peer_log_init(peer, config.nfacctd_bgp_msglog_output, FUNC_TYPE_BGP);
+	bgp_peer_log_init(peer, config.bgp_daemon_msglog_output, FUNC_TYPE_BGP);
 
       /* Check: more than one TCP connection from a peer (IP address) */
-      for (peers_check_idx = 0, peers_num = 0; peers_check_idx < config.nfacctd_bgp_max_peers; peers_check_idx++) { 
+      for (peers_check_idx = 0, peers_num = 0; peers_check_idx < config.bgp_daemon_max_peers; peers_check_idx++) { 
 	if (peers_idx != peers_check_idx && !memcmp(&peers[peers_check_idx].addr, &peer->addr, sizeof(peers[peers_check_idx].addr))) {
 	  int same_peer = FALSE;
 
@@ -855,15 +866,15 @@ void skinny_bgp_daemon_online()
       if (!config.bgp_xconnect_map) {
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP peers usage: %u/%u\n", config.name, bgp_misc_db->log_str,
-		bgp_peer_str, peers_num, config.nfacctd_bgp_max_peers);
+		bgp_peer_str, peers_num, config.bgp_daemon_max_peers);
       }
       else {
         bgp_peer_xconnect_print(peer, bgp_xconnect_peer_str, BGP_XCONNECT_STRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP xconnects usage: %u/%u\n", config.name, bgp_misc_db->log_str,
-		bgp_xconnect_peer_str, peers_num, config.nfacctd_bgp_max_peers);
+		bgp_xconnect_peer_str, peers_num, config.bgp_daemon_max_peers);
       }
 
-      if (config.nfacctd_bgp_neighbors_file) write_neighbors_file(config.nfacctd_bgp_neighbors_file, FUNC_TYPE_BGP);
+      if (config.bgp_daemon_neighbors_file) write_neighbors_file(config.bgp_daemon_neighbors_file, FUNC_TYPE_BGP);
     }
 
     read_data:
@@ -873,24 +884,25 @@ void skinny_bgp_daemon_online()
        FvD: To avoid starvation of the "later established" peers, we
        offset the start of the search in a round-robin style.
     */
-    for (peer = NULL, peers_idx = 0; peers_idx < max_peers_idx; peers_idx++) {
+    for (peer = NULL, peer_buf = NULL, peers_idx = 0; peers_idx < max_peers_idx; peers_idx++) {
       int loc_idx = (peers_idx + peers_idx_rr) % max_peers_idx;
       recv_fd = 0; send_fd = 0;
 
       if (peers[loc_idx].fd && FD_ISSET(peers[loc_idx].fd, &read_descs)) {
         peer = &peers[loc_idx];
+	peer_buf = &peer->buf;
 	recv_fd = peer->fd;
 	if (config.bgp_xconnect_map) send_fd = peer->xconnect_fd;
         peers_idx_rr = (peers_idx_rr + 1) % max_peers_idx;
 	break;
       }
       
-      // XXX: verify round-robin fairness holding up
       if (config.bgp_xconnect_map) {
         loc_idx = (peers_idx + peers_xconnect_idx_rr) % max_peers_idx;
 
 	if (peers[loc_idx].xconnect_fd && FD_ISSET(peers[loc_idx].xconnect_fd, &read_descs)) {
 	  peer = &peers[loc_idx];
+	  peer_buf = &peer->xbuf;
 	  recv_fd = peer->xconnect_fd;
 	  send_fd = peer->fd;
 	  peers_xconnect_idx_rr = (peers_xconnect_idx_rr + 1) % max_peers_idx;
@@ -901,8 +913,59 @@ void skinny_bgp_daemon_online()
 
     if (!peer) goto select_again;
 
-    ret = recv(recv_fd, &peer->buf.base[peer->buf.truncated_len], (peer->buf.len - peer->buf.truncated_len), 0);
-    peer->msglen = (ret + peer->buf.truncated_len);
+    if (!peer_buf->exp_len) {
+      ret = recv(recv_fd, &peer_buf->base[peer_buf->cur_len], (BGP_HEADER_SIZE - peer_buf->cur_len), 0);
+
+      if (ret > 0) {
+	peer_buf->cur_len += ret;
+
+	if (peer_buf->cur_len == BGP_HEADER_SIZE) {
+	  struct bgp_header *bhdr = (struct bgp_header *) peer_buf->base;
+
+	  if (bgp_marker_check(bhdr, BGP_MARKER_SIZE) == ERR) {
+	    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (marker check failed).\n",
+		config.name, bgp_misc_db->log_str, bgp_peer_str);
+
+	    peer->msglen = 0;
+	    peer_buf->cur_len = 0;
+	    peer_buf->exp_len = 0;
+	    ret = ERR;
+	  }
+	  else {
+	    peer_buf->exp_len = ntohs(bhdr->bgpo_len);
+
+	    /* commit */
+	    if (peer_buf->cur_len == peer_buf->exp_len) {
+	      peer->msglen = peer_buf->exp_len;
+	      peer_buf->cur_len = 0;
+	      peer_buf->exp_len = 0;
+	    }
+	  }
+	}
+	else {
+	  goto select_again;
+	}
+      }
+    }
+
+    if (peer_buf->exp_len) {
+      ret = recv(recv_fd, &peer_buf->base[peer_buf->cur_len], (peer_buf->exp_len - peer_buf->cur_len), 0);
+
+      if (ret > 0) {
+	peer_buf->cur_len += ret;
+
+	/* commit */
+        if (peer_buf->cur_len == peer_buf->exp_len) {
+	  peer->msglen = peer_buf->exp_len;
+	  peer_buf->cur_len = 0;
+	  peer_buf->exp_len = 0;
+	}
+	else {
+	  goto select_again;
+	}
+      }
+    }
 
     if (ret <= 0) {
       if (!config.bgp_xconnect_map) {
@@ -953,7 +1016,7 @@ void skinny_bgp_daemon_online()
 	}
       }
       else {
-	ret = send(send_fd, &peer->buf.base[peer->buf.truncated_len], peer->msglen, 0);
+	ret = send(send_fd, peer_buf->base, peer->msglen, 0);
 	if (ret <= 0) {
 	  bgp_peer_xconnect_print(peer, bgp_xconnect_peer_str, BGP_XCONNECT_STRLEN);
 
@@ -983,7 +1046,7 @@ void bgp_prepare_thread()
   memset(bgp_misc_db, 0, sizeof(struct bgp_misc_structs));
 
   bgp_misc_db->is_thread = TRUE;
-  bgp_misc_db->has_lglass = FALSE;
+  if (config.bgp_lg) bgp_misc_db->has_lglass = TRUE;
 
   if (config.rpki_roas_file || config.rpki_rtr_cache) {
     bgp_misc_db->bnv = malloc(sizeof(struct bgp_node_vector));
@@ -1014,3 +1077,41 @@ void bgp_prepare_daemon()
   bgp_misc_db->log_str = malloc(strlen("core") + 1);
   strcpy(bgp_misc_db->log_str, "core");
 }
+
+void bgp_daemon_msglog_prepare_sd_schemas()
+{
+#ifdef WITH_SERDES
+  time_t last_fail = P_broker_timers_get_last_fail(&bgp_daemon_msglog_kafka_host.sd_schema_timers);
+
+  if ((last_fail + P_broker_timers_get_retry_interval(&bgp_daemon_msglog_kafka_host.sd_schema_timers)) <= bgp_misc_db->log_tstamp.tv_sec) {
+    if (!bgp_daemon_msglog_kafka_host.sd_schema[0]) {
+      bgp_daemon_msglog_kafka_host.sd_schema[0] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+												     bgp_misc_db->msglog_avro_schema[0],
+												     "bgp", "msglog",
+												     config.bgp_daemon_msglog_kafka_avro_schema_registry);
+      if (!bgp_daemon_msglog_kafka_host.sd_schema[0]) goto exit_lane;
+    }
+
+    if (!bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGINIT]) {
+      bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGINIT] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+												     bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT],
+												     "bgp", "loginit",
+												     config.bgp_daemon_msglog_kafka_avro_schema_registry);
+      if (!bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGINIT]) goto exit_lane;
+    }
+
+    if (!bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGCLOSE]) {
+      bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGCLOSE] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+												     bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE],
+												     "bgp", "logclose",
+												     config.bgp_daemon_msglog_kafka_avro_schema_registry);
+      if (!bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGCLOSE]) goto exit_lane;
+    }
+  }
+
+  return;
+
+  exit_lane:
+  P_broker_timers_set_last_fail(&bgp_daemon_msglog_kafka_host.sd_schema_timers, time(NULL));
+#endif
+}
