@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -20,6 +20,7 @@
 */
 
 /* includes */
+#include <pthread.h>
 #include <sys/poll.h>
 #include "bgp_prefix.h"
 #include "bgp_packet.h"
@@ -63,7 +64,14 @@
 #define BGP_ATTR_AS4_PATH                       17
 #define BGP_ATTR_AS4_AGGREGATOR                 18
 #define BGP_ATTR_AS_PATHLIMIT                   21
-#define BGP_ATTR_LARGE_COMMUNITIES		32 /* rfc8092 (draft-ietf-idr-large-community) */
+#define BGP_ATTR_AIGP				26
+#define BGP_ATTR_LARGE_COMMUNITIES		32 /* rfc8092 */
+#define BGP_ATTR_PREFIX_SID			40
+
+/* BGP4 internal bitmap type codes.  */
+#define BGP_BMAP_ATTR_MULTI_EXIT_DISC		0x01
+#define BGP_BMAP_ATTR_LOCAL_PREF		0x02
+#define BGP_BMAP_ATTR_AIGP			0x04
 
 #define BGP_NLRI_UNDEFINED			0
 #define BGP_NLRI_UPDATE				1
@@ -81,8 +89,8 @@
 #define MAX_NH_SELF_REFERENCES	1
 #define BGP_XCONNECT_STRLEN	(2 * (INET6_ADDRSTRLEN + PORT_STRLEN + 1) + 4) 
 
-/* Maximum BGP community patterns supported: nfacctd_bgp_stdcomm_pattern,
-   nfacctd_bgp_extcomm_pattern, bgp_blackhole_stdcomm_list, etc. */
+/* Maximum BGP community patterns supported: bgp_daemon_stdcomm_pattern,
+   bgp_daemon_extcomm_pattern, bgp_blackhole_stdcomm_list, etc. */
 #define MAX_BGP_COMM_PATTERNS	16
 #define MAX_BGP_COMM_ELEMS	MAX_BGP_COMM_PATTERNS
 
@@ -104,11 +112,14 @@
 #define BGP_ORIGIN_MAX		2
 #define BGP_ORIGIN_UNKNOWN	3
 
+#define BGP_PREFIX_SID_LI_TLV		1
+#define BGP_PREFIX_SID_OSRGB_TLV	2
+
 /* structures */
 struct bgp_dump_event {
   struct timeval tstamp;
   char tstamp_str[SRVBUFLEN];
-  int period;
+  u_int32_t period;
 };
 
 struct bgp_rt_structs {
@@ -158,8 +169,12 @@ struct bgp_peer_stats {
 
 struct bgp_peer_buf {
   char *base;
-  u_int32_t len;
-  u_int32_t truncated_len;
+  u_int32_t tot_len; /* total buffer length */
+  u_int32_t cur_len; /* current message consumed length (for segmented reads) */
+  u_int32_t exp_len; /* current message expected length */
+#if defined WITH_KAFKA
+  void *kafka_msg;
+#endif
 };
 
 struct bgp_peer {
@@ -179,7 +194,7 @@ struct bgp_peer {
   u_int16_t tcp_port;
   u_int8_t cap_mp;
   char *cap_4as;
-  u_int8_t cap_add_paths;
+  u_int8_t cap_add_paths[AFI_MAX][SAFI_MAX];
   u_int32_t msglen;
   struct bgp_peer_stats stats;
   struct bgp_peer_buf buf;
@@ -192,7 +207,9 @@ struct bgp_peer {
   void *bmp_se;
 
   struct bgp_xconnect xc;
+  struct bgp_peer_buf xbuf;
   int xconnect_fd;
+  int parsed_proxy_header;
 };
 
 struct bgp_msg_data {
@@ -212,6 +229,7 @@ struct bgp_misc_structs {
   char *peer_port_str; /* "bmp_router_port", "peer_src_ip_port", etc. */
   char *log_str; /* BGP, BMP, thread, daemon, etc. */
   int is_thread;
+  int is_readonly; /* disables locking! set to true only in a dump child process */
   int has_lglass;
   int has_blackhole;
   int skip_rib;
@@ -227,6 +245,7 @@ struct bgp_misc_structs {
   int max_peers;
   void *peers_cache;
   void *peers_port_cache;
+  struct log_notification *peers_limit_log;
   void *xconnects;
 
   char *neighbors_file;
@@ -235,6 +254,7 @@ struct bgp_misc_structs {
   int dump_amqp_routing_key_rr;
   char *dump_kafka_topic;
   int dump_kafka_topic_rr;
+  char *dump_kafka_partition_key;
 #if defined WITH_AVRO
   avro_schema_t dump_avro_schema[MAX_AVRO_SCHEMA];
 #endif
@@ -245,16 +265,17 @@ struct bgp_misc_structs {
   int msglog_amqp_routing_key_rr;
   char *msglog_kafka_topic;
   int msglog_kafka_topic_rr;
+  char *msglog_kafka_partition_key;
 #if defined WITH_AVRO
   avro_schema_t msglog_avro_schema[MAX_AVRO_SCHEMA];
 #endif
   char *msglog_kafka_avro_schema_registry;
   char *avro_buf;
-  void (*bgp_peer_log_msg_extras)(struct bgp_peer *, int, void *);
+  void (*bgp_peer_log_msg_extras)(struct bgp_peer *, int, int, int, void *);
   void (*bgp_peer_logdump_initclose_extras)(struct bgp_peer *, int, void *);
 
   void (*bgp_peer_logdump_extra_data)(struct bgp_msg_extra_data *, int, void *);
-  int (*bgp_extra_data_process)(struct bgp_msg_extra_data *, struct bgp_info *);
+  int (*bgp_extra_data_process)(struct bgp_msg_extra_data *, struct bgp_info *, int, int);
   int (*bgp_extra_data_cmp)(struct bgp_msg_extra_data *, struct bgp_msg_extra_data *);
   void (*bgp_extra_data_free)(struct bgp_msg_extra_data *);
 
@@ -309,6 +330,7 @@ struct bgp_attr {
   u_int32_t med;
   u_int32_t local_pref;
   u_int8_t origin;
+  u_int8_t bitmap;
 };
 
 struct bgp_comm_range {
@@ -356,11 +378,12 @@ struct bgp_lg_rep_gp_data {
 #include "bgp_util.h"
 
 /* prototypes */
-extern void nfacctd_bgp_wrapper();
-extern void skinny_bgp_daemon();
+extern void bgp_daemon_wrapper();
+extern int skinny_bgp_daemon();
 extern void skinny_bgp_daemon_online();
 extern void bgp_prepare_thread();
 extern void bgp_prepare_daemon();
+extern void bgp_daemon_msglog_prepare_sd_schemas();
 
 /* global variables */
 extern struct bgp_peer *peers;
