@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -71,22 +71,15 @@ struct ecommunity_val
 
 #define ecom_length(X)    ((X)->size * ECOMMUNITY_SIZE)
 
-#if (!defined __BGP_ECOMMUNITY_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT void ecommunity_init (int, struct hash **);
-EXT void ecommunity_free (struct ecommunity *);
-EXT struct ecommunity *ecommunity_new (struct bgp_peer *);
-EXT struct ecommunity *ecommunity_parse (struct bgp_peer *, u_int8_t *, u_short);
-EXT struct ecommunity *ecommunity_intern (struct bgp_peer *, struct ecommunity *);
-EXT int ecommunity_cmp (const void *, const void *);
-EXT void ecommunity_unintern (struct bgp_peer *, struct ecommunity *);
-EXT unsigned int ecommunity_hash_make (void *);
-EXT char *ecommunity_ecom2str (struct bgp_peer *, struct ecommunity *, int);
-EXT int ecommunity_match (const struct ecommunity *, const struct ecommunity *);
-EXT char *ecommunity_str (struct bgp_peer *, struct ecommunity *);
+extern void ecommunity_init (int, struct hash **);
+extern void ecommunity_free (struct ecommunity *);
+extern struct ecommunity *ecommunity_new (struct bgp_peer *);
+extern struct ecommunity *ecommunity_parse (struct bgp_peer *, u_int8_t *, u_short);
+extern struct ecommunity *ecommunity_intern (struct bgp_peer *, struct ecommunity *);
+extern int ecommunity_cmp (const void *, const void *);
+extern void ecommunity_unintern (struct bgp_peer *, struct ecommunity *);
+extern unsigned int ecommunity_hash_make (void *);
+extern char *ecommunity_ecom2str (struct bgp_peer *, struct ecommunity *, int);
+extern struct ecommunity *ecommunity_dup(struct ecommunity *);
 
-#undef EXT
 #endif
