@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -41,7 +41,7 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
   struct node_match_cmp_term2 nmct2;
   struct prefix default_prefix;
   int compare_bgp_port = config.tmp_bgp_lookup_compare_ports;
-  int follow_default = config.nfacctd_bgp_follow_default;
+  int follow_default = config.bgp_daemon_follow_default;
   struct in_addr pref4;
   struct in6_addr pref6;
   safi_t safi;
@@ -92,7 +92,8 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
     struct host_addr peer_dst_ip;
 
     memset(&peer_dst_ip, 0, sizeof(peer_dst_ip));
-    if (peer->cap_add_paths && (config.acct_type == ACCT_NF || config.acct_type == ACCT_SF)) {
+    if ((peer->cap_add_paths[AFI_IP][SAFI_UNICAST] || peer->cap_add_paths[AFI_IP6][SAFI_UNICAST]) &&
+	(config.acct_type == ACCT_NF || config.acct_type == ACCT_SF)) {
       /* administrativia */
       struct pkt_bgp_primitives pbgp, *pbgp_ptr = &pbgp;
       memset(&pbgp, 0, sizeof(struct pkt_bgp_primitives));
@@ -108,15 +109,22 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
     if (pptrs->bitr) {
       safi = SAFI_MPLS_VPN;
       memcpy(&rd, &pptrs->bitr, sizeof(rd));
+
+      if (type <= RD_ORIGIN_FUNC_TYPE_MAX) {
+	bgp_rd_origin_set(&rd, lookup_type_to_bgp_rd_origin[type]);
+      }
     }
 
     /* XXX: can be further optimized for the case of no SAFI_UNICAST rib */
+    start_again_mpls_vpn:
     start_again_mpls_label:
 
     if (pptrs->l3_proto == ETHERTYPE_IP) {
       if (!pptrs->bgp_src) {
 	memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
 	nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.afi = AFI_IP;
+	nmct2.safi = safi;
 	nmct2.rd = &rd;
 	nmct2.peer_dst_ip = NULL;
 
@@ -144,6 +152,7 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       if (!pptrs->bgp_dst) {
         memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
         nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.afi = AFI_IP;
 	nmct2.safi = safi;
         nmct2.rd = &rd;
         nmct2.peer_dst_ip = &peer_dst_ip;
@@ -173,6 +182,7 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       if (!pptrs->bgp_src) {
         memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
         nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.afi = AFI_IP6;
 	nmct2.safi = safi;
         nmct2.rd = &rd;
         nmct2.peer_dst_ip = NULL;
@@ -201,6 +211,8 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       if (!pptrs->bgp_dst) {
         memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
         nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.afi = AFI_IP6;
+	nmct2.safi = safi;
         nmct2.rd = &rd;
         nmct2.peer_dst_ip = &peer_dst_ip;
 
@@ -226,6 +238,13 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       }
     }
 
+    /* XXX: tackle the case of Peer Distinguisher from BMP header
+       probably needs deeper thoughts for a cleaner approach */
+    if ((!pptrs->bgp_src || !pptrs->bgp_dst) && safi == SAFI_MPLS_VPN && type == FUNC_TYPE_BMP) {
+      safi = SAFI_UNICAST;
+      goto start_again_mpls_vpn;
+    }
+
     if ((!pptrs->bgp_src || !pptrs->bgp_dst) && safi != SAFI_MPLS_LABEL) {
       if (pptrs->l3_proto == ETHERTYPE_IP && inter_domain_routing_db->rib[AFI_IP][SAFI_MPLS_LABEL]) {
         safi = SAFI_MPLS_LABEL;
@@ -309,7 +328,7 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
       }
     }
 
-    if (config.nfacctd_bgp_follow_nexthop[0].family && pptrs->bgp_dst && safi != SAFI_MPLS_VPN)
+    if (config.bgp_daemon_follow_nexthop[0].family && pptrs->bgp_dst && safi != SAFI_MPLS_VPN)
       bgp_follow_nexthop_lookup(pptrs, type);
   }
 }
@@ -340,7 +359,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 
   start_again:
 
-  if (config.nfacctd_bgp_to_agent_map && (*find_id_func)) {
+  if (config.bgp_daemon_to_xflow_agent_map && (*find_id_func)) {
     bta = 0;
     (*find_id_func)((struct id_table *)pptrs->bta_table, pptrs, &bta, NULL);
     if (bta) {
@@ -417,8 +436,8 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	nh.prefixlen = 32;
 	memcpy(&nh.u.prefix4, &info->attr->mp_nexthop.address.ipv4, 4);
 
-	for (nh_idx = 0; config.nfacctd_bgp_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
-	  matched = prefix_match(&config.nfacctd_bgp_follow_nexthop[nh_idx], &nh);
+	for (nh_idx = 0; config.bgp_daemon_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
+	  matched = prefix_match(&config.bgp_daemon_follow_nexthop[nh_idx], &nh);
 	  if (matched) break;
 	}
 
@@ -434,7 +453,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
           goto start_again;
         }
 	else {
-	  if (config.nfacctd_bgp_follow_nexthop_external) saved_info = (char *) info;
+	  if (config.bgp_daemon_follow_nexthop_external) saved_info = (char *) info;
 	  goto end;
 	}
       }
@@ -443,8 +462,8 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	nh.prefixlen = 128;
 	memcpy(&nh.u.prefix6, &info->attr->mp_nexthop.address.ipv6, 16);
 
-        for (nh_idx = 0; config.nfacctd_bgp_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
-          matched = prefix_match(&config.nfacctd_bgp_follow_nexthop[nh_idx], &nh);
+        for (nh_idx = 0; config.bgp_daemon_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
+          matched = prefix_match(&config.bgp_daemon_follow_nexthop[nh_idx], &nh);
           if (matched) break;
         }
 
@@ -460,7 +479,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
           goto start_again;
 	}
 	else {
-	  if (config.nfacctd_bgp_follow_nexthop_external) saved_info = (char *) info;
+	  if (config.bgp_daemon_follow_nexthop_external) saved_info = (char *) info;
 	  goto end;
 	}
       }
@@ -469,8 +488,8 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	nh.prefixlen = 32;
 	memcpy(&nh.u.prefix4, &info->attr->nexthop, 4);
 
-        for (nh_idx = 0; config.nfacctd_bgp_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
-          matched = prefix_match(&config.nfacctd_bgp_follow_nexthop[nh_idx], &nh);
+        for (nh_idx = 0; config.bgp_daemon_follow_nexthop[nh_idx].family && nh_idx < FOLLOW_BGP_NH_ENTRIES; nh_idx++) {
+          matched = prefix_match(&config.bgp_daemon_follow_nexthop[nh_idx], &nh);
           if (matched) break;
         }
 
@@ -486,7 +505,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
           goto start_again;
 	}
 	else {
-	  if (config.nfacctd_bgp_follow_nexthop_external) saved_info = (char *) info;
+	  if (config.bgp_daemon_follow_nexthop_external) saved_info = (char *) info;
 	  goto end;
 	}
       }
@@ -530,7 +549,7 @@ struct bgp_peer *bgp_lookup_find_bgp_peer(struct sockaddr *sa, struct xflow_stat
     }
   }
   else {
-    for (peer = NULL, peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+    for (peer = NULL, peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
       if ((!config.bgp_disable_router_id_check && !sa_addr_cmp(sa, &peers[peers_idx].id)) ||
 	  (!sa_addr_cmp(sa, &peers[peers_idx].addr) && 
 	  (!compare_bgp_port || !sa_port_cmp(sa, peers[peer_idx].tcp_port)))) {
@@ -551,21 +570,22 @@ int bgp_lookup_node_match_cmp_bgp(struct bgp_info *info, struct node_match_cmp_t
   if (info->peer == nmct2->peer) {
     if (nmct2->safi == SAFI_MPLS_VPN) no_match++;
 
-    if (nmct2->peer->cap_add_paths && info->extra && info->extra->path_id &&
-	info->attr && nmct2->peer_dst_ip) no_match++;
+    if (nmct2->peer->cap_add_paths[nmct2->afi][nmct2->safi] && nmct2->peer_dst_ip) no_match++;
 
     if (nmct2->safi == SAFI_MPLS_VPN) {
-      if (info->extra && !memcmp(&info->extra->rd, nmct2->rd, sizeof(rd_t))) no_match--;
+      if (info->attr_extra && !memcmp(&info->attr_extra->rd, nmct2->rd, sizeof(rd_t))) no_match--;
     }
 
-    if (nmct2->peer->cap_add_paths) {
-      if (info->extra && info->extra->path_id) {
-        if (nmct2->peer_dst_ip && info->attr) {
-	  if (info->attr->mp_nexthop.family == nmct2->peer_dst_ip->family) {
-	    if (!memcmp(&info->attr->mp_nexthop, nmct2->peer_dst_ip, HostAddrSz)) no_match--;
+    if (nmct2->peer->cap_add_paths[nmct2->afi][nmct2->safi]) {
+      if (nmct2->peer_dst_ip && info->attr) {
+	if (info->attr->mp_nexthop.family) {
+	  if (!host_addr_cmp(&info->attr->mp_nexthop, nmct2->peer_dst_ip)) {
+	    no_match--;
 	  }
-	  else if (info->attr->nexthop.s_addr && nmct2->peer_dst_ip->family == AF_INET) {
-	    if (info->attr->nexthop.s_addr == nmct2->peer_dst_ip->address.ipv4.s_addr) no_match--;
+	}
+	else if (info->attr->nexthop.s_addr && nmct2->peer_dst_ip->family == AF_INET) {
+	  if (info->attr->nexthop.s_addr == nmct2->peer_dst_ip->address.ipv4.s_addr) {
+	    no_match--;
 	  }
 	}
       }
@@ -599,6 +619,7 @@ int bgp_lookup_node_vector_unicast(struct prefix *p, struct bgp_peer *peer, stru
 
   memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
   nmct2.peer = peer;
+  nmct2.afi = afi;
   nmct2.safi = safi;
   nmct2.p = p;
 
@@ -829,6 +850,7 @@ int bgp_lg_daemon_ip_lookup(struct bgp_lg_req_ipl_data *req, struct bgp_lg_rep *
 
     memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
     nmct2.peer = peer;
+    nmct2.afi = AFI_IP;
     nmct2.safi = safi;
     nmct2.rd = &req->rd;
 
