@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -29,7 +29,7 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
 {
   struct bgp_misc_structs *bms;
   struct bgp_msg_data bmd;
-  char tmp_packet[BGP_BUFFER_SIZE], *bgp_packet_ptr;
+  char *bgp_packet_ptr;
   char bgp_peer_str[INET6_ADDRSTRLEN];
   struct bgp_header *bhdr;
   int ret, bgp_len = 0;
@@ -53,15 +53,7 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
       return BGP_NOTIFY_HEADER_ERR;
     }
 
-    /* BGP buffer segmentation + reassembly */
-    if (peer->msglen < BGP_HEADER_SIZE || peer->msglen < (bgp_len = ntohs(bhdr->bgpo_len))) {
-      memcpy(tmp_packet, bgp_packet_ptr, peer->msglen);
-      memcpy(peer->buf.base, tmp_packet, peer->msglen);
-      peer->buf.truncated_len = peer->msglen;
-
-      break;
-    }
-    else peer->buf.truncated_len = 0;
+    bgp_len = ntohs(bhdr->bgpo_len);
 
     if (bgp_max_msglen_check(bgp_len) == ERR) {
       bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
@@ -135,6 +127,9 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
 	return BGP_NOTIFY_UPDATE_ERR;
       }
 
+      break;
+    case BGP_ROUTE_REFRESH:
+      /* just ignore */
       break;
     default:
       bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
@@ -191,6 +186,14 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 	u_int8_t len, opt_type, opt_len;
 	char *ptr;
 
+	/* pre-flight check */
+	if (ntohs(bopen->bgpo_len) < (BGP_MIN_OPEN_MSG_SIZE + bopen->bgpo_optlen)) {
+          bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (too long option length).\n",
+	      config.name, bms->log_str, bgp_peer_str);
+	  return ERR;
+	}
+
 	ptr = bgp_packet_ptr + BGP_MIN_OPEN_MSG_SIZE;
 	if (online) memset(bgp_open_cap_reply, 0, sizeof(bgp_open_cap_reply));
 
@@ -200,14 +203,14 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 
 	  if (opt_len > len) {
             bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (option length).\n",
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (option length overrun).\n",
 		config.name, bms->log_str, bgp_peer_str);
 	    return ERR;
 	  } 
 
 	  /* 
- 	   * If we stumble upon capabilities let's curse through them to find
- 	   * some we are forced to support (ie. MP-BGP or 4-bytes AS support)
+	   * If we stumble upon capabilities let's iterate through them to find
+	   * those that we do support (ie. MP-BGP, 4-bytes AS support, etc.)
  	   */
 	  if (opt_type == BGP_OPTION_CAPABILITY) {
 	    char *optcap_ptr, *bgp_open_cap_len_reply_ptr, *bgp_open_cap_base_reply_ptr;
@@ -230,7 +233,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 
 	      if (cap_len > optcap_len) {
                 bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (malformed capability: %x).\n",
+		Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (malformed capability: %x).\n",
 			config.name, bms->log_str, bgp_peer_str, cap_type);
 		return ERR;
    	      }
@@ -279,7 +282,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		}
 		else {
                   bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (malformed AS4 option).\n",
+		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (malformed AS4 option).\n",
 			config.name, bms->log_str, bgp_peer_str);
 		  return ERR;
 		}
@@ -302,7 +305,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 			cap_data.sndrcv);
 		  }
 		  if ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)) {
-		    peer->cap_add_paths = TRUE; 
+		    peer->cap_add_paths[ntohs(cap_data.afi)][cap_data.safi] = TRUE; 
 		    if (online) {
 		      if (!cap_set) {
 			/* we need to send BGP_CAPABILITY_ADD_PATHS first */
@@ -320,6 +323,16 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		  }
 	        }
 	      }
+	      else if (cap_type == BGP_CAPABILITY_ROUTE_REFRESH) {
+		if (config.tmp_bgp_daemon_route_refresh) {
+		  bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: Route Refresh [%x]\n",
+		      config.name, bms->log_str, bgp_peer_str, cap_type);
+
+		  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2); 
+		  bgp_open_cap_reply_ptr += cap_len+2;
+		}
+	      }
 
 	      optcap_ptr += (cap_len + 2);
 	      optcap_len -= (cap_len + 2);
@@ -349,7 +362,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
  	   present an ASN == 0 or ASN == 23456 in the 4AS capability */
 	else {
           bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (invalid AS4 option).\n",
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (invalid AS4 option).\n",
 		config.name, bms->log_str, bgp_peer_str);
 	  return ERR;
 	}
@@ -361,7 +374,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 	   present an ASN != remote_as in the 4AS capability */
 	else {
           bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (mismatching AS4 option).\n",
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (mismatching AS4 option).\n",
 		config.name, bms->log_str, bgp_peer_str);
 	  return ERR;
 	}
@@ -371,8 +384,8 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
         bgp_reply_pkt_ptr = bgp_reply_pkt;
 
         /* Replying to OPEN message */
-	if (!config.nfacctd_bgp_as) peer->myas = peer->as;
-	else peer->myas = config.nfacctd_bgp_as;
+	if (!config.bgp_daemon_as) peer->myas = peer->as;
+	else peer->myas = config.bgp_daemon_as;
 
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP_OPEN: Local AS: %u Remote AS: %u HoldTime: %u\n", config.name,
@@ -395,7 +408,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
     }
     else {
       bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-      Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (unsupported version).\n",
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP Open packet (unsupported version).\n",
 		config.name, bms->log_str, bgp_peer_str);
       return ERR;
     }
@@ -473,24 +486,24 @@ int bgp_write_open_msg(char *msg, char *cp_msg, int cp_msglen, struct bgp_peer *
   if (cp_msglen) bopen_reply->bgpo_optlen = cp_msglen;
   bopen_reply->bgpo_len = htons(BGP_MIN_OPEN_MSG_SIZE + bopen_reply->bgpo_optlen);
 
-  if (config.nfacctd_bgp_ip) str_to_addr(config.nfacctd_bgp_ip, &bgp_ip);
+  if (config.bgp_daemon_ip) str_to_addr(config.bgp_daemon_ip, &bgp_ip);
   else memset(&bgp_ip, 0, sizeof(bgp_ip));
 
-  if (config.nfacctd_bgp_id) str_to_addr(config.nfacctd_bgp_id, &bgp_id);
+  if (config.bgp_daemon_id) str_to_addr(config.bgp_daemon_id, &bgp_id);
   else memset(&bgp_id, 0, sizeof(bgp_id));
 
   /* set BGP router-ID trial #1 */
   memset(&my_id_addr, 0, sizeof(my_id_addr));
 
-  if (config.nfacctd_bgp_id && !is_any(&bgp_id) && !my_id_addr.family) {
-    my_id = config.nfacctd_bgp_id;
+  if (config.bgp_daemon_id && !is_any(&bgp_id) && !my_id_addr.family) {
+    my_id = config.bgp_daemon_id;
     str_to_addr(my_id, &my_id_addr);
     if (my_id_addr.family != AF_INET) memset(&my_id_addr, 0, sizeof(my_id_addr));
   }
 
   /* set BGP router-ID trial #2 */
-  if (config.nfacctd_bgp_ip && !is_any(&bgp_ip) && !my_id_addr.family) {
-    my_id = config.nfacctd_bgp_ip;
+  if (config.bgp_daemon_ip && !is_any(&bgp_ip) && !my_id_addr.family) {
+    my_id = config.bgp_daemon_ip;
     str_to_addr(my_id, &my_id_addr);
     if (my_id_addr.family != AF_INET) memset(&my_id_addr, 0, sizeof(my_id_addr));
   }
@@ -530,7 +543,7 @@ int bgp_write_notification_msg(char *msg, int msglen, u_int8_t n_major, u_int8_t
 
     ret += BGP_MIN_NOTIFICATION_MSG_SIZE;
 
-    /* rfc8203 (draft-ietf-idr-shutdown) */
+    /* rfc8203 */
     if (shutdown_msg) {
       shutdown_msglen = strlen(shutdown_msg);
 
@@ -569,7 +582,7 @@ int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *re
   (*res_maj) = bn->bgpn_major;
   (*res_min) = bn->bgpn_minor;
 
-  /* rfc8203 (draft-ietf-idr-shutdown) */
+  /* rfc8203 */
   if (bn->bgpn_major == BGP_NOTIFY_CEASE &&
       (bn->bgpn_minor == BGP_NOTIFY_CEASE_ADMIN_SHUTDOWN || bn->bgpn_minor == BGP_NOTIFY_CEASE_ADMIN_RESET)) {
     if (rem_len) {
@@ -591,9 +604,12 @@ int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *re
 
 int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
 {
+  struct bgp_misc_structs *bms;
   struct bgp_peer *peer = bmd->peer;
+  char bgp_peer_str[INET6_ADDRSTRLEN];
   struct bgp_header bhdr;
   struct bgp_attr attr;
+  struct bgp_attr_extra attr_extra;
   u_int16_t attribute_len;
   u_int16_t update_len;
   u_int16_t withdraw_len;
@@ -602,12 +618,17 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   struct bgp_nlri withdraw;
   struct bgp_nlri mp_update;
   struct bgp_nlri mp_withdraw;
-  int ret;
+  int ret, parsed = FALSE;
 
   if (!peer || !pkt) return ERR;
 
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return ERR;
+
   /* Set initial values. */
   memset(&attr, 0, sizeof (struct bgp_attr));
+  memset(&attr_extra, 0, sizeof (struct bgp_attr_extra));
   memset(&update, 0, sizeof (struct bgp_nlri));
   memset(&withdraw, 0, sizeof (struct bgp_nlri));
   memset(&mp_update, 0, sizeof (struct bgp_nlri));
@@ -645,7 +666,7 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   }
 
   if (attribute_len > 0) {
-    ret = bgp_attr_parse(peer, &attr, pkt, attribute_len, &mp_update, &mp_withdraw);
+    ret = bgp_attr_parse(peer, &attr, &attr_extra, pkt, attribute_len, &mp_update, &mp_withdraw);
     if (ret < 0) return ret;
     pkt += attribute_len;
   }
@@ -660,36 +681,73 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   }
 
   /* NLRI parsing */
-  if (withdraw.length) bgp_nlri_parse(bmd, NULL, &withdraw, BGP_NLRI_WITHDRAW);
-  if (update.length)  bgp_nlri_parse(bmd, &attr, &update, BGP_NLRI_UPDATE);
+  if (withdraw.length) {
+    bgp_nlri_parse(bmd, NULL, &attr_extra, &withdraw, BGP_NLRI_WITHDRAW);
+    parsed = TRUE;
+  }
+
+  if (update.length) {
+    bgp_nlri_parse(bmd, &attr, &attr_extra, &update, BGP_NLRI_UPDATE);
+    parsed = TRUE;
+  }
 	
   if (mp_update.length
 	  && mp_update.afi == AFI_IP
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
-	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
+	      mp_update.safi == SAFI_MPLS_VPN)) {
+    bgp_nlri_parse(bmd, &attr, &attr_extra, &mp_update, BGP_NLRI_UPDATE);
+    parsed = TRUE;
+  }
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
-	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse (bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
+	      mp_withdraw.safi == SAFI_MPLS_VPN)) {
+    bgp_nlri_parse (bmd, NULL, &attr_extra, &mp_withdraw, BGP_NLRI_WITHDRAW);
+    parsed = TRUE;
+  }
 
   if (mp_update.length
 	  && mp_update.afi == AFI_IP6
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
-	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
+	      mp_update.safi == SAFI_MPLS_VPN)) {
+    bgp_nlri_parse(bmd, &attr, &attr_extra, &mp_update, BGP_NLRI_UPDATE);
+    parsed = TRUE;
+  }
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP6
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
-	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
+	      mp_withdraw.safi == SAFI_MPLS_VPN)) {
+    bgp_nlri_parse(bmd, NULL, &attr_extra, &mp_withdraw, BGP_NLRI_WITHDRAW);
+    parsed = TRUE;
+  }
+
+  if (mp_update.length
+	  && mp_update.afi == AFI_BGP_LS
+	  && (mp_update.safi == SAFI_LS_GLOBAL || mp_update.safi == SAFI_LS_VPN)) {
+    bgp_nlri_parse(bmd, &attr, &attr_extra, &mp_update, BGP_NLRI_UPDATE);
+    parsed = TRUE;
+  }
+
+  if (mp_withdraw.length
+	  && mp_withdraw.afi == AFI_BGP_LS
+	  && (mp_withdraw.safi == SAFI_LS_GLOBAL || mp_withdraw.safi == SAFI_LS_VPN)) {
+    bgp_nlri_parse(bmd, &attr, &attr_extra, &mp_update, BGP_NLRI_WITHDRAW);
+    parsed = TRUE;
+  }
 
   /* Receipt of End-of-RIB can be processed here; being a silent
 	 BGP receiver only, honestly it doesn't matter to us */
 
+  if ((update_len || withdraw_len ||  mp_update.length || mp_withdraw.length) && !parsed) {
+    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+    Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] bgp_parse_update_msg() Received unsupported NLRI afi=%u safi=%u\n",
+        config.name, bms->log_str, bgp_peer_str,
+	mp_update.length ? mp_update.afi : mp_withdraw.afi,
+	mp_update.length ? mp_update.safi : mp_withdraw.safi);
+  }
+
   /* Everything is done.  We unintern temporary structures which
 	 interned in bgp_attr_parse(). */
   if (attr.aspath)
@@ -702,11 +760,13 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
     lcommunity_unintern(peer, attr.lcommunity);
 
   ret = ntohs(bhdr.bgpo_len);
+
   return ret;
 }
 
 /* BGP UPDATE Attribute parsing */
-int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int len, struct bgp_nlri *mp_update, struct bgp_nlri *mp_withdraw)
+int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, struct bgp_attr_extra *attr_extra,
+		   char *ptr, int len, struct bgp_nlri *mp_update, struct bgp_nlri *mp_withdraw)
 {
   int to_the_end = len, ret;
   u_int8_t flag, type, *tmp;
@@ -765,6 +825,12 @@ int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int
     case BGP_ATTR_MP_UNREACH_NLRI:
       ret = bgp_attr_parse_mp_unreach(peer, attr_len, attr, ptr, mp_withdraw);
       break;
+    case BGP_ATTR_AIGP:
+      ret = bgp_attr_parse_aigp(peer, attr_len, attr_extra, ptr, flag);
+      break;
+    case BGP_ATTR_PREFIX_SID:
+      ret = bgp_attr_parse_prefix_sid(peer, attr_len, attr_extra, ptr, flag);
+      break;
     default:
       ret = 0;
       break;
@@ -854,6 +920,7 @@ int bgp_attr_parse_med(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *at
 
   memcpy(&tmp, ptr, 4);
   attr->med = ntohl(tmp);
+  attr->bitmap |= BGP_BMAP_ATTR_MULTI_EXIT_DISC;
   ptr += 4;
 
   return SUCCESS;
@@ -868,6 +935,7 @@ int bgp_attr_parse_local_pref(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 
   memcpy(&tmp, ptr, 4);
   attr->local_pref = ntohl(tmp);
+  attr->bitmap |= BGP_BMAP_ATTR_LOCAL_PREF;
   ptr += 4;
 
   return SUCCESS;
@@ -878,7 +946,7 @@ int bgp_attr_parse_origin(struct bgp_peer *peer, u_int16_t len, struct bgp_attr
 {
   if (len != 1) return ERR;
 
-  memcpy(&attr->local_pref, ptr, 1);
+  memcpy(&attr->origin, ptr, 1);
   ptr += 1;
 
   return SUCCESS;
@@ -981,38 +1049,39 @@ int bgp_attr_parse_mp_unreach(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 }
 
 /* BGP UPDATE NLRI parsing */
-int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info, int type)
+int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_attr_extra *attr_extra, struct bgp_nlri *info, int type)
 {
+  struct bgp_misc_structs *bms;
   struct bgp_peer *peer = bmd->peer;
+  char bgp_peer_str[INET6_ADDRSTRLEN];
   u_char *pnt;
   u_char *lim;
-  u_char safi, label[3];
   struct prefix p;
   int psize = 0, end;
-  int ret;
+  int ret, idx;
   u_int32_t tmp32;
   u_int16_t tmp16;
   struct rd_ip  *rdi;
   struct rd_as  *rda;
   struct rd_as4 *rda4;
-  rd_t rd;
-  path_id_t path_id;
 
+  if (!peer) return ERR;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return ERR;
+  
   memset(&p, 0, sizeof(struct prefix));
-  memset(&rd, 0, sizeof(rd_t));
-  memset(&path_id, 0, sizeof(path_id_t));
 
   pnt = info->nlri;
   lim = pnt + info->length;
   end = info->length;
-  safi = info->safi;
-
-  for (; pnt < lim; pnt += psize) {
 
+  for (idx = 0; pnt < lim; pnt += psize, idx++) {
     /* handle path identifier */
-    if (peer->cap_add_paths) {
-      memcpy(&path_id, pnt, 4);
-      path_id = ntohl(path_id);
+    if (peer->cap_add_paths[info->afi][info->safi]) {
+      memcpy(&attr_extra->path_id, pnt, 4);
+      attr_extra->path_id = ntohl(attr_extra->path_id);
       pnt += 4;
     }
 
@@ -1022,7 +1091,7 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
     p.prefixlen = *pnt++; end--;
     p.family = bgp_afi2family (info->afi);
 
-    if (info->safi == SAFI_UNICAST) { 
+    if (info->safi == SAFI_UNICAST) {
       if ((info->afi == AFI_IP && p.prefixlen > 32) || (info->afi == AFI_IP6 && p.prefixlen > 128)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
@@ -1051,7 +1120,7 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
       }
 
       if ((labels_size + 3) <= psize) {
-        memcpy(label, label_ptr, 3);
+        memcpy(attr_extra->label, label_ptr, 3);
         label_ptr += 3;
         labels_size += 3;
       }
@@ -1080,7 +1149,7 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
       }
 
       if ((labels_size + 3) <= psize) {
-	memcpy(label, label_ptr, 3);
+	memcpy(attr_extra->label, label_ptr, 3);
 	label_ptr += 3;
 	labels_size += 3;
       }
@@ -1088,24 +1157,24 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
 
       if (labels_size + 8 /* RD */ > psize) return ERR; 
 
-      memcpy(&rd.type, (pnt + labels_size), 2);
-      rd.type = ntohs(rd.type);
-      switch(rd.type) {
+      memcpy(&attr_extra->rd.type, (pnt + labels_size), 2);
+      attr_extra->rd.type = ntohs(attr_extra->rd.type);
+      switch(attr_extra->rd.type) {
       case RD_TYPE_AS: 
-	rda = (struct rd_as *) &rd;
+	rda = (struct rd_as *) &attr_extra->rd;
 	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */), 2);
 	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */ + 2 /* RD AS */), 4);
 	rda->as = ntohs(tmp16);
 	rda->val = ntohl(tmp32);
 	break;
       case RD_TYPE_IP: 
-	rdi = (struct rd_ip *) &rd;
+	rdi = (struct rd_ip *) &attr_extra->rd;
 	memcpy(&rdi->ip.s_addr, (pnt + labels_size + 2 /* RD type */), 4);
 	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD IP */), 2);
 	rdi->val = ntohs(tmp16);
 	break;
       case RD_TYPE_AS4: 
-	rda4 = (struct rd_as4 *) &rd;
+	rda4 = (struct rd_as4 *) &attr_extra->rd;
 	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */), 4);
 	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD AS4 */), 2);
 	rda4->as = ntohl(tmp32);
@@ -1115,10 +1184,17 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
 	return ERR;
 	break;
       }
+      bgp_rd_origin_set(&attr_extra->rd, RD_ORIGIN_BGP);
     
       memcpy(&p.u.prefix, (pnt + labels_size + 8 /* RD */), (psize - (labels_size + 8 /* RD */)));
       p.prefixlen -= (8 * (labels_size + 8 /* RD */));
     }
+    else {
+      bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+      Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] bgp_nlri_parse() Received unsupported NLRI afi=%u safi=%u\n",
+        config.name, bms->log_str, bgp_peer_str, info->afi, info->safi);
+      continue;
+    }
 
     // XXX: check prefix correctnesss now that we have it?
 
@@ -1128,26 +1204,25 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
 
       /* let's process withdraws before withdrawing */  
       if (!attr && bmd->is_blackhole) {
-	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, info->safi);
       }
     }
 #endif
 
     /* Let's do our job now! */
     if (attr) {
-      ret = bgp_process_update(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
+      ret = bgp_process_update(bmd, &p, attr, attr_extra, info->afi, info->safi, idx);
     }
     else {
-      ret = bgp_process_withdraw(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
+      ret = bgp_process_withdraw(bmd, &p, attr, attr_extra, info->afi, info->safi, idx);
       (void)ret; //Treat error?
-
     }
 
 #if defined WITH_ZMQ
     if (config.bgp_blackhole_stdcomm_list) {
       /* let's process updates after installing */  
       if (attr && bmd->is_blackhole) {
-	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, info->safi);
       }
     }
 #endif
@@ -1156,8 +1231,71 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info,
   return SUCCESS;
 }
 
-int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-		       rd_t *rd, path_id_t *path_id, u_char *label)
+/* AIGP attribute. */
+int bgp_attr_parse_aigp(struct bgp_peer *peer, u_int16_t len, struct bgp_attr_extra *attr_extra, char *ptr, u_char flag)
+{
+  u_int64_t tmp64;
+  int ret = SUCCESS;
+
+  /* Length check. */
+  if (len < 3) return ERR;
+
+  /* XXX: skipping type check as only type 1 is defined */
+
+  switch (len) {
+  case 3:
+    attr_extra->aigp = 0; 
+    break;
+  /* rfc7311: [If present] The value field of the AIGP TLV is always 8 octets long */
+  case 11:
+    memcpy(&tmp64, (ptr + 3), 8);
+    attr_extra->aigp = pm_ntohll(tmp64);
+    attr_extra->bitmap |= BGP_BMAP_ATTR_AIGP;
+    break;
+  default:
+    /* unsupported */
+    attr_extra->aigp = 0;
+    ret = ERR;
+    break;
+  }
+
+  ptr += len;
+
+  return ret;
+}
+
+/* Prefix-SID attribute */
+int bgp_attr_parse_prefix_sid(struct bgp_peer *peer, u_int16_t len, struct bgp_attr_extra *attr_extra, char *ptr, u_char flag)
+{
+  u_int8_t tlv_type;
+  u_int16_t tlv_len;
+  u_int32_t tmp;
+
+  /* Length check. */
+  if (len < 3) return ERR;
+
+  tlv_type = (u_int8_t) (*ptr);
+  memcpy(&tlv_len, (ptr + 1), 2);
+  tlv_len = ntohs(tlv_len);
+
+  if (tlv_type == BGP_PREFIX_SID_LI_TLV) {
+    if (tlv_len == 7) {
+      memcpy(&tmp, (ptr + 6), 4);
+      attr_extra->psid_li = ntohl(tmp); 
+    }
+    else {
+      return ERR;
+    }
+  }
+
+  /* XXX: Originator SRGB TLV not decoded yet */
+
+  ptr += len;
+
+  return SUCCESS;
+}
+
+int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, struct bgp_attr_extra *attr_extra, afi_t afi, safi_t safi, int idx)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
@@ -1175,30 +1313,24 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, a
   if (!inter_domain_routing_db || !bms) return ERR;
 
   if (!bms->skip_rib) { 
-    modulo = bms->route_info_modulo(peer, path_id, bms->table_per_peer_buckets);
+    modulo = bms->route_info_modulo(peer, &attr_extra->path_id, bms->table_per_peer_buckets);
     route = bgp_node_get(peer, inter_domain_routing_db->rib[afi][safi], p);
 
     /* Check previously received route. */
     for (ri = route->info[modulo]; ri; ri = ri->next) {
       if (ri->peer == peer) { 
         if (safi == SAFI_MPLS_VPN) {
-	  if (ri->extra && !memcmp(&ri->extra->rd, rd, sizeof(rd_t)));
+	  if (ri->attr_extra && !memcmp(&ri->attr_extra->rd, &attr_extra->rd, sizeof(rd_t)));
 	  else continue;
         }
 
-        if (peer->cap_add_paths) {
-	  if (path_id && *path_id) {
-	    if (ri->extra && *path_id == ri->extra->path_id);
-	    else continue;
-	  }
-	  else {
-	    if (!ri->extra || (ri->extra && !ri->extra->path_id));
-	    else continue;
-	  }
+        if (peer->cap_add_paths[afi][safi]) {
+	  if (ri->attr_extra && (attr_extra->path_id == ri->attr_extra->path_id));
+	  else continue;
         }
 
-	if (ri->extra && ri->extra->bmed.id) {
-	  if (bms->bgp_extra_data_cmp && !(*bms->bgp_extra_data_cmp)(&bmd->extra, &ri->extra->bmed));
+	if (ri->bmed.id) {
+	  if (bms->bgp_extra_data_cmp && !(*bms->bgp_extra_data_cmp)(&bmd->extra, &ri->bmed));
 	  else continue;
 	} 
 
@@ -1223,8 +1355,8 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, a
         /* Update to new attribute.  */
         bgp_attr_unintern(peer, ri->attr);
         ri->attr = attr_new;
-        bgp_info_extra_process(peer, ri, safi, path_id, rd, label);
-        if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri);
+        bgp_attr_extra_process(peer, ri, afi, safi, attr_extra);
+        if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri, idx, BGP_NLRI_UPDATE);
 
         bgp_unlock_node (peer, route);
 
@@ -1240,8 +1372,8 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, a
     if (new) {
       new->peer = peer;
       new->attr = attr_new;
-      bgp_info_extra_process(peer, new, safi, path_id, rd, label);
-      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, new);
+      bgp_attr_extra_process(peer, new, afi, safi, attr_extra);
+      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, new, idx, BGP_NLRI_UPDATE);
     }
     else return ERR;
 
@@ -1267,8 +1399,8 @@ int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, a
 
       ri->peer = peer;
       ri->attr = bgp_attr_intern(peer, attr);
-      bgp_info_extra_process(peer, ri, safi, path_id, rd, label);
-      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri);
+      bgp_attr_extra_process(peer, ri, afi, safi, attr_extra);
+      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri, idx, BGP_NLRI_UPDATE);
 
       goto log_update;
     }
@@ -1284,15 +1416,15 @@ log_update:
   }
 
   if (bms->skip_rib) {
-    if (ri->extra) bgp_info_extra_free(peer, &ri->extra);
+    if (ri->attr_extra) bgp_attr_extra_free(peer, &ri->attr_extra);
+    if (bms->bgp_extra_data_free) (*bms->bgp_extra_data_free)(&ri->bmed);
     bgp_attr_unintern(peer, ri->attr);
   }
 
   return SUCCESS;
 }
 
-int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-			 rd_t *rd, path_id_t *path_id, u_char *label)
+int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr, struct bgp_attr_extra *attr_extra, afi_t afi, safi_t safi, int idx)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
@@ -1309,7 +1441,7 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
   if (!inter_domain_routing_db || !bms) return ERR;
 
   if (!bms->skip_rib) {
-    modulo = bms->route_info_modulo(peer, path_id, bms->table_per_peer_buckets);
+    modulo = bms->route_info_modulo(peer, &attr_extra->path_id, bms->table_per_peer_buckets);
 
     /* Lookup node. */
     route = bgp_node_get(peer, inter_domain_routing_db->rib[afi][safi], p);
@@ -1318,23 +1450,17 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
     for (ri = route->info[modulo]; ri; ri = ri->next) {
       if (ri->peer == peer) {
         if (safi == SAFI_MPLS_VPN) {
-          if (ri->extra && !memcmp(&ri->extra->rd, rd, sizeof(rd_t)));
+          if (ri->attr_extra && !memcmp(&ri->attr_extra->rd, &attr_extra->rd, sizeof(rd_t)));
           else continue;
         }
 
-        if (peer->cap_add_paths) {
-          if (path_id && *path_id) {
-            if (ri->extra && *path_id == ri->extra->path_id);
-            else continue;
-          }
-          else {
-            if (!ri->extra || (ri->extra && !ri->extra->path_id));
-            else continue;
-          }
-        }
+        if (peer->cap_add_paths[afi][safi]) {
+          if (ri->attr_extra && (attr_extra->path_id == ri->attr_extra->path_id));
+          else continue;
+	}
 
-        if (ri->extra && ri->extra->bmed.id) {
-          if (bms->bgp_extra_data_cmp && !(*bms->bgp_extra_data_cmp)(&bmd->extra, &ri->extra->bmed));
+        if (ri->bmed.id) {
+          if (bms->bgp_extra_data_cmp && !(*bms->bgp_extra_data_cmp)(&bmd->extra, &ri->bmed));
           else continue;
         }
 
@@ -1352,8 +1478,8 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
       memset(&ri_local, 0, sizeof(struct bgp_info));
 
       ri->peer = peer;
-      bgp_info_extra_process(peer, ri, safi, path_id, rd, label);
-      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri);
+      bgp_attr_extra_process(peer, ri, afi, safi, attr_extra);
+      if (bms->bgp_extra_data_process) (*bms->bgp_extra_data_process)(&bmd->extra, ri, idx, BGP_NLRI_WITHDRAW);
     }
   }
 
@@ -1372,7 +1498,8 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
   }
   else {
     if (bms->msglog_backend_methods) {
-      if (ri->extra) bgp_info_extra_free(peer, &ri->extra);
+      if (ri->attr_extra) bgp_attr_extra_free(peer, &ri->attr_extra);
+      if (bms->bgp_extra_data_free) (*bms->bgp_extra_data_free)(&ri->bmed);
     }
   }
 
