@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -27,6 +27,9 @@
 #ifndef _BGP_PREFIX_H_
 #define _BGP_PREFIX_H_
 
+#define PREFIX_MASKSTRLEN	4
+#define PREFIX_STRLEN		(INET6_ADDRSTRLEN + PREFIX_MASKSTRLEN)
+
 /*
  * A struct prefix contains an address family, a prefix length, and an
  * address.  This can represent either a 'network prefix' as defined
@@ -45,15 +48,7 @@ struct prefix
   {
     u_char prefix;
     struct in_addr prefix4;
-#ifdef ENABLE_IPV6
     struct in6_addr prefix6;
-#endif /* ENABLE_IPV6 */
-    struct 
-    {
-      struct in_addr id;
-      struct in_addr adv_router;
-    } lp;
-    u_char val[8];
   } u __attribute__ ((aligned (8)));
 };
 
@@ -66,30 +61,12 @@ struct prefix_ipv4
 };
 
 /* IPv6 prefix structure. */
-#ifdef ENABLE_IPV6
 struct prefix_ipv6
 {
   u_char family;
   u_char prefixlen;
   struct in6_addr prefix __attribute__ ((aligned (8)));
 };
-#endif /* ENABLE_IPV6 */
-
-struct prefix_ls
-{
-  u_char family;
-  u_char prefixlen;
-  struct in_addr id __attribute__ ((aligned (8)));
-  struct in_addr adv_router;
-};
-
-/* Prefix for routing distinguisher. */
-struct prefix_rd
-{
-  u_char family;
-  u_char prefixlen;
-  u_char val[8] __attribute__ ((aligned (8)));
-};
 
 #ifndef INET_ADDRSTRLEN
 #define INET_ADDRSTRLEN 16
@@ -130,67 +107,58 @@ struct prefix_rd
 #define PREFIX_FAMILY(p)  ((p)->family)
 
 /* Prototypes. */
-#if (!defined __BGP_PREFIX_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT int afi2family (int);
-EXT int family2afi (int);
+extern int afi2family (int);
+extern int family2afi (int);
 
-EXT struct prefix *prefix_new (void);
-EXT void prefix_free (struct prefix *);
-EXT const char *prefix_family_str (const struct prefix *);
-EXT int prefix_blen (const struct prefix *);
-EXT int str2prefix (const char *, struct prefix *);
-EXT int prefix2str (const struct prefix *, char *, int);
-EXT int prefix_match (const struct prefix *, const struct prefix *);
-EXT int prefix_same (const struct prefix *, const struct prefix *);
-EXT int prefix_cmp (const struct prefix *, const struct prefix *);
-EXT void prefix_copy (struct prefix *dest, const struct prefix *src);
-EXT void apply_mask (struct prefix *);
+extern struct prefix *prefix_new (void);
+extern void prefix_free (struct prefix *);
+extern const char *prefix_family_str (const struct prefix *);
+extern int prefix_blen (const struct prefix *);
+extern int str2prefix (const char *, struct prefix *);
+extern int prefix2str (const struct prefix *, char *, int);
+extern int prefix_match (const struct prefix *, const struct prefix *);
+extern int prefix_same (const struct prefix *, const struct prefix *);
+extern int prefix_cmp (const struct prefix *, const struct prefix *);
+extern void prefix_copy (struct prefix *dest, const struct prefix *src);
+extern void apply_mask (struct prefix *);
 
-EXT struct prefix_ipv4 *prefix_ipv4_new (void);
-EXT void prefix_ipv4_free (struct prefix_ipv4 *);
-EXT int str2prefix_ipv4 (const char *, struct prefix_ipv4 *);
-EXT void apply_mask_ipv4 (struct prefix_ipv4 *);
+extern struct prefix_ipv4 *prefix_ipv4_new (void);
+extern void prefix_ipv4_free (struct prefix_ipv4 *);
+extern int str2prefix_ipv4 (const char *, struct prefix_ipv4 *);
+extern void apply_mask_ipv4 (struct prefix_ipv4 *);
 
 #define PREFIX_COPY_IPV4(DST, SRC)	\
 	*((struct prefix_ipv4 *)(DST)) = *((const struct prefix_ipv4 *)(SRC));
 
-EXT int prefix_ipv4_any (const struct prefix_ipv4 *);
-EXT void apply_classful_mask_ipv4 (struct prefix_ipv4 *);
+extern int prefix_ipv4_any (const struct prefix_ipv4 *);
+extern void apply_classful_mask_ipv4 (struct prefix_ipv4 *);
 
-EXT u_char ip_masklen (struct in_addr);
-EXT void masklen2ip (int, struct in_addr *);
+extern u_char ip_masklen (struct in_addr);
+extern void masklen2ip (int, struct in_addr *);
 /* returns the network portion of the host address */
-EXT in_addr_t ipv4_network_addr (in_addr_t hostaddr, int masklen);
+extern in_addr_t ipv4_network_addr (in_addr_t hostaddr, int masklen);
 /* given the address of a host on a network and the network mask length,
  * calculate the broadcast address for that network;
  * special treatment for /31: returns the address of the other host
  * on the network by flipping the host bit */
-EXT in_addr_t ipv4_broadcast_addr (in_addr_t hostaddr, int masklen);
+extern in_addr_t ipv4_broadcast_addr (in_addr_t hostaddr, int masklen);
 
-EXT int netmask_str2prefix_str (const char *, const char *, char *);
+extern int netmask_str2prefix_str (const char *, const char *, char *);
 
-#ifdef ENABLE_IPV6
-EXT struct prefix_ipv6 *prefix_ipv6_new (void);
-EXT void prefix_ipv6_free (struct prefix_ipv6 *);
-EXT int str2prefix_ipv6 (const char *, struct prefix_ipv6 *);
-EXT void apply_mask_ipv6 (struct prefix_ipv6 *);
+extern struct prefix_ipv6 *prefix_ipv6_new (void);
+extern void prefix_ipv6_free (struct prefix_ipv6 *);
+extern int str2prefix_ipv6 (const char *, struct prefix_ipv6 *);
+extern void apply_mask_ipv6 (struct prefix_ipv6 *);
 
 #define PREFIX_COPY_IPV6(DST, SRC)	\
 	*((struct prefix_ipv6 *)(DST)) = *((const struct prefix_ipv6 *)(SRC));
 
-EXT int ip6_masklen (struct in6_addr);
-EXT void masklen2ip6 (int, struct in6_addr *);
+extern int ip6_masklen (struct in6_addr);
+extern void masklen2ip6 (int, struct in6_addr *);
 
-EXT void str2in6_addr (const char *, struct in6_addr *);
-EXT const char *inet6_ntoa (struct in6_addr);
+extern void str2in6_addr (const char *, struct in6_addr *);
+extern const char *inet6_ntoa (struct in6_addr);
 
-#endif /* ENABLE_IPV6 */
+extern int all_digit (const char *);
 
-EXT int all_digit (const char *);
-
-#undef EXT
 #endif
