@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -31,7 +31,7 @@
 #include "kafka_common.h"
 #endif
 
-u_int32_t bmp_process_packet(char *bmp_packet, u_int32_t len, struct bmp_peer *bmpp)
+u_int32_t bmp_process_packet(char *bmp_packet, u_int32_t len, struct bmp_peer *bmpp, int *do_term)
 {
   struct bgp_misc_structs *bms;
   struct bgp_peer *peer;
@@ -40,6 +40,7 @@ u_int32_t bmp_process_packet(char *bmp_packet, u_int32_t len, struct bmp_peer *b
 
   struct bmp_common_hdr *bch = NULL;
 
+  if (do_term) (*do_term) = FALSE;
   if (!bmpp) return FALSE;
 
   peer = &bmpp->self;
@@ -54,11 +55,12 @@ u_int32_t bmp_process_packet(char *bmp_packet, u_int32_t len, struct bmp_peer *b
   }
 
   for (msg_start_len = pkt_remaining_len = len; pkt_remaining_len; msg_start_len = pkt_remaining_len) {
-    if (!(bch = (struct bmp_common_hdr *) bmp_get_and_check_length(&bmp_packet_ptr, &pkt_remaining_len, sizeof(struct bmp_common_hdr))))
+    if (!(bch = (struct bmp_common_hdr *) bmp_get_and_check_length(&bmp_packet_ptr, &pkt_remaining_len, sizeof(struct bmp_common_hdr)))) { 
       return msg_start_len;
+    }
 
     if (bch->version != BMP_V3 && bch->version != BMP_V4) {
-      Log(LOG_INFO, "INFO ( %s/%s ): [%s] packet discarded: unknown BMP version: %u\n",
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] packet discarded: unknown BMP version: %u (2)\n",
 	  config.name, bms->log_str, peer->addr_str, bch->version);
       return FALSE;
     }
@@ -93,10 +95,14 @@ u_int32_t bmp_process_packet(char *bmp_packet, u_int32_t len, struct bmp_peer *b
       break;
     case BMP_MSG_TERM:
       bmp_process_msg_term(&bmp_packet_ptr, &msg_len, bmpp); 
+      if (do_term) (*do_term) = TRUE;
       break;
     case BMP_MSG_ROUTE_MIRROR:
       bmp_process_msg_route_mirror(&bmp_packet_ptr, &msg_len, bmpp);
       break;
+    case BMP_MSG_TMP_RPAT:
+      bmp_process_msg_rpat(&bmp_packet_ptr, &msg_len, bmpp);
+      break;
     default:
       Log(LOG_INFO, "INFO ( %s/%s ): [%s] packet discarded: unknown message type (%u)\n",
 	  config.name, bms->log_str, peer->addr_str, bch->type);
@@ -120,11 +126,13 @@ void bmp_process_msg_init(char **bmp_packet, u_int32_t *len, struct bmp_peer *bm
   struct bgp_misc_structs *bms;
   struct bgp_peer *peer;
   struct bmp_data bdata;
-  struct bmp_tlv_hdr *bth;
-  u_int16_t bmp_init_type, bmp_init_len;
-  char *bmp_init_info;
+  int ret;
 
-  struct bmp_log_init_array blinit;
+  /* TLV vars */
+  struct bmp_tlv_hdr *bth;
+  u_int16_t bmp_tlv_type, bmp_tlv_len;
+  char *bmp_tlv_value;
+  struct pm_list *tlvs = NULL;
 
   if (!bmpp) return;
 
@@ -134,42 +142,60 @@ void bmp_process_msg_init(char **bmp_packet, u_int32_t *len, struct bmp_peer *bm
   if (!bms) return;
 
   memset(&bdata, 0, sizeof(bdata));
-  blinit.entries = 0;
 
-  gettimeofday(&bdata.tstamp, NULL);
+  tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+  if (!tlvs) return;
+
+  /* Init message does not contain a timestamp */
+  gettimeofday(&bdata.tstamp_arrival, NULL);
+  memset(&bdata.tstamp, 0, sizeof(struct timeval));
 
   while ((*len)) {
+    u_int32_t pen = 0;
+
     if (!(bth = (struct bmp_tlv_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_tlv_hdr)))) {
       Log(LOG_INFO, "INFO ( %s/%s ): [%s] [init] packet discarded: failed bmp_get_and_check_length() BMP TLV hdr\n",
-		config.name, bms->log_str, peer->addr_str);
+	  config.name, bms->log_str, peer->addr_str);
+      bmp_tlv_list_destroy(tlvs);
       return;
     }
 
-    bmp_tlv_hdr_get_type(bth, &bmp_init_type);
-    bmp_tlv_hdr_get_len(bth, &bmp_init_len);
+    bmp_tlv_hdr_get_type(bth, &bmp_tlv_type);
+    bmp_tlv_hdr_get_len(bth, &bmp_tlv_len);
+    if (bmp_tlv_handle_ebit(&bmp_tlv_type)) {
+      if (!(bmp_tlv_get_pen(bmp_packet, len, &bmp_tlv_len, &pen))) {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [init] packet discarded: failed bmp_tlv_get_pen()\n",
+	    config.name, bms->log_str, peer->addr_str);
+	bmp_tlv_list_destroy(tlvs);
+	return;
+      }
+    }
 
-    if (!(bmp_init_info = bmp_get_and_check_length(bmp_packet, len, bmp_init_len))) {
+    if (!(bmp_tlv_value = bmp_get_and_check_length(bmp_packet, len, bmp_tlv_len))) {
       Log(LOG_INFO, "INFO ( %s/%s ): [%s] [init] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
-		config.name, bms->log_str, peer->addr_str);
+	  config.name, bms->log_str, peer->addr_str);
+      bmp_tlv_list_destroy(tlvs);
       return;
     }
 
-    blinit.e[blinit.entries].type = bmp_init_type;
-    blinit.e[blinit.entries].len = bmp_init_len;
-    blinit.e[blinit.entries].val = bmp_init_info;
-
-    blinit.entries = bmp_tlv_array_increment(blinit.entries, BMP_INIT_INFO_ENTRIES);
+    ret = bmp_tlv_list_add(tlvs, pen, bmp_tlv_type, bmp_tlv_len, bmp_tlv_value);
+    if (ret == ERR) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [init] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str); 
+      exit_gracefully(1);
+    }
   }
 
   if (bms->msglog_backend_methods) {
     char event_type[] = "log";
 
-    bmp_log_msg(peer, &bdata, &blinit, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.nfacctd_bmp_msglog_output, BMP_LOG_TYPE_INIT);
+    bmp_log_msg(peer, &bdata, tlvs, NULL, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.bmp_daemon_msglog_output, BMP_LOG_TYPE_INIT);
   }
 
-  if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, &blinit, BMP_LOG_TYPE_INIT);
+  if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, tlvs, NULL, BMP_LOG_TYPE_INIT);
 
   if (bms->msglog_backend_methods || bms->dump_backend_methods) bgp_peer_log_seq_increment(&bms->log_seq);
+
+  if (!pm_listcount(tlvs) || !bms->dump_backend_methods) bmp_tlv_list_destroy(tlvs);
 }
 
 void bmp_process_msg_term(char **bmp_packet, u_int32_t *len, struct bmp_peer *bmpp)
@@ -177,11 +203,13 @@ void bmp_process_msg_term(char **bmp_packet, u_int32_t *len, struct bmp_peer *bm
   struct bgp_misc_structs *bms;
   struct bgp_peer *peer;
   struct bmp_data bdata;
-  struct bmp_tlv_hdr *bth;
-  u_int16_t bmp_term_type, bmp_term_len, reason_type = 0;
-  char *bmp_term_info;
+  int ret = 0;
 
-  struct bmp_log_term_array blterm;
+  /* TLV vars */
+  struct bmp_tlv_hdr *bth;
+  u_int16_t bmp_tlv_type, bmp_tlv_len;
+  char *bmp_tlv_value;
+  struct pm_list *tlvs = NULL;
 
   if (!bmpp) return;
 
@@ -191,51 +219,61 @@ void bmp_process_msg_term(char **bmp_packet, u_int32_t *len, struct bmp_peer *bm
   if (!bms) return;
 
   memset(&bdata, 0, sizeof(bdata));
-  blterm.entries = 0;
 
-  gettimeofday(&bdata.tstamp, NULL);
+  tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+  if (!tlvs) return;
+
+  /* Term message does not contain a timestamp */
+  gettimeofday(&bdata.tstamp_arrival, NULL);
+  memset(&bdata.tstamp, 0, sizeof(struct timeval));
 
   while ((*len)) {
+    u_int32_t pen = 0;
+
     if (!(bth = (struct bmp_tlv_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_tlv_hdr)))) {
       Log(LOG_INFO, "INFO ( %s/%s ): [%s] [term] packet discarded: failed bmp_get_and_check_length() BMP TLV hdr\n",
-		config.name, bms->log_str, peer->addr_str);
-       return;
-    }
-
-    bmp_tlv_hdr_get_type(bth, &bmp_term_type);
-    bmp_tlv_hdr_get_len(bth, &bmp_term_len);
-
-    if (!(bmp_term_info = bmp_get_and_check_length(bmp_packet, len, bmp_term_len))) {
-      Log(LOG_INFO, "INFO ( %s/%s ): [%s] [term] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
-		config.name, bms->log_str, peer->addr_str);
+	  config.name, bms->log_str, peer->addr_str);
+      bmp_tlv_list_destroy(tlvs);
       return;
     }
 
-    if (bmp_term_type == BMP_TERM_INFO_REASON && bmp_term_len == 2) {
-      char *bmp_term_info_loc = bmp_term_info;
-      u_int32_t len_loc = (*len) + bmp_term_len;
-
-      bmp_term_hdr_get_reason_type(&bmp_term_info_loc, &len_loc, &reason_type);
+    bmp_tlv_hdr_get_type(bth, &bmp_tlv_type);
+    bmp_tlv_hdr_get_len(bth, &bmp_tlv_len);
+    if (bmp_tlv_handle_ebit(&bmp_tlv_type)) {
+      if (!(bmp_tlv_get_pen(bmp_packet, len, &bmp_tlv_len, &pen))) {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [term] packet discarded: failed bmp_tlv_get_pen()\n",
+	    config.name, bms->log_str, peer->addr_str);
+	bmp_tlv_list_destroy(tlvs);
+	return;
+      }
     }
 
-    blterm.e[blterm.entries].type = bmp_term_type;
-    blterm.e[blterm.entries].len = bmp_term_len;
-    blterm.e[blterm.entries].val = bmp_term_info;
-    blterm.e[blterm.entries].reas_type = reason_type;
+    if (!(bmp_tlv_value = bmp_get_and_check_length(bmp_packet, len, bmp_tlv_len))) {
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] [term] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
+	  config.name, bms->log_str, peer->addr_str);
+      bmp_tlv_list_destroy(tlvs);
+      return;
+    }
 
-    blterm.entries = bmp_tlv_array_increment(blterm.entries, BMP_TERM_INFO_ENTRIES);
+    ret = bmp_tlv_list_add(tlvs, pen, bmp_tlv_type, bmp_tlv_len, bmp_tlv_value);
+    if (ret == ERR) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [term] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str);
+      exit_gracefully(1);
+    }
   }
 
   if (bms->msglog_backend_methods) {
     char event_type[] = "log";
 
-    bmp_log_msg(peer, &bdata, &blterm, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.nfacctd_bmp_msglog_output, BMP_LOG_TYPE_TERM);
+    bmp_log_msg(peer, &bdata, tlvs, NULL, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.bmp_daemon_msglog_output, BMP_LOG_TYPE_TERM);
   }
 
-  if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, &blterm, BMP_LOG_TYPE_TERM);
+  if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, tlvs, NULL, BMP_LOG_TYPE_TERM);
 
   if (bms->msglog_backend_methods || bms->dump_backend_methods) bgp_peer_log_seq_increment(&bms->log_seq);
 
+  if (!pm_listcount(tlvs) || !bms->dump_backend_methods) bmp_tlv_list_destroy(tlvs);
+
   /* BGP peers are deleted as part of bmp_peer_close() */
 }
 
@@ -282,102 +320,185 @@ void bmp_process_msg_peer_up(char **bmp_packet, u_int32_t *len, struct bmp_peer
 
   bmp_peer_hdr_get_peer_ip(bph, &bdata.peer_ip, &bdata.family);
   bmp_peer_hdr_get_bgp_id(bph, &bdata.bgp_id);
+  bmp_peer_hdr_get_rd(bph, &bdata.chars.rd);
   bmp_peer_hdr_get_tstamp(bph, &bdata.tstamp);
   bmp_peer_hdr_get_peer_asn(bph, &bdata.peer_asn);
 
   if (bdata.family) {
-    /* If no timestamp in BMP then let's generate one */
-    if (!bdata.tstamp.tv_sec) gettimeofday(&bdata.tstamp, NULL);
+    gettimeofday(&bdata.tstamp_arrival, NULL);
 
     {
       struct bmp_log_peer_up blpu;
       struct bgp_peer bgp_peer_loc, bgp_peer_rem, *bmpp_bgp_peer;
       struct bmp_chars bmed_bmp;
       struct bgp_msg_data bmd;
-      int bgp_open_len;
-      void *ret;
+      int bgp_open_len, ret2 = 0;
+      u_int8_t bgp_msg_type = 0;
+      void *ret = NULL;
 
       /* TLV vars */
       struct bmp_tlv_hdr *bth; 
       u_int16_t bmp_tlv_type, bmp_tlv_len;
       char *bmp_tlv_value;
+      struct pm_list *tlvs = NULL;
 
       memset(&bgp_peer_loc, 0, sizeof(bgp_peer_loc));
       memset(&bgp_peer_rem, 0, sizeof(bgp_peer_rem));
       memset(&bmd, 0, sizeof(bmd));
       memset(&bmed_bmp, 0, sizeof(bmed_bmp));
-      blpu.tlv.entries = 0;
+
+      tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+      if (!tlvs) return;
 
       bmp_peer_up_hdr_get_loc_port(bpuh, &blpu.loc_port);
       bmp_peer_up_hdr_get_rem_port(bpuh, &blpu.rem_port);
       bmp_peer_up_hdr_get_local_ip(bpuh, &blpu.local_ip, bdata.family);
 
+      bgp_peer_loc.type = FUNC_TYPE_BMP;
       bmd.peer = &bgp_peer_loc;
+
       bmd.extra.id = BGP_MSG_EXTRA_DATA_BMP;
       bmd.extra.len = sizeof(bmed_bmp);
       bmd.extra.data = &bmed_bmp;
       bgp_msg_data_set_data_bmp(&bmed_bmp, &bdata);
 
-      /* XXX: checks, ie. marker, message length, etc., bypassed */
-      bgp_open_len = bgp_parse_open_msg(&bmd, (*bmp_packet), FALSE, FALSE);
-      if (bgp_open_len == ERR) {
-	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_parse_open_msg()\n",
-		config.name, bms->log_str, peer->addr_str);
+      /* length checks */
+      if ((*len) >= sizeof(struct bgp_header)) {
+	bgp_open_len = bgp_get_packet_len((*bmp_packet));
+	if (bgp_open_len <= 0 || bgp_open_len > (*len)) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_get_packet_len()\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
+	  return;
+	}
+      }
+      else {
+        Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer_up] packet discarded: incomplete BGP header\n",
+            config.name, bms->log_str, peer->addr_str);
+	bmp_tlv_list_destroy(tlvs);
+        return;
+      }
+
+      if ((bgp_msg_type = bgp_get_packet_type((*bmp_packet))) == BGP_OPEN) {
+	bgp_open_len = bgp_parse_open_msg(&bmd, (*bmp_packet), FALSE, FALSE);
+	if (bgp_open_len == ERR) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_parse_open_msg()\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
+	  return;
+        }
+      }
+      else {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: wrong BGP message type: %s (%u)\n",
+	    config.name, bms->log_str, peer->addr_str,
+	    (bgp_msg_type <= BGP_MSG_TYPE_MAX ? bgp_msg_types[bgp_msg_type] : bgp_msg_types[0]),
+	    bgp_msg_type);
+	bmp_tlv_list_destroy(tlvs);
 	return;
       }
+
       bmp_get_and_check_length(bmp_packet, len, bgp_open_len);
+      memcpy(&bmpp->self.id, &bgp_peer_loc.id, sizeof(struct host_addr));
       memcpy(&bgp_peer_loc.addr, &blpu.local_ip, sizeof(struct host_addr));
 
+      bgp_peer_rem.type = FUNC_TYPE_BMP;
       bmd.peer = &bgp_peer_rem;
-      bgp_open_len = bgp_parse_open_msg(&bmd, (*bmp_packet), FALSE, FALSE);
-      if (bgp_open_len == ERR) {
-	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_parse_open_msg() (2)\n",
-		config.name, bms->log_str, peer->addr_str);
+
+      /* length checks */
+      if ((*len) >= sizeof(struct bgp_header)) {
+	bgp_open_len = bgp_get_packet_len((*bmp_packet));
+	if (bgp_open_len <= 0 || bgp_open_len > (*len)) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_get_packet_len() (2)\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
+	  return;
+        }
+      }
+      else {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer_up] packet discarded: incomplete BGP header (2)\n",
+	    config.name, bms->log_str, peer->addr_str);
+	bmp_tlv_list_destroy(tlvs);
 	return;
       }
+
+      if ((bgp_msg_type = bgp_get_packet_type((*bmp_packet))) == BGP_OPEN) {
+	bgp_open_len = bgp_parse_open_msg(&bmd, (*bmp_packet), FALSE, FALSE);
+	if (bgp_open_len == ERR) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bgp_parse_open_msg() (2)\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
+	  return;
+	}
+      }
+      else {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: wrong BGP message type: %u (2)\n",
+	    config.name, bms->log_str, peer->addr_str, bgp_msg_type);
+	bmp_tlv_list_destroy(tlvs);
+	return;
+      }
+
       bmp_get_and_check_length(bmp_packet, len, bgp_open_len);
       memcpy(&bgp_peer_rem.addr, &bdata.peer_ip, sizeof(struct host_addr));
 
       bmpp_bgp_peer = bmp_sync_loc_rem_peers(&bgp_peer_loc, &bgp_peer_rem);
       bmpp_bgp_peer->log = bmpp->self.log; 
       bmpp_bgp_peer->bmp_se = bmpp; /* using bmp_se field to back-point a BGP peer to its parent BMP peer */  
-      ret = pm_tsearch(bmpp_bgp_peer, &bmpp->bgp_peers, bgp_peer_cmp, sizeof(struct bgp_peer));
+
+      if (bdata.family == AF_INET) {
+	ret = pm_tsearch(bmpp_bgp_peer, &bmpp->bgp_peers_v4, bgp_peer_cmp, sizeof(struct bgp_peer));
+      }
+      else if (bdata.family == AF_INET6) {
+	ret = pm_tsearch(bmpp_bgp_peer, &bmpp->bgp_peers_v6, bgp_peer_cmp, sizeof(struct bgp_peer));
+      }
+
       if (!ret) Log(LOG_WARNING, "WARN ( %s/%s ): [%s] [peer up] tsearch() unable to insert.\n", config.name, bms->log_str, peer->addr_str);
 
       while ((*len)) {
+	u_int32_t pen = 0;
+
 	if (!(bth = (struct bmp_tlv_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_tlv_hdr)))) {
 	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bmp_get_and_check_length() BMP TLV hdr\n",
-		config.name, bms->log_str, peer->addr_str);
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
 	  return;
 	}
 
 	bmp_tlv_hdr_get_type(bth, &bmp_tlv_type);
 	bmp_tlv_hdr_get_len(bth, &bmp_tlv_len);
+	if (bmp_tlv_handle_ebit(&bmp_tlv_type)) {
+	  if (!(bmp_tlv_get_pen(bmp_packet, len, &bmp_tlv_len, &pen))) {
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bmp_tlv_get_pen()\n",
+		config.name, bms->log_str, peer->addr_str);
+	    bmp_tlv_list_destroy(tlvs);
+	    return;
+	  }
+	}
 
 	if (!(bmp_tlv_value = bmp_get_and_check_length(bmp_packet, len, bmp_tlv_len))) {
 	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer up] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
-		config.name, bms->log_str, peer->addr_str);
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
 	  return;
 	}
 
-	blpu.tlv.e[blpu.tlv.entries].type = bmp_tlv_type;
-	blpu.tlv.e[blpu.tlv.entries].len = bmp_tlv_len;
-	blpu.tlv.e[blpu.tlv.entries].val = bmp_tlv_value;
-
-        blpu.tlv.entries = bmp_tlv_array_increment(blpu.tlv.entries, BMP_PEER_UP_INFO_ENTRIES);
+	ret2 = bmp_tlv_list_add(tlvs, pen, bmp_tlv_type, bmp_tlv_len, bmp_tlv_value);
+	if (ret2 == ERR) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [peer up] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str);
+	  exit_gracefully(1);
+	}
       }
 
       if (bms->msglog_backend_methods) {
         char event_type[] = "log";
 
-        bmp_log_msg(peer, &bdata, &blpu, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.nfacctd_bmp_msglog_output, BMP_LOG_TYPE_PEER_UP);
+        bmp_log_msg(peer, &bdata, tlvs, &blpu, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.bmp_daemon_msglog_output, BMP_LOG_TYPE_PEER_UP);
       }
 
-      if (bms->dump_backend_methods)
-        bmp_dump_se_ll_append(peer, &bdata, &blpu, BMP_LOG_TYPE_PEER_UP);
+      if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, tlvs, &blpu, BMP_LOG_TYPE_PEER_UP);
 
-      if (bms->msglog_backend_methods || bms->dump_backend_methods)
-        bgp_peer_log_seq_increment(&bms->log_seq);
+      if (bms->msglog_backend_methods || bms->dump_backend_methods) bgp_peer_log_seq_increment(&bms->log_seq);
+
+      if (!pm_listcount(tlvs) || !bms->dump_backend_methods) bmp_tlv_list_destroy(tlvs);
     }
   }
 }
@@ -389,7 +510,7 @@ void bmp_process_msg_peer_down(char **bmp_packet, u_int32_t *len, struct bmp_pee
   struct bmp_data bdata;
   struct bmp_peer_hdr *bph;
   struct bmp_peer_down_hdr *bpdh;
-  void *ret;
+  void *ret = NULL;
 
   if (!bmpp) return;
 
@@ -424,77 +545,122 @@ void bmp_process_msg_peer_down(char **bmp_packet, u_int32_t *len, struct bmp_pee
 
   bmp_peer_hdr_get_peer_ip(bph, &bdata.peer_ip, &bdata.family);
   bmp_peer_hdr_get_bgp_id(bph, &bdata.bgp_id);
+  bmp_peer_hdr_get_rd(bph, &bdata.chars.rd);
   bmp_peer_hdr_get_tstamp(bph, &bdata.tstamp);
   bmp_peer_hdr_get_peer_asn(bph, &bdata.peer_asn);
 
   if (bdata.family) {
-    /* If no timestamp in BMP then let's generate one */
-    if (!bdata.tstamp.tv_sec) gettimeofday(&bdata.tstamp, NULL);
+    gettimeofday(&bdata.tstamp_arrival, NULL);
 
     {
       struct bmp_log_peer_down blpd;
+      int ret2 = 0;
 
       /* TLV vars */
       struct bmp_tlv_hdr *bth; 
       u_int16_t bmp_tlv_type, bmp_tlv_len;
       char *bmp_tlv_value;
+      struct pm_list *tlvs = NULL;
 
-      blpd.tlv.entries = 0;
+      tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+      if (!tlvs) return;
 
       bmp_peer_down_hdr_get_reason(bpdh, &blpd.reason);
       if (blpd.reason == BMP_PEER_DOWN_LOC_CODE) bmp_peer_down_hdr_get_loc_code(bmp_packet, len, &blpd.loc_code);
 
       /* draft-ietf-grow-bmp-tlv */
       if (peer->version == BMP_V4) {
+	/* let's skip intermediate data in order to get to TLVs */
+	if (blpd.reason == BMP_PEER_DOWN_LOC_NOT_MSG || blpd.reason == BMP_PEER_DOWN_REM_NOT_MSG) {
+	  int bgp_notification_len = 0;
+
+	  bgp_notification_len = bgp_get_packet_len((*bmp_packet));
+	  if (bgp_notification_len <= 0 || bgp_notification_len > (*len)) {
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: failed bgp_get_packet_len() reason=%u\n",
+		config.name, bms->log_str, peer->addr_str, blpd.reason);
+	    bmp_tlv_list_destroy(tlvs);
+	    return;
+	  }
+
+	  bmp_jump_offset(bmp_packet, len, bgp_notification_len);
+	}
+	else if (blpd.reason == BMP_PEER_DOWN_LOC_CODE) {
+	  ret2 = bmp_jump_offset(bmp_packet, len, 2);
+	  if (ret2 == ERR) {
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: failed bmp_jump_offset() reason=%u\n",
+		config.name, bms->log_str, peer->addr_str, blpd.reason);
+	    bmp_tlv_list_destroy(tlvs);
+	    return;
+	  }
+	}
+
         while ((*len)) {
+	  u_int32_t pen = 0;
+
 	  if (!(bth = (struct bmp_tlv_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_tlv_hdr)))) {
 	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: failed bmp_get_and_check_length() BMP TLV hdr\n",
 		config.name, bms->log_str, peer->addr_str);
+	    bmp_tlv_list_destroy(tlvs);
 	    return;
 	  }
 
 	  bmp_tlv_hdr_get_type(bth, &bmp_tlv_type);
 	  bmp_tlv_hdr_get_len(bth, &bmp_tlv_len);
+	  if (bmp_tlv_handle_ebit(&bmp_tlv_type)) {
+	    if (!(bmp_tlv_get_pen(bmp_packet, len, &bmp_tlv_len, &pen))) {
+	      Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: failed bmp_tlv_get_pen()\n",
+		  config.name, bms->log_str, peer->addr_str);
+	      bmp_tlv_list_destroy(tlvs);
+	      return;
+	    }
+	  }
 
 	  if (!(bmp_tlv_value = bmp_get_and_check_length(bmp_packet, len, bmp_tlv_len))) {
 	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
 		config.name, bms->log_str, peer->addr_str);
+	    bmp_tlv_list_destroy(tlvs);
 	    return;
 	  }
 
-	  blpd.tlv.e[blpd.tlv.entries].type = bmp_tlv_type;
-	  blpd.tlv.e[blpd.tlv.entries].len = bmp_tlv_len;
-	  blpd.tlv.e[blpd.tlv.entries].val = bmp_tlv_value;
-
-	  blpd.tlv.entries = bmp_tlv_array_increment(blpd.tlv.entries, BMP_PEER_DOWN_INFO_ENTRIES);
+	  ret2 = bmp_tlv_list_add(tlvs, pen, bmp_tlv_type, bmp_tlv_len, bmp_tlv_value);
+	  if (ret2 == ERR) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [peer down] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str);
+	    exit_gracefully(1);
+	  }
 	}
       }
 
       if (bms->msglog_backend_methods) {
         char event_type[] = "log";
 
-        bmp_log_msg(peer, &bdata, &blpd, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.nfacctd_bmp_msglog_output, BMP_LOG_TYPE_PEER_DOWN);
+        bmp_log_msg(peer, &bdata, tlvs, &blpd, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.bmp_daemon_msglog_output, BMP_LOG_TYPE_PEER_DOWN);
       }
 
-      if (bms->dump_backend_methods)
-        bmp_dump_se_ll_append(peer, &bdata, &blpd, BMP_LOG_TYPE_PEER_DOWN);
+      if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, tlvs, &blpd, BMP_LOG_TYPE_PEER_DOWN);
 
-      if (bms->msglog_backend_methods || bms->dump_backend_methods)
-        bgp_peer_log_seq_increment(&bms->log_seq);
+      if (bms->msglog_backend_methods || bms->dump_backend_methods) bgp_peer_log_seq_increment(&bms->log_seq);
+
+      if (!pm_listcount(tlvs) || !bms->dump_backend_methods) bmp_tlv_list_destroy(tlvs);
     }
 
-    ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers, bgp_peer_host_addr_cmp);
+    if (bdata.family == AF_INET) {
+      ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers_v4, bgp_peer_host_addr_cmp);
+    }
+    else if (bdata.family == AF_INET6) {
+      ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers_v6, bgp_peer_host_addr_cmp);
+    }
 
     if (ret) {
-      char peer_str[] = "peer_ip", *saved_peer_str = bms->peer_str;
-
       bmpp_bgp_peer = (*(struct bgp_peer **) ret);
     
-      bms->peer_str = peer_str;
       bgp_peer_info_delete(bmpp_bgp_peer);
-      bms->peer_str = saved_peer_str;
 
-      pm_tdelete(&bdata.peer_ip, &bmpp->bgp_peers, bgp_peer_host_addr_cmp);
+      if (bdata.family == AF_INET) {
+	pm_tdelete(&bdata.peer_ip, &bmpp->bgp_peers_v4, bgp_peer_host_addr_cmp);
+      }
+      else if (bdata.family == AF_INET6) {
+	pm_tdelete(&bdata.peer_ip, &bmpp->bgp_peers_v6, bgp_peer_host_addr_cmp);
+      }
     } 
     /* missing BMP peer up message, ie. case of replay/replication of BMP messages */
     else {
@@ -502,10 +668,10 @@ void bmp_process_msg_peer_down(char **bmp_packet, u_int32_t *len, struct bmp_pee
 
       addr_to_str(peer_ip, &bdata.peer_ip);
 
-      if (!log_notification_isset(&bmpp->missing_peer_up, bdata.tstamp.tv_sec)) {
-        log_notification_set(&bmpp->missing_peer_up, bdata.tstamp.tv_sec, BMP_MISSING_PEER_UP_LOG_TOUT);
+      if (!log_notification_isset(&bmpp->missing_peer_up, bdata.tstamp_arrival.tv_sec)) {
+        log_notification_set(&bmpp->missing_peer_up, bdata.tstamp_arrival.tv_sec, BMP_MISSING_PEER_UP_LOG_TOUT);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] [peer down] packet discarded: missing peer up BMP message for peer %s\n",
-                config.name, bms->log_str, peer->addr_str, peer_ip);
+	    config.name, bms->log_str, peer->addr_str, peer_ip);
       }
     }
   }
@@ -517,9 +683,9 @@ void bmp_process_msg_route_monitor(char **bmp_packet, u_int32_t *len, struct bmp
   struct bgp_peer *peer, *bmpp_bgp_peer;
   struct bmp_data bdata;
   struct bmp_peer_hdr *bph;
-  char tstamp_str[SRVBUFLEN], peer_ip[INET6_ADDRSTRLEN];
-  int bgp_update_len;
-  void *ret;
+  int bgp_update_len, ret2 = 0;
+  u_int8_t bgp_msg_type = 0;
+  void *ret = NULL;
 
   if (!bmpp) return;
 
@@ -531,7 +697,7 @@ void bmp_process_msg_route_monitor(char **bmp_packet, u_int32_t *len, struct bmp
   memset(&bdata, 0, sizeof(bdata));
 
   if (!(bph = (struct bmp_peer_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_peer_hdr)))) {
-    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route] packet discarded: failed bmp_get_and_check_length() BMP peer hdr\n",
+    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: failed bmp_get_and_check_length() BMP peer hdr\n",
         config.name, bms->log_str, peer->addr_str);
     return;
   }
@@ -550,46 +716,63 @@ void bmp_process_msg_route_monitor(char **bmp_packet, u_int32_t *len, struct bmp
 
   bmp_peer_hdr_get_peer_ip(bph, &bdata.peer_ip, &bdata.family);
   bmp_peer_hdr_get_bgp_id(bph, &bdata.bgp_id);
+  bmp_peer_hdr_get_rd(bph, &bdata.chars.rd);
   bmp_peer_hdr_get_tstamp(bph, &bdata.tstamp);
   bmp_peer_hdr_get_peer_asn(bph, &bdata.peer_asn);
 
   if (bdata.family) {
-    /* If no timestamp in BMP then let's generate one */
-    if (!bdata.tstamp.tv_sec) gettimeofday(&bdata.tstamp, NULL);
+    gettimeofday(&bdata.tstamp_arrival, NULL);
 
-    compose_timestamp(tstamp_str, SRVBUFLEN, &bdata.tstamp, TRUE,
-		      config.timestamps_since_epoch, config.timestamps_rfc3339,
-		      config.timestamps_utc);
-    addr_to_str(peer_ip, &bdata.peer_ip);
-
-    ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers, bgp_peer_host_addr_cmp);
+    if (bdata.family == AF_INET) {
+      ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers_v4, bgp_peer_host_addr_cmp);
+    }
+    else if (bdata.family == AF_INET6) {
+      ret = pm_tfind(&bdata.peer_ip, &bmpp->bgp_peers_v6, bgp_peer_host_addr_cmp);
+    }
 
     if (ret) {
-      char peer_str[] = "peer_ip", *saved_peer_str = bms->peer_str;
-      char peer_port_str[] = "peer_tcp_port", *saved_peer_port_str = bms->peer_port_str;
       struct bmp_chars bmed_bmp;
       struct bgp_msg_data bmd;
-      struct bmp_log_route_monitor_tlv_array blm_tlv;
 
       bmpp_bgp_peer = (*(struct bgp_peer **) ret);
       memset(&bmd, 0, sizeof(bmd));
       memset(&bmed_bmp, 0, sizeof(bmed_bmp));
 
-      bms->peer_str = peer_str;
-      bms->peer_port_str = peer_port_str;
       bmd.peer = bmpp_bgp_peer;
       bmd.extra.id = BGP_MSG_EXTRA_DATA_BMP;
       bmd.extra.len = sizeof(bmed_bmp);
       bmd.extra.data = &bmed_bmp;
       bgp_msg_data_set_data_bmp(&bmed_bmp, &bdata);
-      blm_tlv.entries = 0;
 
-      /* draft-ietf-grow-bmp-tlv */
-      bgp_update_len = bgp_get_packet_len((*bmp_packet));
+      compose_timestamp(bms->log_tstamp_str, SRVBUFLEN, &bdata.tstamp, TRUE,
+			config.timestamps_since_epoch, config.timestamps_rfc3339,
+			config.timestamps_utc);
+
+      encode_tstamp_arrival(bms->log_tstamp_str, SRVBUFLEN, &bdata.tstamp_arrival, TRUE);
+
+      /* length checks & draft-ietf-grow-bmp-tlv preps */
+      if ((*len) >= sizeof(struct bgp_header)) {
+        bgp_update_len = bgp_get_packet_len((*bmp_packet));
+        if (bgp_update_len <= 0 || bgp_update_len > (*len)) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: bgp_get_packet_len() failed\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  return;
+	}
+      }
+      else {
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: incomplete BGP header\n",
+	    config.name, bms->log_str, peer->addr_str);
+	return;
+      }
+
       if (peer->version == BMP_V4 && bgp_update_len && bgp_update_len < (*len)) {
 	struct bmp_tlv_hdr *bth;
 	u_int16_t bmp_tlv_type, bmp_tlv_len;
 	char *bmp_tlv_value;
+	struct pm_list *tlvs = NULL;
+
+	tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+	if (!tlvs) return;
 
 	u_int32_t loc_len = (*len);
 	char *loc_ptr = (*bmp_packet);
@@ -597,50 +780,73 @@ void bmp_process_msg_route_monitor(char **bmp_packet, u_int32_t *len, struct bmp
 	bmp_jump_offset(&loc_ptr, &loc_len, bgp_update_len);
 
 	while (loc_len) {
+	  u_int32_t pen = 0;
+
 	  if (!(bth = (struct bmp_tlv_hdr *) bmp_get_and_check_length(&loc_ptr, &loc_len, sizeof(struct bmp_tlv_hdr)))) {
 	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: failed bmp_get_and_check_length() BMP TLV hdr\n",
 		config.name, bms->log_str, peer->addr_str);
+	    bmp_tlv_list_destroy(tlvs);
 	    return;
 	  }
 
 	  bmp_tlv_hdr_get_type(bth, &bmp_tlv_type);
 	  bmp_tlv_hdr_get_len(bth, &bmp_tlv_len);
+	  if (bmp_tlv_handle_ebit(&bmp_tlv_type)) {
+	    if (!(bmp_tlv_get_pen(&loc_ptr, &loc_len, &bmp_tlv_len, &pen))) {
+	      Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: failed bmp_tlv_get_pen()\n",
+		  config.name, bms->log_str, peer->addr_str);
+	      bmp_tlv_list_destroy(tlvs);
+	      return;
+	    }
+	  }
 
 	  if (!(bmp_tlv_value = bmp_get_and_check_length(&loc_ptr, &loc_len, bmp_tlv_len))) {
 	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
 		config.name, bms->log_str, peer->addr_str);
+	    bmp_tlv_list_destroy(tlvs);
 	    return;
 	  }
 
-	  blm_tlv.e[blm_tlv.entries].type = bmp_tlv_type;
-	  blm_tlv.e[blm_tlv.entries].len = bmp_tlv_len;
-	  blm_tlv.e[blm_tlv.entries].val = bmp_tlv_value;
+	  ret2 = bmp_tlv_list_add(tlvs, pen, bmp_tlv_type, bmp_tlv_len, bmp_tlv_value);
+	  if (ret2 == ERR) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [route monitor] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str);
+	    exit_gracefully(1);
+	  }
+        }
 
-	  blm_tlv.entries = bmp_tlv_array_increment(blm_tlv.entries, BMP_ROUTE_MONITOR_INFO_ENTRIES);
+        bmed_bmp.tlvs = tlvs;
+      }
+
+      if ((bgp_msg_type = bgp_get_packet_type((*bmp_packet))) == BGP_UPDATE) {
+	bgp_update_len = bgp_parse_update_msg(&bmd, (*bmp_packet));
+	if (bgp_update_len <= 0) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: bgp_parse_update_msg() failed\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(bmed_bmp.tlvs);
+	  return;
 	}
-	
-        bmed_bmp.tlv = &blm_tlv;
       }
-
-      /* XXX: checks, ie. marker, message length, etc., bypassed */
-      bgp_update_len = bgp_parse_update_msg(&bmd, (*bmp_packet)); 
-      if (!bgp_update_len) {
-	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route] packet discarded: bgp_parse_update_msg() failed\n",
-		config.name, bms->log_str, peer->addr_str);
-	return;
+      else {
+	Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] [route monitor] packet discarded: unsupported BGP message type: %s (%u)\n",
+	    config.name, bms->log_str, peer->addr_str,
+	    (bgp_msg_type <= BGP_MSG_TYPE_MAX ? bgp_msg_types[bgp_msg_type] : bgp_msg_types[0]),
+	    bgp_msg_type);
       }
 
-      bms->peer_str = saved_peer_str;
-      bms->peer_port_str = saved_peer_port_str;
-
       bmp_get_and_check_length(bmp_packet, len, bgp_update_len);
+
+      bmp_tlv_list_destroy(bmed_bmp.tlvs);
     }
     /* missing BMP peer up message, ie. case of replay/replication of BMP messages */
     else {
-      if (!log_notification_isset(&bmpp->missing_peer_up, bdata.tstamp.tv_sec)) {
-	log_notification_set(&bmpp->missing_peer_up, bdata.tstamp.tv_sec, BMP_MISSING_PEER_UP_LOG_TOUT);
-	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route] packet discarded: missing peer up BMP message for peer %s\n",
-		config.name, bms->log_str, peer->addr_str, peer_ip);
+      if (!log_notification_isset(&bmpp->missing_peer_up, bdata.tstamp_arrival.tv_sec)) {
+	char peer_ip[INET6_ADDRSTRLEN];
+
+	addr_to_str(peer_ip, &bdata.peer_ip);
+
+	log_notification_set(&bmpp->missing_peer_up, bdata.tstamp_arrival.tv_sec, BMP_MISSING_PEER_UP_LOG_TOUT);
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route monitor] packet discarded: missing peer up BMP message for peer %s\n",
+	    config.name, bms->log_str, peer->addr_str, peer_ip);
       }
     }
   }
@@ -648,6 +854,19 @@ void bmp_process_msg_route_monitor(char **bmp_packet, u_int32_t *len, struct bmp
 
 void bmp_process_msg_route_mirror(char **bmp_packet, u_int32_t *len, struct bmp_peer *bmpp)
 {
+  struct bgp_misc_structs *bms;
+  struct bgp_peer *peer;
+
+  if (!bmpp) return;
+
+  peer = &bmpp->self;
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return;
+
+  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [route mirror] packet discarded: Unicorn! Message type currently not supported.\n",
+      config.name, bms->log_str, peer->addr_str);
+
   // XXX: maybe support route mirroring
 }
 
@@ -662,9 +881,13 @@ void bmp_process_msg_stats(char **bmp_packet, u_int32_t *len, struct bmp_peer *b
   u_int64_t cnt_data64;
   u_int32_t index, count = 0, cnt_data32;
   u_int16_t cnt_type, cnt_len;
-  u_int8_t got_data;
   afi_t afi;
   safi_t safi;
+  int ret;
+
+  /* unknown stats TLVs */
+  char *cnt_value;
+  struct pm_list *tlvs = NULL;
 
   if (!bmpp) return;
 
@@ -700,24 +923,28 @@ void bmp_process_msg_stats(char **bmp_packet, u_int32_t *len, struct bmp_peer *b
 
   bmp_peer_hdr_get_peer_ip(bph, &bdata.peer_ip, &bdata.family);
   bmp_peer_hdr_get_bgp_id(bph, &bdata.bgp_id);
+  bmp_peer_hdr_get_rd(bph, &bdata.chars.rd);
   bmp_peer_hdr_get_tstamp(bph, &bdata.tstamp);
   bmp_peer_hdr_get_peer_asn(bph, &bdata.peer_asn);
   bmp_stats_hdr_get_count(bsh, &count);
 
   if (bdata.family) {
-    /* If no timestamp in BMP then let's generate one */
-    if (!bdata.tstamp.tv_sec) gettimeofday(&bdata.tstamp, NULL);
+    gettimeofday(&bdata.tstamp_arrival, NULL);
 
     for (index = 0; index < count; index++) {
       if (!(bsch = (struct bmp_stats_cnt_hdr *) bmp_get_and_check_length(bmp_packet, len, sizeof(struct bmp_stats_cnt_hdr)))) {
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] [stats] packet discarded: failed bmp_get_and_check_length() BMP stats cnt hdr #%u\n",
-		config.name, bms->log_str, peer->addr_str, index);
+	    config.name, bms->log_str, peer->addr_str, index);
         return;
       }
 
+      cnt_value = 0;
+      tlvs = bmp_tlv_list_new(NULL, bmp_tlv_list_node_del);
+      if (!tlvs) return;
+
       bmp_stats_cnt_hdr_get_type(bsch, &cnt_type);
       bmp_stats_cnt_hdr_get_len(bsch, &cnt_len);
-      cnt_data32 = 0, cnt_data64 = 0, afi = 0, safi = 0, got_data = TRUE;
+      cnt_data32 = 0, cnt_data64 = 0, afi = 0, safi = 0;
 
       switch (cnt_type) {
       case BMP_STATS_TYPE0:
@@ -775,12 +1002,19 @@ void bmp_process_msg_stats(char **bmp_packet, u_int32_t *len, struct bmp_peer *b
 	if (cnt_len == 11) bmp_stats_cnt_get_afi_safi_data64(bmp_packet, len, &afi, &safi, &cnt_data64);
 	break;
       default:
-        if (cnt_len == 4) bmp_stats_cnt_get_data32(bmp_packet, len, &cnt_data32);
-        else if (cnt_len == 8) bmp_stats_cnt_get_data64(bmp_packet, len, &cnt_data64);
-        else {
-          bmp_get_and_check_length(bmp_packet, len, cnt_len);
-          got_data = FALSE;
-        }
+	if (!(cnt_value = bmp_get_and_check_length(bmp_packet, len, cnt_len))) {
+	  Log(LOG_INFO, "INFO ( %s/%s ): [%s] [stats] packet discarded: failed bmp_get_and_check_length() BMP TLV info\n",
+	      config.name, bms->log_str, peer->addr_str);
+	  bmp_tlv_list_destroy(tlvs);
+	  return;
+	}
+
+	ret = bmp_tlv_list_add(tlvs, 0, cnt_type, cnt_len, cnt_value);
+	if (ret == ERR) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): [%s] [stats] bmp_tlv_list_add() failed.\n", config.name, bms->log_str, peer->addr_str);
+	  exit_gracefully(1);
+	}
+
         break;
       }
 
@@ -789,23 +1023,23 @@ void bmp_process_msg_stats(char **bmp_packet, u_int32_t *len, struct bmp_peer *b
       { 
         struct bmp_log_stats blstats;
 
+	memset(&blstats, 0, sizeof(blstats));
         blstats.cnt_type = cnt_type;
 	blstats.cnt_afi = afi;
 	blstats.cnt_safi = safi;
         blstats.cnt_data = cnt_data64;
-        blstats.got_data = got_data;
 
         if (bms->msglog_backend_methods) {
           char event_type[] = "log";
 
-          bmp_log_msg(peer, &bdata, &blstats, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.nfacctd_bmp_msglog_output, BMP_LOG_TYPE_STATS);
+          bmp_log_msg(peer, &bdata, tlvs, &blstats, bgp_peer_log_seq_get(&bms->log_seq), event_type, config.bmp_daemon_msglog_output, BMP_LOG_TYPE_STATS);
         } 
 
-        if (bms->dump_backend_methods)
-          bmp_dump_se_ll_append(peer, &bdata, &blstats, BMP_LOG_TYPE_STATS);
+        if (bms->dump_backend_methods) bmp_dump_se_ll_append(peer, &bdata, tlvs, &blstats, BMP_LOG_TYPE_STATS);
 
-        if (bms->msglog_backend_methods || bms->dump_backend_methods)
-          bgp_peer_log_seq_increment(&bms->log_seq);
+        if (bms->msglog_backend_methods || bms->dump_backend_methods) bgp_peer_log_seq_increment(&bms->log_seq);
+
+	if (!pm_listcount(tlvs) || !bms->dump_backend_methods) bmp_tlv_list_destroy(tlvs);
       }
     }
   }
@@ -849,7 +1083,7 @@ void bmp_peer_hdr_get_v_flag(struct bmp_peer_hdr *bph, u_int8_t *family)
     (*family) = FALSE;
 
     if (version == 0) (*family) = AF_INET;
-    else if (version == 1) (*family) = AF_INET6;
+    else (*family) = AF_INET6;
   }
 }
 
@@ -909,6 +1143,20 @@ void bmp_peer_hdr_get_bgp_id(struct bmp_peer_hdr *bph, struct host_addr *a)
   }
 }
 
+void bmp_peer_hdr_get_rd(struct bmp_peer_hdr *bph, rd_t *rd)
+{
+  if (bph && rd) {
+    if (bph->type == BMP_PEER_TYPE_L3VPN || bph->type == BMP_PEER_TYPE_LOC_RIB) {
+      memcpy(rd, bph->rd, RD_LEN);
+      bgp_rd_ntoh(rd);
+
+      if (!is_empty_256b(rd, RD_LEN)) {
+        bgp_rd_origin_set(rd, RD_ORIGIN_BMP);
+      }
+    }
+  }
+}
+
 void bmp_peer_hdr_get_tstamp(struct bmp_peer_hdr *bph, struct timeval *tv)
 {
   u_int32_t sec, usec;
