@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -34,8 +34,14 @@ static const struct _dictionary_line dictionary[] = {
   {"daemonize", cfg_key_daemonize},
   {"aggregate", cfg_key_aggregate},
   {"aggregate_primitives", cfg_key_aggregate_primitives},
+  {"cluster_name", cfg_key_cluster_name},
+  {"cluster_id", cfg_key_cluster_id},
+  {"redis_host", cfg_key_redis_host},
+  {"redis_db", cfg_key_redis_db},
   {"snaplen", cfg_key_snaplen},
+  {"propagate_signals", cfg_key_propagate_signals},
   {"aggregate_filter", cfg_key_aggregate_filter},
+  {"dtls_path", cfg_key_dtls_path},
   {"promisc", cfg_key_promisc},
   {"pcap_filter", cfg_key_pcap_filter},
   {"pcap_protocol", cfg_key_pcap_protocol},
@@ -48,6 +54,8 @@ static const struct _dictionary_line dictionary[] = {
   {"pcap_direction", cfg_key_pcap_direction},
   {"pcap_ifindex", cfg_key_pcap_ifindex},
   {"pcap_interfaces_map", cfg_key_pcap_interfaces_map},
+  {"pcap_arista_trailer_offset", cfg_key_pcap_arista_trailer_offset},
+  {"pcap_arista_trailer_flag_value", cfg_key_pcap_arista_trailer_flag_value},
   {"core_proc_name", cfg_key_proc_name},
   {"proc_priority", cfg_key_proc_priority},
   {"pmacctd_as", cfg_key_nfacctd_as_new},
@@ -55,7 +63,6 @@ static const struct _dictionary_line dictionary[] = {
   {"pmacctd_net", cfg_key_nfacctd_net},
   {"uacctd_net", cfg_key_nfacctd_net},
   {"use_ip_next_hop", cfg_key_use_ip_next_hop},
-  {"decode_arista_trailer", cfg_key_decode_arista_trailer},
   {"thread_stack", cfg_key_thread_stack},
   {"plugins", NULL},
   {"plugin_pipe_size", cfg_key_plugin_pipe_size},
@@ -65,12 +72,9 @@ static const struct _dictionary_line dictionary[] = {
   {"plugin_pipe_zmq_profile", cfg_key_plugin_pipe_zmq_profile},
   {"plugin_pipe_zmq_hwm", cfg_key_plugin_pipe_zmq_hwm},
   {"plugin_exit_any", cfg_key_plugin_exit_any},
-  {"interface", cfg_key_pcap_interface}, 		/* Legacy key */
-  {"interface_wait", cfg_key_pcap_interface_wait},	/* Legacy key */
   {"files_umask", cfg_key_files_umask},
   {"files_uid", cfg_key_files_uid},
   {"files_gid", cfg_key_files_gid},
-  {"savefile_wait", cfg_key_pcap_savefile_wait}, /* Legacy key */
   {"networks_mask", cfg_key_networks_mask},
   {"networks_file", cfg_key_networks_file},
   {"networks_file_filter", cfg_key_networks_file_filter},
@@ -104,8 +108,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sql_history", cfg_key_sql_history},
   {"sql_history_offset", cfg_key_sql_history_offset},
   {"sql_history_roundoff", cfg_key_sql_history_roundoff},
-  {"sql_history_since_epoch", cfg_key_timestamps_since_epoch}, /* Legacy key */
-  {"sql_backup_host", cfg_key_sql_recovery_backup_host}, /* Legacy feature; to be obsoleted */
+  {"sql_backup_host", cfg_key_sql_recovery_backup_host}, /* Legacy feature */
   {"sql_recovery_backup_host", cfg_key_sql_recovery_backup_host},
   {"sql_delimiter", cfg_key_sql_delimiter},
   {"sql_max_writers", cfg_key_dump_max_writers},
@@ -187,8 +190,8 @@ static const struct _dictionary_line dictionary[] = {
   {"amqp_vhost", cfg_key_amqp_vhost},
   {"amqp_markers", cfg_key_print_markers},
   {"amqp_output", cfg_key_message_broker_output},
-  {"amqp_avro_schema_routing_key", cfg_key_amqp_avro_schema_routing_key}, /* XXX: Legacy key */
-  {"amqp_avro_schema_refresh_time", cfg_key_amqp_avro_schema_refresh_time}, /* XXX: Legacy key */
+  {"amqp_avro_schema_routing_key", cfg_key_amqp_avro_schema_routing_key}, /* Legacy feature */
+  {"amqp_avro_schema_refresh_time", cfg_key_amqp_avro_schema_refresh_time}, /* Legacy feature */
   {"amqp_trigger_exec", cfg_key_sql_trigger_exec},
   {"kafka_refresh_time", cfg_key_sql_refresh_time},
   {"kafka_history", cfg_key_sql_history},
@@ -216,6 +219,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_proc_name", cfg_key_proc_name},
   {"nfacctd_port", cfg_key_nfacctd_port},
   {"nfacctd_ip", cfg_key_nfacctd_ip},
+  {"nfacctd_ipv6_only", cfg_key_nfacctd_ipv6_only},
   {"nfacctd_allow_file", cfg_key_nfacctd_allow_file},
   {"nfacctd_time_secs", cfg_key_nfacctd_time_secs},
   {"nfacctd_time_new", cfg_key_nfacctd_time_new},
@@ -227,6 +231,8 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_pipe_size", cfg_key_nfacctd_pipe_size},
   {"nfacctd_pro_rating", cfg_key_nfacctd_pro_rating},
   {"nfacctd_templates_file", cfg_key_nfacctd_templates_file},
+  {"nfacctd_templates_receiver", cfg_key_nfacctd_templates_receiver},
+  {"nfacctd_templates_port", cfg_key_nfacctd_templates_port},
   {"nfacctd_account_options", cfg_key_nfacctd_account_options},
   {"nfacctd_stitching", cfg_key_nfacctd_stitching},
   {"nfacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
@@ -238,6 +244,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_kafka_topic", cfg_key_nfacctd_kafka_topic},
   {"nfacctd_kafka_config_file", cfg_key_nfacctd_kafka_config_file},
   {"nfacctd_zmq_address", cfg_key_nfacctd_zmq_address},
+  {"nfacctd_dtls_port", cfg_key_nfacctd_dtls_port},
   {"pmacctd_proc_name", cfg_key_proc_name},
   {"pmacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
   {"pmacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
@@ -268,7 +275,15 @@ static const struct _dictionary_line dictionary[] = {
   {"telemetry_daemon_port_tcp", cfg_key_telemetry_port_tcp},
   {"telemetry_daemon_port_udp", cfg_key_telemetry_port_udp},
   {"telemetry_daemon_ip", cfg_key_telemetry_ip},
+  {"telemetry_daemon_udp_notif_port", cfg_key_telemetry_udp_notif_port},
+  {"telemetry_daemon_udp_notif_ip", cfg_key_telemetry_udp_notif_ip},
+  {"telemetry_daemon_udp_notif_nmsgs", cfg_key_telemetry_udp_notif_nmsgs},
+  {"telemetry_daemon_ipv6_only", cfg_key_telemetry_ipv6_only},
   {"telemetry_daemon_zmq_address", cfg_key_telemetry_zmq_address},
+  {"telemetry_daemon_kafka_broker_host", cfg_key_telemetry_kafka_broker_host},
+  {"telemetry_daemon_kafka_broker_port", cfg_key_telemetry_kafka_broker_port},
+  {"telemetry_daemon_kafka_topic", cfg_key_telemetry_kafka_topic},
+  {"telemetry_daemon_kafka_config_file", cfg_key_telemetry_kafka_config_file},
   {"telemetry_daemon_decoder", cfg_key_telemetry_decoder},
   {"telemetry_daemon_max_peers", cfg_key_telemetry_max_peers},
   {"telemetry_daemon_peer_timeout", cfg_key_telemetry_peer_timeout},
@@ -319,13 +334,12 @@ static const struct _dictionary_line dictionary[] = {
   {"telemetry_dump_kafka_partition", cfg_key_telemetry_dump_kafka_partition},
   {"telemetry_dump_kafka_partition_key", cfg_key_telemetry_dump_kafka_partition_key},
   {"telemetry_dump_kafka_config_file", cfg_key_telemetry_dump_kafka_config_file},
-  {"refresh_maps", cfg_key_maps_refresh}, /* Legacy key */
+  {"telemetry_dump_workers", cfg_key_telemetry_dump_workers},
   {"maps_refresh", cfg_key_maps_refresh},
   {"maps_index", cfg_key_maps_index},
   {"maps_entries", cfg_key_maps_entries},
   {"maps_row_len", cfg_key_maps_row_len},
   {"pre_tag_map", cfg_key_pre_tag_map},	
-  {"pre_tag_map_entries", cfg_key_maps_entries}, /* Legacy key */
   {"pre_tag_filter", cfg_key_pre_tag_filter},
   {"pre_tag2_filter", cfg_key_pre_tag2_filter},
   {"pre_tag_label_filter", cfg_key_pre_tag_label_filter},
@@ -391,6 +405,8 @@ static const struct _dictionary_line dictionary[] = {
   {"nfprobe_hoplimit", cfg_key_nfprobe_hoplimit},
   {"nfprobe_maxflows", cfg_key_nfprobe_maxflows},
   {"nfprobe_receiver", cfg_key_nfprobe_receiver},
+  {"nfprobe_dtls", cfg_key_nfprobe_dtls},
+  {"nfprobe_dtls_verify_cert", cfg_key_nfprobe_dtls_verify_cert},
   {"nfprobe_engine", cfg_key_nfprobe_engine},
   {"nfprobe_version", cfg_key_nfprobe_version},
   {"nfprobe_peer_as", cfg_key_nfprobe_peer_as},
@@ -400,6 +416,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfprobe_ifindex", cfg_key_nfprobe_ifindex},
   {"nfprobe_dont_cache", cfg_key_nfprobe_dont_cache},
   {"nfprobe_ifindex_override", cfg_key_nfprobe_ifindex_override},
+  {"nfprobe_tstamp_usec", cfg_key_nfprobe_tstamp_usec},
   {"sfprobe_receiver", cfg_key_sfprobe_receiver},
   {"sfprobe_agentip", cfg_key_sfprobe_agentip},
   {"sfprobe_agentsubid", cfg_key_sfprobe_agentsubid},
@@ -418,95 +435,97 @@ static const struct _dictionary_line dictionary[] = {
   {"tee_ipprec", cfg_key_nfprobe_ip_precedence},
   {"tee_pipe_size", cfg_key_tee_pipe_size},
   {"tee_kafka_config_file", cfg_key_tee_kafka_config_file},
-  {"bgp_daemon", cfg_key_nfacctd_bgp},
-  {"bgp_daemon_ip", cfg_key_nfacctd_bgp_ip},
-  {"bgp_daemon_id", cfg_key_nfacctd_bgp_id},
-  {"bgp_daemon_as", cfg_key_nfacctd_bgp_as},
-  {"bgp_daemon_port", cfg_key_nfacctd_bgp_port},
-  {"bgp_daemon_pipe_size", cfg_key_nfacctd_bgp_pipe_size},
-  {"bgp_daemon_max_peers", cfg_key_nfacctd_bgp_max_peers},
-  {"bgp_daemon_msglog_output", cfg_key_nfacctd_bgp_msglog_output},
-  {"bgp_daemon_msglog_file", cfg_key_nfacctd_bgp_msglog_file},
-  {"bgp_daemon_msglog_avro_schema_file", cfg_key_nfacctd_bgp_msglog_avro_schema_file},
-  {"bgp_daemon_msglog_amqp_host", cfg_key_nfacctd_bgp_msglog_amqp_host},
-  {"bgp_daemon_msglog_amqp_vhost", cfg_key_nfacctd_bgp_msglog_amqp_vhost},
-  {"bgp_daemon_msglog_amqp_user", cfg_key_nfacctd_bgp_msglog_amqp_user},
-  {"bgp_daemon_msglog_amqp_passwd", cfg_key_nfacctd_bgp_msglog_amqp_passwd},
-  {"bgp_daemon_msglog_amqp_exchange", cfg_key_nfacctd_bgp_msglog_amqp_exchange},
-  {"bgp_daemon_msglog_amqp_exchange_type", cfg_key_nfacctd_bgp_msglog_amqp_exchange_type},
-  {"bgp_daemon_msglog_amqp_routing_key", cfg_key_nfacctd_bgp_msglog_amqp_routing_key},
-  {"bgp_daemon_msglog_amqp_routing_key_rr", cfg_key_nfacctd_bgp_msglog_amqp_routing_key_rr},
-  {"bgp_daemon_msglog_amqp_persistent_msg", cfg_key_nfacctd_bgp_msglog_amqp_persistent_msg},
-  {"bgp_daemon_msglog_amqp_frame_max", cfg_key_nfacctd_bgp_msglog_amqp_frame_max},
-  {"bgp_daemon_msglog_amqp_heartbeat_interval", cfg_key_nfacctd_bgp_msglog_amqp_heartbeat_interval},
-  {"bgp_daemon_msglog_amqp_retry", cfg_key_nfacctd_bgp_msglog_amqp_retry},
-  {"bgp_daemon_msglog_kafka_broker_host", cfg_key_nfacctd_bgp_msglog_kafka_broker_host},
-  {"bgp_daemon_msglog_kafka_broker_port", cfg_key_nfacctd_bgp_msglog_kafka_broker_port},
-  {"bgp_daemon_msglog_kafka_topic", cfg_key_nfacctd_bgp_msglog_kafka_topic},
-  {"bgp_daemon_msglog_kafka_topic_rr", cfg_key_nfacctd_bgp_msglog_kafka_topic_rr},
-  {"bgp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bgp_msglog_kafka_partition},
-  {"bgp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bgp_msglog_kafka_partition_key},
-  {"bgp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bgp_msglog_kafka_retry},
-  {"bgp_daemon_msglog_kafka_config_file", cfg_key_nfacctd_bgp_msglog_kafka_config_file},
-  {"bgp_daemon_msglog_kafka_avro_schema_registry", cfg_key_nfacctd_bgp_msglog_kafka_avro_schema_registry},
-  {"bgp_daemon_allow_file", cfg_key_nfacctd_bgp_allow_file},
-  {"bgp_daemon_ipprec", cfg_key_nfacctd_bgp_ip_precedence},
-  {"bgp_daemon_md5_file", cfg_key_nfacctd_bgp_md5_file},
-  {"bgp_daemon_batch", cfg_key_nfacctd_bgp_batch},
-  {"bgp_daemon_batch_interval", cfg_key_nfacctd_bgp_batch_interval},
-  {"bgp_aspath_radius", cfg_key_nfacctd_bgp_aspath_radius},
-  {"bgp_stdcomm_pattern", cfg_key_nfacctd_bgp_stdcomm_pattern},
-  {"bgp_extcomm_pattern", cfg_key_nfacctd_bgp_extcomm_pattern},
-  {"bgp_lrgcomm_pattern", cfg_key_nfacctd_bgp_lrgcomm_pattern},
-  {"bgp_stdcomm_pattern_to_asn", cfg_key_nfacctd_bgp_stdcomm_pattern_to_asn},
-  {"bgp_lrgcomm_pattern_to_asn", cfg_key_nfacctd_bgp_lrgcomm_pattern_to_asn},
+  {"bgp_daemon", cfg_key_bgp_daemon},
+  {"bgp_daemon_ip", cfg_key_bgp_daemon_ip},
+  {"bgp_daemon_ipv6_only", cfg_key_bgp_daemon_ipv6_only},
+  {"bgp_daemon_id", cfg_key_bgp_daemon_id},
+  {"bgp_daemon_as", cfg_key_bgp_daemon_as},
+  {"bgp_daemon_port", cfg_key_bgp_daemon_port},
+  {"bgp_daemon_pipe_size", cfg_key_bgp_daemon_pipe_size},
+  {"bgp_daemon_max_peers", cfg_key_bgp_daemon_max_peers},
+  {"bgp_daemon_msglog_output", cfg_key_bgp_daemon_msglog_output},
+  {"bgp_daemon_msglog_file", cfg_key_bgp_daemon_msglog_file},
+  {"bgp_daemon_msglog_avro_schema_file", cfg_key_bgp_daemon_msglog_avro_schema_file},
+  {"bgp_daemon_msglog_amqp_host", cfg_key_bgp_daemon_msglog_amqp_host},
+  {"bgp_daemon_msglog_amqp_vhost", cfg_key_bgp_daemon_msglog_amqp_vhost},
+  {"bgp_daemon_msglog_amqp_user", cfg_key_bgp_daemon_msglog_amqp_user},
+  {"bgp_daemon_msglog_amqp_passwd", cfg_key_bgp_daemon_msglog_amqp_passwd},
+  {"bgp_daemon_msglog_amqp_exchange", cfg_key_bgp_daemon_msglog_amqp_exchange},
+  {"bgp_daemon_msglog_amqp_exchange_type", cfg_key_bgp_daemon_msglog_amqp_exchange_type},
+  {"bgp_daemon_msglog_amqp_routing_key", cfg_key_bgp_daemon_msglog_amqp_routing_key},
+  {"bgp_daemon_msglog_amqp_routing_key_rr", cfg_key_bgp_daemon_msglog_amqp_routing_key_rr},
+  {"bgp_daemon_msglog_amqp_persistent_msg", cfg_key_bgp_daemon_msglog_amqp_persistent_msg},
+  {"bgp_daemon_msglog_amqp_frame_max", cfg_key_bgp_daemon_msglog_amqp_frame_max},
+  {"bgp_daemon_msglog_amqp_heartbeat_interval", cfg_key_bgp_daemon_msglog_amqp_heartbeat_interval},
+  {"bgp_daemon_msglog_amqp_retry", cfg_key_bgp_daemon_msglog_amqp_retry},
+  {"bgp_daemon_msglog_kafka_broker_host", cfg_key_bgp_daemon_msglog_kafka_broker_host},
+  {"bgp_daemon_msglog_kafka_broker_port", cfg_key_bgp_daemon_msglog_kafka_broker_port},
+  {"bgp_daemon_msglog_kafka_topic", cfg_key_bgp_daemon_msglog_kafka_topic},
+  {"bgp_daemon_msglog_kafka_topic_rr", cfg_key_bgp_daemon_msglog_kafka_topic_rr},
+  {"bgp_daemon_msglog_kafka_partition", cfg_key_bgp_daemon_msglog_kafka_partition},
+  {"bgp_daemon_msglog_kafka_partition_key", cfg_key_bgp_daemon_msglog_kafka_partition_key},
+  {"bgp_daemon_msglog_kafka_retry", cfg_key_bgp_daemon_msglog_kafka_retry},
+  {"bgp_daemon_msglog_kafka_config_file", cfg_key_bgp_daemon_msglog_kafka_config_file},
+  {"bgp_daemon_msglog_kafka_avro_schema_registry", cfg_key_bgp_daemon_msglog_kafka_avro_schema_registry},
+  {"bgp_daemon_allow_file", cfg_key_bgp_daemon_allow_file},
+  {"bgp_daemon_ipprec", cfg_key_bgp_daemon_ip_precedence},
+  {"bgp_daemon_md5_file", cfg_key_bgp_daemon_md5_file},
+  {"bgp_daemon_batch", cfg_key_bgp_daemon_batch},
+  {"bgp_daemon_batch_interval", cfg_key_bgp_daemon_batch_interval},
+  {"bgp_aspath_radius", cfg_key_bgp_daemon_aspath_radius},
+  {"bgp_stdcomm_pattern", cfg_key_bgp_daemon_stdcomm_pattern},
+  {"bgp_extcomm_pattern", cfg_key_bgp_daemon_extcomm_pattern},
+  {"bgp_lrgcomm_pattern", cfg_key_bgp_daemon_lrgcomm_pattern},
+  {"bgp_stdcomm_pattern_to_asn", cfg_key_bgp_daemon_stdcomm_pattern_to_asn},
+  {"bgp_lrgcomm_pattern_to_asn", cfg_key_bgp_daemon_lrgcomm_pattern_to_asn},
   {"bgp_blackhole_stdcomm_list", cfg_key_bgp_blackhole_stdcomm_list},
-  {"bgp_peer_as_skip_subas", cfg_key_nfacctd_bgp_peer_as_skip_subas},
-  {"bgp_peer_src_as_map", cfg_key_nfacctd_bgp_peer_src_as_map},
-  {"bgp_src_local_pref_map", cfg_key_nfacctd_bgp_src_local_pref_map},
-  {"bgp_src_med_map", cfg_key_nfacctd_bgp_src_med_map},
-  {"bgp_peer_src_as_type", cfg_key_nfacctd_bgp_peer_src_as_type},
-  {"bgp_src_std_comm_type", cfg_key_nfacctd_bgp_src_std_comm_type},
-  {"bgp_src_ext_comm_type", cfg_key_nfacctd_bgp_src_ext_comm_type},
-  {"bgp_src_lrg_comm_type", cfg_key_nfacctd_bgp_src_lrg_comm_type},
-  {"bgp_src_as_path_type", cfg_key_nfacctd_bgp_src_as_path_type},
-  {"bgp_src_local_pref_type", cfg_key_nfacctd_bgp_src_local_pref_type},
-  {"bgp_src_med_type", cfg_key_nfacctd_bgp_src_med_type},
-  {"bgp_src_roa_type", cfg_key_nfacctd_bgp_src_roa_type},
-  {"bgp_agent_map", cfg_key_nfacctd_bgp_to_agent_map},
-  {"bgp_follow_default", cfg_key_nfacctd_bgp_follow_default},
-  {"bgp_follow_nexthop", cfg_key_nfacctd_bgp_follow_nexthop},
-  {"bgp_follow_nexthop_external", cfg_key_nfacctd_bgp_follow_nexthop_external},
-  {"bgp_disable_router_id_check", cfg_key_nfacctd_bgp_disable_router_id_check},
-  {"bgp_neighbors_file", cfg_key_nfacctd_bgp_neighbors_file},
-  {"bgp_table_peer_buckets", cfg_key_nfacctd_bgp_table_peer_buckets},
-  {"bgp_table_per_peer_buckets", cfg_key_nfacctd_bgp_table_per_peer_buckets},
-  {"bgp_table_attr_hash_buckets", cfg_key_nfacctd_bgp_table_attr_hash_buckets},
-  {"bgp_table_per_peer_hash", cfg_key_nfacctd_bgp_table_per_peer_hash},
-  {"bgp_table_dump_output", cfg_key_nfacctd_bgp_table_dump_output},
-  {"bgp_table_dump_file", cfg_key_nfacctd_bgp_table_dump_file},
-  {"bgp_table_dump_latest_file", cfg_key_nfacctd_bgp_table_dump_latest_file},
-  {"bgp_table_dump_avro_schema_file", cfg_key_nfacctd_bgp_table_dump_avro_schema_file},
-  {"bgp_table_dump_refresh_time", cfg_key_nfacctd_bgp_table_dump_refresh_time},
-  {"bgp_table_dump_amqp_host", cfg_key_nfacctd_bgp_table_dump_amqp_host},
-  {"bgp_table_dump_amqp_vhost", cfg_key_nfacctd_bgp_table_dump_amqp_vhost},
-  {"bgp_table_dump_amqp_user", cfg_key_nfacctd_bgp_table_dump_amqp_user},
-  {"bgp_table_dump_amqp_passwd", cfg_key_nfacctd_bgp_table_dump_amqp_passwd},
-  {"bgp_table_dump_amqp_exchange", cfg_key_nfacctd_bgp_table_dump_amqp_exchange},
-  {"bgp_table_dump_amqp_exchange_type", cfg_key_nfacctd_bgp_table_dump_amqp_exchange_type},
-  {"bgp_table_dump_amqp_routing_key", cfg_key_nfacctd_bgp_table_dump_amqp_routing_key},
-  {"bgp_table_dump_amqp_routing_key_rr", cfg_key_nfacctd_bgp_table_dump_amqp_routing_key_rr},
-  {"bgp_table_dump_amqp_persistent_msg", cfg_key_nfacctd_bgp_table_dump_amqp_persistent_msg},
-  {"bgp_table_dump_amqp_frame_max", cfg_key_nfacctd_bgp_table_dump_amqp_frame_max},
-  {"bgp_table_dump_amqp_heartbeat_interval", cfg_key_nfacctd_bgp_table_dump_amqp_heartbeat_interval},
-  {"bgp_table_dump_kafka_broker_host", cfg_key_nfacctd_bgp_table_dump_kafka_broker_host},
-  {"bgp_table_dump_kafka_broker_port", cfg_key_nfacctd_bgp_table_dump_kafka_broker_port},
-  {"bgp_table_dump_kafka_topic", cfg_key_nfacctd_bgp_table_dump_kafka_topic},
-  {"bgp_table_dump_kafka_topic_rr", cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr},
-  {"bgp_table_dump_kafka_partition", cfg_key_nfacctd_bgp_table_dump_kafka_partition},
-  {"bgp_table_dump_kafka_partition_key", cfg_key_nfacctd_bgp_table_dump_kafka_partition_key},
-  {"bgp_table_dump_kafka_config_file", cfg_key_nfacctd_bgp_table_dump_kafka_config_file},
-  {"bgp_table_dump_kafka_avro_schema_registry", cfg_key_nfacctd_bgp_table_dump_kafka_avro_schema_registry},
+  {"bgp_peer_as_skip_subas", cfg_key_bgp_daemon_peer_as_skip_subas},
+  {"bgp_peer_src_as_map", cfg_key_bgp_daemon_peer_src_as_map},
+  {"bgp_src_local_pref_map", cfg_key_bgp_daemon_src_local_pref_map},
+  {"bgp_src_med_map", cfg_key_bgp_daemon_src_med_map},
+  {"bgp_peer_src_as_type", cfg_key_bgp_daemon_peer_src_as_type},
+  {"bgp_src_std_comm_type", cfg_key_bgp_daemon_src_std_comm_type},
+  {"bgp_src_ext_comm_type", cfg_key_bgp_daemon_src_ext_comm_type},
+  {"bgp_src_lrg_comm_type", cfg_key_bgp_daemon_src_lrg_comm_type},
+  {"bgp_src_as_path_type", cfg_key_bgp_daemon_src_as_path_type},
+  {"bgp_src_local_pref_type", cfg_key_bgp_daemon_src_local_pref_type},
+  {"bgp_src_med_type", cfg_key_bgp_daemon_src_med_type},
+  {"bgp_src_roa_type", cfg_key_bgp_daemon_src_roa_type},
+  {"bgp_agent_map", cfg_key_bgp_daemon_to_xflow_agent_map},
+  {"bgp_follow_default", cfg_key_bgp_daemon_follow_default},
+  {"bgp_follow_nexthop", cfg_key_bgp_daemon_follow_nexthop},
+  {"bgp_follow_nexthop_external", cfg_key_bgp_daemon_follow_nexthop_external},
+  {"bgp_disable_router_id_check", cfg_key_bgp_daemon_disable_router_id_check},
+  {"bgp_neighbors_file", cfg_key_bgp_daemon_neighbors_file},
+  {"bgp_table_peer_buckets", cfg_key_bgp_daemon_table_peer_buckets},
+  {"bgp_table_per_peer_buckets", cfg_key_bgp_daemon_table_per_peer_buckets},
+  {"bgp_table_attr_hash_buckets", cfg_key_bgp_daemon_table_attr_hash_buckets},
+  {"bgp_table_per_peer_hash", cfg_key_bgp_daemon_table_per_peer_hash},
+  {"bgp_table_dump_output", cfg_key_bgp_daemon_table_dump_output},
+  {"bgp_table_dump_file", cfg_key_bgp_daemon_table_dump_file},
+  {"bgp_table_dump_latest_file", cfg_key_bgp_daemon_table_dump_latest_file},
+  {"bgp_table_dump_avro_schema_file", cfg_key_bgp_daemon_table_dump_avro_schema_file},
+  {"bgp_table_dump_refresh_time", cfg_key_bgp_daemon_table_dump_refresh_time},
+  {"bgp_table_dump_amqp_host", cfg_key_bgp_daemon_table_dump_amqp_host},
+  {"bgp_table_dump_amqp_vhost", cfg_key_bgp_daemon_table_dump_amqp_vhost},
+  {"bgp_table_dump_amqp_user", cfg_key_bgp_daemon_table_dump_amqp_user},
+  {"bgp_table_dump_amqp_passwd", cfg_key_bgp_daemon_table_dump_amqp_passwd},
+  {"bgp_table_dump_amqp_exchange", cfg_key_bgp_daemon_table_dump_amqp_exchange},
+  {"bgp_table_dump_amqp_exchange_type", cfg_key_bgp_daemon_table_dump_amqp_exchange_type},
+  {"bgp_table_dump_amqp_routing_key", cfg_key_bgp_daemon_table_dump_amqp_routing_key},
+  {"bgp_table_dump_amqp_routing_key_rr", cfg_key_bgp_daemon_table_dump_amqp_routing_key_rr},
+  {"bgp_table_dump_amqp_persistent_msg", cfg_key_bgp_daemon_table_dump_amqp_persistent_msg},
+  {"bgp_table_dump_amqp_frame_max", cfg_key_bgp_daemon_table_dump_amqp_frame_max},
+  {"bgp_table_dump_amqp_heartbeat_interval", cfg_key_bgp_daemon_table_dump_amqp_heartbeat_interval},
+  {"bgp_table_dump_kafka_broker_host", cfg_key_bgp_daemon_table_dump_kafka_broker_host},
+  {"bgp_table_dump_kafka_broker_port", cfg_key_bgp_daemon_table_dump_kafka_broker_port},
+  {"bgp_table_dump_kafka_topic", cfg_key_bgp_daemon_table_dump_kafka_topic},
+  {"bgp_table_dump_kafka_topic_rr", cfg_key_bgp_daemon_table_dump_kafka_topic_rr},
+  {"bgp_table_dump_kafka_partition", cfg_key_bgp_daemon_table_dump_kafka_partition},
+  {"bgp_table_dump_kafka_partition_key", cfg_key_bgp_daemon_table_dump_kafka_partition_key},
+  {"bgp_table_dump_kafka_config_file", cfg_key_bgp_daemon_table_dump_kafka_config_file},
+  {"bgp_table_dump_kafka_avro_schema_registry", cfg_key_bgp_daemon_table_dump_kafka_avro_schema_registry},
+  {"bgp_table_dump_workers", cfg_key_bgp_daemon_table_dump_workers},
   {"bgp_daemon_lg", cfg_key_bgp_lg},
   {"bgp_daemon_lg_ip", cfg_key_bgp_lg_ip},
   {"bgp_daemon_lg_port", cfg_key_bgp_lg_port},
@@ -514,67 +533,71 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_daemon_lg_user", cfg_key_bgp_lg_user},
   {"bgp_daemon_lg_passwd", cfg_key_bgp_lg_passwd},
   {"bgp_daemon_xconnect_map", cfg_key_bgp_xconnect_map},
-  {"bmp_daemon", cfg_key_nfacctd_bmp},
-  {"bmp_daemon_ip", cfg_key_nfacctd_bmp_ip},
-  {"bmp_daemon_port", cfg_key_nfacctd_bmp_port},
-  {"bmp_daemon_pipe_size", cfg_key_nfacctd_bmp_pipe_size},
-  {"bmp_daemon_max_peers", cfg_key_nfacctd_bmp_max_peers},
-  {"bmp_daemon_allow_file", cfg_key_nfacctd_bmp_allow_file},
-  {"bmp_daemon_ipprec", cfg_key_nfacctd_bmp_ip_precedence},
-  {"bmp_daemon_batch", cfg_key_nfacctd_bmp_batch},
-  {"bmp_daemon_batch_interval", cfg_key_nfacctd_bmp_batch_interval},
-  {"bmp_daemon_msglog_output", cfg_key_nfacctd_bmp_msglog_output},
-  {"bmp_daemon_msglog_file", cfg_key_nfacctd_bmp_msglog_file},
-  {"bmp_daemon_msglog_avro_schema_file", cfg_key_nfacctd_bmp_msglog_avro_schema_file},
-  {"bmp_daemon_msglog_amqp_host", cfg_key_nfacctd_bmp_msglog_amqp_host},
-  {"bmp_daemon_msglog_amqp_vhost", cfg_key_nfacctd_bmp_msglog_amqp_vhost},
-  {"bmp_daemon_msglog_amqp_user", cfg_key_nfacctd_bmp_msglog_amqp_user},
-  {"bmp_daemon_msglog_amqp_passwd", cfg_key_nfacctd_bmp_msglog_amqp_passwd},
-  {"bmp_daemon_msglog_amqp_exchange", cfg_key_nfacctd_bmp_msglog_amqp_exchange},
-  {"bmp_daemon_msglog_amqp_exchange_type", cfg_key_nfacctd_bmp_msglog_amqp_exchange_type},
-  {"bmp_daemon_msglog_amqp_routing_key", cfg_key_nfacctd_bmp_msglog_amqp_routing_key},
-  {"bmp_daemon_msglog_amqp_routing_key_rr", cfg_key_nfacctd_bmp_msglog_amqp_routing_key_rr},
-  {"bmp_daemon_msglog_amqp_persistent_msg", cfg_key_nfacctd_bmp_msglog_amqp_persistent_msg},
-  {"bmp_daemon_msglog_amqp_frame_max", cfg_key_nfacctd_bmp_msglog_amqp_frame_max},
-  {"bmp_daemon_msglog_amqp_heartbeat_interval", cfg_key_nfacctd_bmp_msglog_amqp_heartbeat_interval},
-  {"bmp_daemon_msglog_amqp_retry", cfg_key_nfacctd_bmp_msglog_amqp_retry},
-  {"bmp_daemon_msglog_kafka_broker_host", cfg_key_nfacctd_bmp_msglog_kafka_broker_host},
-  {"bmp_daemon_msglog_kafka_broker_port", cfg_key_nfacctd_bmp_msglog_kafka_broker_port},
-  {"bmp_daemon_msglog_kafka_topic", cfg_key_nfacctd_bmp_msglog_kafka_topic},
-  {"bmp_daemon_msglog_kafka_topic_rr", cfg_key_nfacctd_bmp_msglog_kafka_topic_rr},
-  {"bmp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bmp_msglog_kafka_partition},
-  {"bmp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bmp_msglog_kafka_partition_key},
-  {"bmp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bmp_msglog_kafka_retry},
-  {"bmp_daemon_msglog_kafka_config_file", cfg_key_nfacctd_bmp_msglog_kafka_config_file},
-  {"bmp_daemon_msglog_kafka_avro_schema_registry", cfg_key_nfacctd_bmp_msglog_kafka_avro_schema_registry},
-  {"bmp_table_peer_buckets", cfg_key_nfacctd_bmp_table_peer_buckets},
-  {"bmp_table_per_peer_buckets", cfg_key_nfacctd_bmp_table_per_peer_buckets},
-  {"bmp_table_attr_hash_buckets", cfg_key_nfacctd_bmp_table_attr_hash_buckets},
-  {"bmp_table_per_peer_hash", cfg_key_nfacctd_bmp_table_per_peer_hash},
-  {"bmp_dump_output", cfg_key_nfacctd_bmp_dump_output},
-  {"bmp_dump_file", cfg_key_nfacctd_bmp_dump_file},
-  {"bmp_dump_latest_file", cfg_key_nfacctd_bmp_dump_latest_file},
-  {"bmp_dump_avro_schema_file", cfg_key_nfacctd_bmp_dump_avro_schema_file},
-  {"bmp_dump_refresh_time", cfg_key_nfacctd_bmp_dump_refresh_time},
-  {"bmp_dump_amqp_host", cfg_key_nfacctd_bmp_dump_amqp_host},
-  {"bmp_dump_amqp_vhost", cfg_key_nfacctd_bmp_dump_amqp_vhost},
-  {"bmp_dump_amqp_user", cfg_key_nfacctd_bmp_dump_amqp_user},
-  {"bmp_dump_amqp_passwd", cfg_key_nfacctd_bmp_dump_amqp_passwd},
-  {"bmp_dump_amqp_exchange", cfg_key_nfacctd_bmp_dump_amqp_exchange},
-  {"bmp_dump_amqp_exchange_type", cfg_key_nfacctd_bmp_dump_amqp_exchange_type},
-  {"bmp_dump_amqp_routing_key", cfg_key_nfacctd_bmp_dump_amqp_routing_key},
-  {"bmp_dump_amqp_routing_key_rr", cfg_key_nfacctd_bmp_dump_amqp_routing_key_rr},
-  {"bmp_dump_amqp_persistent_msg", cfg_key_nfacctd_bmp_dump_amqp_persistent_msg},
-  {"bmp_dump_amqp_frame_max", cfg_key_nfacctd_bmp_dump_amqp_frame_max},
-  {"bmp_dump_amqp_heartbeat_interval", cfg_key_nfacctd_bmp_dump_amqp_heartbeat_interval},
-  {"bmp_dump_kafka_broker_host", cfg_key_nfacctd_bmp_dump_kafka_broker_host},
-  {"bmp_dump_kafka_broker_port", cfg_key_nfacctd_bmp_dump_kafka_broker_port},
-  {"bmp_dump_kafka_topic", cfg_key_nfacctd_bmp_dump_kafka_topic},
-  {"bmp_dump_kafka_topic_rr", cfg_key_nfacctd_bmp_dump_kafka_topic_rr},
-  {"bmp_dump_kafka_partition", cfg_key_nfacctd_bmp_dump_kafka_partition},
-  {"bmp_dump_kafka_partition_key", cfg_key_nfacctd_bmp_dump_kafka_partition_key},
-  {"bmp_dump_kafka_config_file", cfg_key_nfacctd_bmp_dump_kafka_config_file},
-  {"bmp_dump_kafka_avro_schema_registry", cfg_key_nfacctd_bmp_dump_kafka_avro_schema_registry},
+  {"bmp_daemon", cfg_key_bmp_daemon},
+  {"bmp_daemon_ip", cfg_key_bmp_daemon_ip},
+  {"bmp_daemon_ipv6_only", cfg_key_bmp_daemon_ipv6_only},
+  {"bmp_daemon_port", cfg_key_bmp_daemon_port},
+  {"bmp_daemon_pipe_size", cfg_key_bmp_daemon_pipe_size},
+  {"bmp_daemon_max_peers", cfg_key_bmp_daemon_max_peers},
+  {"bmp_daemon_allow_file", cfg_key_bmp_daemon_allow_file},
+  {"bmp_daemon_ipprec", cfg_key_bmp_daemon_ip_precedence},
+  {"bmp_daemon_batch", cfg_key_bmp_daemon_batch},
+  {"bmp_daemon_batch_interval", cfg_key_bmp_daemon_batch_interval},
+  {"bmp_agent_map", cfg_key_bgp_daemon_to_xflow_agent_map},
+  {"bmp_daemon_msglog_output", cfg_key_bmp_daemon_msglog_output},
+  {"bmp_daemon_msglog_file", cfg_key_bmp_daemon_msglog_file},
+  {"bmp_daemon_msglog_avro_schema_file", cfg_key_bmp_daemon_msglog_avro_schema_file},
+  {"bmp_daemon_msglog_amqp_host", cfg_key_bmp_daemon_msglog_amqp_host},
+  {"bmp_daemon_msglog_amqp_vhost", cfg_key_bmp_daemon_msglog_amqp_vhost},
+  {"bmp_daemon_msglog_amqp_user", cfg_key_bmp_daemon_msglog_amqp_user},
+  {"bmp_daemon_msglog_amqp_passwd", cfg_key_bmp_daemon_msglog_amqp_passwd},
+  {"bmp_daemon_msglog_amqp_exchange", cfg_key_bmp_daemon_msglog_amqp_exchange},
+  {"bmp_daemon_msglog_amqp_exchange_type", cfg_key_bmp_daemon_msglog_amqp_exchange_type},
+  {"bmp_daemon_msglog_amqp_routing_key", cfg_key_bmp_daemon_msglog_amqp_routing_key},
+  {"bmp_daemon_msglog_amqp_routing_key_rr", cfg_key_bmp_daemon_msglog_amqp_routing_key_rr},
+  {"bmp_daemon_msglog_amqp_persistent_msg", cfg_key_bmp_daemon_msglog_amqp_persistent_msg},
+  {"bmp_daemon_msglog_amqp_frame_max", cfg_key_bmp_daemon_msglog_amqp_frame_max},
+  {"bmp_daemon_msglog_amqp_heartbeat_interval", cfg_key_bmp_daemon_msglog_amqp_heartbeat_interval},
+  {"bmp_daemon_msglog_amqp_retry", cfg_key_bmp_daemon_msglog_amqp_retry},
+  {"bmp_daemon_msglog_kafka_broker_host", cfg_key_bmp_daemon_msglog_kafka_broker_host},
+  {"bmp_daemon_msglog_kafka_broker_port", cfg_key_bmp_daemon_msglog_kafka_broker_port},
+  {"bmp_daemon_msglog_kafka_topic", cfg_key_bmp_daemon_msglog_kafka_topic},
+  {"bmp_daemon_msglog_kafka_topic_rr", cfg_key_bmp_daemon_msglog_kafka_topic_rr},
+  {"bmp_daemon_msglog_kafka_partition", cfg_key_bmp_daemon_msglog_kafka_partition},
+  {"bmp_daemon_msglog_kafka_partition_key", cfg_key_bmp_daemon_msglog_kafka_partition_key},
+  {"bmp_daemon_msglog_kafka_retry", cfg_key_bmp_daemon_msglog_kafka_retry},
+  {"bmp_daemon_msglog_kafka_config_file", cfg_key_bmp_daemon_msglog_kafka_config_file},
+  {"bmp_daemon_msglog_kafka_avro_schema_registry", cfg_key_bmp_daemon_msglog_kafka_avro_schema_registry},
+  {"bmp_table_peer_buckets", cfg_key_bmp_daemon_table_peer_buckets},
+  {"bmp_table_per_peer_buckets", cfg_key_bmp_daemon_table_per_peer_buckets},
+  {"bmp_table_attr_hash_buckets", cfg_key_bmp_daemon_table_attr_hash_buckets},
+  {"bmp_table_per_peer_hash", cfg_key_bmp_daemon_table_per_peer_hash},
+  {"bmp_dump_output", cfg_key_bmp_daemon_dump_output},
+  {"bmp_dump_file", cfg_key_bmp_daemon_dump_file},
+  {"bmp_dump_latest_file", cfg_key_bmp_daemon_dump_latest_file},
+  {"bmp_dump_workers", cfg_key_bmp_daemon_dump_workers},
+  {"bmp_dump_avro_schema_file", cfg_key_bmp_daemon_dump_avro_schema_file},
+  {"bmp_dump_refresh_time", cfg_key_bmp_daemon_dump_refresh_time},
+  {"bmp_dump_amqp_host", cfg_key_bmp_daemon_dump_amqp_host},
+  {"bmp_dump_amqp_vhost", cfg_key_bmp_daemon_dump_amqp_vhost},
+  {"bmp_dump_amqp_user", cfg_key_bmp_daemon_dump_amqp_user},
+  {"bmp_dump_amqp_passwd", cfg_key_bmp_daemon_dump_amqp_passwd},
+  {"bmp_dump_amqp_exchange", cfg_key_bmp_daemon_dump_amqp_exchange},
+  {"bmp_dump_amqp_exchange_type", cfg_key_bmp_daemon_dump_amqp_exchange_type},
+  {"bmp_dump_amqp_routing_key", cfg_key_bmp_daemon_dump_amqp_routing_key},
+  {"bmp_dump_amqp_routing_key_rr", cfg_key_bmp_daemon_dump_amqp_routing_key_rr},
+  {"bmp_dump_amqp_persistent_msg", cfg_key_bmp_daemon_dump_amqp_persistent_msg},
+  {"bmp_dump_amqp_frame_max", cfg_key_bmp_daemon_dump_amqp_frame_max},
+  {"bmp_dump_amqp_heartbeat_interval", cfg_key_bmp_daemon_dump_amqp_heartbeat_interval},
+  {"bmp_dump_kafka_broker_host", cfg_key_bmp_daemon_dump_kafka_broker_host},
+  {"bmp_dump_kafka_broker_port", cfg_key_bmp_daemon_dump_kafka_broker_port},
+  {"bmp_dump_kafka_topic", cfg_key_bmp_daemon_dump_kafka_topic},
+  {"bmp_dump_kafka_topic_rr", cfg_key_bmp_daemon_dump_kafka_topic_rr},
+  {"bmp_dump_kafka_partition", cfg_key_bmp_daemon_dump_kafka_partition},
+  {"bmp_dump_kafka_partition_key", cfg_key_bmp_daemon_dump_kafka_partition_key},
+  {"bmp_dump_kafka_config_file", cfg_key_bmp_daemon_dump_kafka_config_file},
+  {"bmp_dump_kafka_avro_schema_registry", cfg_key_bmp_daemon_dump_kafka_avro_schema_registry},
+  {"bmp_daemon_parse_proxy_header", cfg_key_nfacctd_bmp_daemon_parse_proxy_header},
   {"rpki_roas_file", cfg_key_rpki_roas_file},
   {"rpki_rtr_cache", cfg_key_rpki_rtr_cache},
   {"rpki_rtr_cache_version", cfg_key_rpki_rtr_cache_version},
@@ -603,6 +626,8 @@ static const struct _dictionary_line dictionary[] = {
   {"tunnel_0", cfg_key_tunnel_0},
   {"tmp_asa_bi_flow", cfg_key_tmp_asa_bi_flow},
   {"tmp_bgp_lookup_compare_ports", cfg_key_tmp_bgp_lookup_compare_ports},
+  {"tmp_bgp_daemon_route_refresh", cfg_key_tmp_bgp_daemon_route_refresh},
+  {"tmp_bgp_daemon_origin_type_int", cfg_key_tmp_bgp_daemon_origin_type_int},
   {"", NULL}
 };
 
@@ -622,8 +647,8 @@ static struct plugin_type_entry plugin_types_list[] = {
   {PLUGIN_ID_SQLITE3,	"sqlite3",	sqlite3_plugin},
 #endif
 #ifdef WITH_MONGODB
-  {PLUGIN_ID_UNKNOWN,	"mongodb",		mongodb_legacy_warning},
-  {PLUGIN_ID_MONGODB,  	"mongodb_legacy",	mongodb_plugin},
+  {PLUGIN_ID_UNKNOWN,	"mongodb",		mongodb_legacy_warning}, /* Legacy plugin */
+  {PLUGIN_ID_MONGODB,  	"mongodb_legacy",	mongodb_plugin}, /* Legacy plugin */
 #endif
 #ifdef WITH_RABBITMQ
   {PLUGIN_ID_AMQP,	"amqp",		amqp_plugin},
@@ -700,10 +725,11 @@ void evaluate_configuration(char *filename, int rows)
    plugin structures and parses supported config keys */
 int parse_configuration_file(char *filename)
 {
+  struct stat st;
   char localbuf[10240];
   char cmdline [] = "cmdline"; 
   FILE *file;
-  int num = 0, cmdlineflag = FALSE, rows_cmdline = rows, idx;
+  int num = 0, cmdlineflag = FALSE, rows_cmdline = rows, idx, ret;
   rows = 0;
 
   /* NULL filename means we don't have a configuration file; 1st stage: read from
@@ -711,11 +737,19 @@ int parse_configuration_file(char *filename)
      required, placing them at the tail - in order to override directives placed
      in the configuration file */
   if (filename) { 
-    if ((file = fopen(filename,"r")) == NULL) {
+    ret = stat(filename, &st);
+    if (ret < 0) {
       Log(LOG_ERR, "ERROR: [%s] file not found.\n", filename);
       return ERR;
     }
     else {
+      if (!S_ISREG(st.st_mode)) {
+	Log(LOG_ERR, "ERROR: [%s] path is not a regular file.\n", filename);
+	return ERR;
+      }
+    }
+
+    if ((file = fopen(filename, "r"))) {
       while (!feof(file)) {
         if (rows == LARGEBUFLEN) {
 	  Log(LOG_ERR, "ERROR: [%s] maximum number of %d lines reached.\n", filename, LARGEBUFLEN);
