@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -81,13 +81,19 @@ struct configuration {
   pm_cfgreg_t nfprobe_what_to_count_2;
   char *aggregate_primitives;
   struct custom_primitives_ptrs cpptrs;
+  char *progname;
   char *name;
   char *type;
   int type_id;
   int is_forked;
+  int propagate_signals;
   int pmacctd_nonroot;
   char *proc_name;
   int proc_priority;
+  char *cluster_name;
+  int cluster_id;
+  char *redis_host;
+  int redis_db;
   int sock;
   int bgp_sock;
   int acct_type; 
@@ -114,6 +120,10 @@ struct configuration {
   int flow_tcp_lifetime;
   int num_protos;
   int num_hosts;
+  char *dtls_path;
+#ifdef WITH_GNUTLS
+  pm_dtls_glob_t dtls_globs;
+#endif
   char *imt_plugin_path;
   char *imt_plugin_passwd;
   char *sql_db;
@@ -185,16 +195,24 @@ struct configuration {
   char *print_latest_file;
   int nfacctd_port;
   char *nfacctd_ip;
+  int nfacctd_ipv6_only;
   char *nfacctd_kafka_broker_host;
   int nfacctd_kafka_broker_port;
   char *nfacctd_kafka_topic;
   char *nfacctd_kafka_config_file;
   char *nfacctd_zmq_address;
+  int nfacctd_dtls_port;
+#ifdef WITH_GNUTLS
+  int nfacctd_dtls_sock;
+#endif
   char *nfacctd_allow_file;
   int nfacctd_time;
   int nfacctd_time_new;
   int nfacctd_pro_rating;
   char *nfacctd_templates_file;
+  char *nfacctd_templates_receiver;
+  int nfacctd_templates_port;
+  int nfacctd_templates_sock;
   int nfacctd_account_options;
   int nfacctd_stitching;
   u_int32_t nfacctd_as;
@@ -230,7 +248,15 @@ struct configuration {
   int telemetry_port_tcp;
   int telemetry_port_udp;
   char *telemetry_ip;
+  int telemetry_udp_notif_port;
+  char *telemetry_udp_notif_ip;
+  int telemetry_udp_notif_nmsgs;
+  int telemetry_ipv6_only;
   char *telemetry_zmq_address;
+  char *telemetry_kafka_broker_host;
+  int telemetry_kafka_broker_port;
+  char *telemetry_kafka_topic;
+  char *telemetry_kafka_config_file;
   char *telemetry_decoder;
   int telemetry_decoder_id;
   int telemetry_max_peers;
@@ -267,6 +293,7 @@ struct configuration {
   int telemetry_dump_amqp_persistent_msg;
   u_int32_t telemetry_dump_amqp_frame_max;
   u_int32_t telemetry_dump_amqp_heartbeat_interval;
+  int telemetry_dump_workers;
   char *telemetry_msglog_kafka_broker_host;
   int telemetry_msglog_kafka_broker_port;
   char *telemetry_msglog_kafka_topic;
@@ -284,68 +311,69 @@ struct configuration {
   char *telemetry_dump_kafka_partition_key;
   int telemetry_dump_kafka_partition_keylen;
   char *telemetry_dump_kafka_config_file;
-  int nfacctd_bgp;
-  int nfacctd_bgp_msglog_output;
-  char *nfacctd_bgp_msglog_file;
-  char *nfacctd_bgp_msglog_avro_schema_file;
-  char *nfacctd_bgp_msglog_amqp_host;
-  char *nfacctd_bgp_msglog_amqp_vhost;
-  char *nfacctd_bgp_msglog_amqp_user;
-  char *nfacctd_bgp_msglog_amqp_passwd;
-  char *nfacctd_bgp_msglog_amqp_exchange;
-  char *nfacctd_bgp_msglog_amqp_exchange_type;
-  char *nfacctd_bgp_msglog_amqp_routing_key;
-  int nfacctd_bgp_msglog_amqp_routing_key_rr;
-  int nfacctd_bgp_msglog_amqp_persistent_msg;
-  u_int32_t nfacctd_bgp_msglog_amqp_frame_max;
-  u_int32_t nfacctd_bgp_msglog_amqp_heartbeat_interval;
-  int nfacctd_bgp_msglog_amqp_retry;
-  char *nfacctd_bgp_msglog_kafka_broker_host;
-  char *nfacctd_bgp_msglog_kafka_topic;
-  int nfacctd_bgp_msglog_kafka_topic_rr;
-  int nfacctd_bgp_msglog_kafka_partition;
-  char *nfacctd_bgp_msglog_kafka_partition_key;
-  int nfacctd_bgp_msglog_kafka_partition_keylen;
-  int nfacctd_bgp_msglog_kafka_broker_port;
-  int nfacctd_bgp_msglog_kafka_retry;
-  char *nfacctd_bgp_msglog_kafka_config_file;
-  char *nfacctd_bgp_msglog_kafka_avro_schema_registry;
-  char *nfacctd_bgp_id;
-  char *nfacctd_bgp_ip;
-  as_t nfacctd_bgp_as;
-  int nfacctd_bgp_port;
-  int nfacctd_bgp_pipe_size;
-  int nfacctd_bgp_ipprec;
-  char *nfacctd_bgp_allow_file;
-  int nfacctd_bgp_max_peers;
-  int nfacctd_bgp_aspath_radius;
-  char *nfacctd_bgp_stdcomm_pattern;
-  char *nfacctd_bgp_extcomm_pattern;
-  char *nfacctd_bgp_lrgcomm_pattern;
-  char *nfacctd_bgp_stdcomm_pattern_to_asn;
-  char *nfacctd_bgp_lrgcomm_pattern_to_asn;
+  int bgp_daemon;
+  int bgp_daemon_msglog_output;
+  char *bgp_daemon_msglog_file;
+  char *bgp_daemon_msglog_avro_schema_file;
+  char *bgp_daemon_msglog_amqp_host;
+  char *bgp_daemon_msglog_amqp_vhost;
+  char *bgp_daemon_msglog_amqp_user;
+  char *bgp_daemon_msglog_amqp_passwd;
+  char *bgp_daemon_msglog_amqp_exchange;
+  char *bgp_daemon_msglog_amqp_exchange_type;
+  char *bgp_daemon_msglog_amqp_routing_key;
+  int bgp_daemon_msglog_amqp_routing_key_rr;
+  int bgp_daemon_msglog_amqp_persistent_msg;
+  u_int32_t bgp_daemon_msglog_amqp_frame_max;
+  u_int32_t bgp_daemon_msglog_amqp_heartbeat_interval;
+  int bgp_daemon_msglog_amqp_retry;
+  char *bgp_daemon_msglog_kafka_broker_host;
+  char *bgp_daemon_msglog_kafka_topic;
+  int bgp_daemon_msglog_kafka_topic_rr;
+  int bgp_daemon_msglog_kafka_partition;
+  char *bgp_daemon_msglog_kafka_partition_key;
+  int bgp_daemon_msglog_kafka_partition_keylen;
+  int bgp_daemon_msglog_kafka_broker_port;
+  int bgp_daemon_msglog_kafka_retry;
+  char *bgp_daemon_msglog_kafka_config_file;
+  char *bgp_daemon_msglog_kafka_avro_schema_registry;
+  char *bgp_daemon_id;
+  char *bgp_daemon_ip;
+  int bgp_daemon_ipv6_only;
+  as_t bgp_daemon_as;
+  int bgp_daemon_port;
+  int bgp_daemon_pipe_size;
+  int bgp_daemon_ipprec;
+  char *bgp_daemon_allow_file;
+  int bgp_daemon_max_peers;
+  int bgp_daemon_aspath_radius;
+  char *bgp_daemon_stdcomm_pattern;
+  char *bgp_daemon_extcomm_pattern;
+  char *bgp_daemon_lrgcomm_pattern;
+  char *bgp_daemon_stdcomm_pattern_to_asn;
+  char *bgp_daemon_lrgcomm_pattern_to_asn;
   char *bgp_blackhole_stdcomm_list;
-  int nfacctd_bgp_peer_as_src_type;
-  int nfacctd_bgp_src_std_comm_type;
-  int nfacctd_bgp_src_ext_comm_type;
-  int nfacctd_bgp_src_lrg_comm_type;
-  int nfacctd_bgp_src_as_path_type;
-  int nfacctd_bgp_src_local_pref_type;
-  int nfacctd_bgp_src_med_type;
-  int nfacctd_bgp_src_roa_type;
-  int nfacctd_bgp_peer_as_skip_subas;
-  int nfacctd_bgp_batch;
-  int nfacctd_bgp_batch_interval;
-  char *nfacctd_bgp_peer_as_src_map;
-  char *nfacctd_bgp_src_local_pref_map;
-  char *nfacctd_bgp_src_med_map;
-  char *nfacctd_bgp_to_agent_map;
+  int bgp_daemon_peer_as_src_type;
+  int bgp_daemon_src_std_comm_type;
+  int bgp_daemon_src_ext_comm_type;
+  int bgp_daemon_src_lrg_comm_type;
+  int bgp_daemon_src_as_path_type;
+  int bgp_daemon_src_local_pref_type;
+  int bgp_daemon_src_med_type;
+  int bgp_daemon_src_roa_type;
+  int bgp_daemon_peer_as_skip_subas;
+  int bgp_daemon_batch;
+  int bgp_daemon_batch_interval;
+  char *bgp_daemon_peer_as_src_map;
+  char *bgp_daemon_src_local_pref_map;
+  char *bgp_daemon_src_med_map;
+  char *bgp_daemon_to_xflow_agent_map;
   char *nfacctd_flow_to_rd_map;
-  int nfacctd_bgp_follow_default;
-  struct prefix nfacctd_bgp_follow_nexthop[FOLLOW_BGP_NH_ENTRIES];
-  int nfacctd_bgp_follow_nexthop_external;
-  char *nfacctd_bgp_neighbors_file;
-  char *nfacctd_bgp_md5_file;
+  int bgp_daemon_follow_default;
+  struct prefix bgp_daemon_follow_nexthop[FOLLOW_BGP_NH_ENTRIES];
+  int bgp_daemon_follow_nexthop_external;
+  char *bgp_daemon_neighbors_file;
+  char *bgp_daemon_md5_file;
   int bgp_table_peer_buckets;
   int bgp_table_per_peer_buckets;
   int bgp_table_attr_hash_buckets;
@@ -375,6 +403,7 @@ struct configuration {
   int bgp_table_dump_kafka_broker_port;
   char *bgp_table_dump_kafka_config_file;
   char *bgp_table_dump_kafka_avro_schema_registry;
+  int bgp_table_dump_workers;
   int bgp_lg;
   char *bgp_lg_ip;
   int bgp_lg_port;
@@ -384,45 +413,47 @@ struct configuration {
   char *bgp_xconnect_map;
   int bgp_disable_router_id_check;
   int bmp_sock;
-  int nfacctd_bmp;
-  char *nfacctd_bmp_ip;
-  int nfacctd_bmp_port;
-  int nfacctd_bmp_pipe_size;
-  int nfacctd_bmp_max_peers;
-  char *nfacctd_bmp_allow_file;
-  int nfacctd_bmp_ipprec;
-  int nfacctd_bmp_batch;
-  int nfacctd_bmp_batch_interval;
-  int nfacctd_bmp_msglog_output;
-  char *nfacctd_bmp_msglog_file;
-  char *nfacctd_bmp_msglog_avro_schema_file;
-  char *nfacctd_bmp_msglog_amqp_host;
-  char *nfacctd_bmp_msglog_amqp_vhost;
-  char *nfacctd_bmp_msglog_amqp_user;
-  char *nfacctd_bmp_msglog_amqp_passwd;
-  char *nfacctd_bmp_msglog_amqp_exchange;
-  char *nfacctd_bmp_msglog_amqp_exchange_type;
-  char *nfacctd_bmp_msglog_amqp_routing_key;
-  int nfacctd_bmp_msglog_amqp_routing_key_rr;
-  int nfacctd_bmp_msglog_amqp_persistent_msg;
-  u_int32_t nfacctd_bmp_msglog_amqp_frame_max;
-  u_int32_t nfacctd_bmp_msglog_amqp_heartbeat_interval;
-  int nfacctd_bmp_msglog_amqp_retry;
-  char *nfacctd_bmp_msglog_kafka_broker_host;
-  char *nfacctd_bmp_msglog_kafka_topic;
-  int nfacctd_bmp_msglog_kafka_topic_rr;
-  int nfacctd_bmp_msglog_kafka_partition;
-  char *nfacctd_bmp_msglog_kafka_partition_key;
-  int nfacctd_bmp_msglog_kafka_partition_keylen;
-  int nfacctd_bmp_msglog_kafka_broker_port;
-  int nfacctd_bmp_msglog_kafka_retry;
-  char *nfacctd_bmp_msglog_kafka_config_file;
-  char *nfacctd_bmp_msglog_kafka_avro_schema_registry;
+  int bmp_daemon;
+  char *bmp_daemon_ip;
+  int bmp_daemon_ipv6_only;
+  int bmp_daemon_port;
+  int bmp_daemon_pipe_size;
+  int bmp_daemon_max_peers;
+  char *bmp_daemon_allow_file;
+  int bmp_daemon_ipprec;
+  int bmp_daemon_batch;
+  int bmp_daemon_batch_interval;
+  int bmp_daemon_msglog_output;
+  char *bmp_daemon_msglog_file;
+  char *bmp_daemon_msglog_avro_schema_file;
+  char *bmp_daemon_msglog_amqp_host;
+  char *bmp_daemon_msglog_amqp_vhost;
+  char *bmp_daemon_msglog_amqp_user;
+  char *bmp_daemon_msglog_amqp_passwd;
+  char *bmp_daemon_msglog_amqp_exchange;
+  char *bmp_daemon_msglog_amqp_exchange_type;
+  char *bmp_daemon_msglog_amqp_routing_key;
+  int bmp_daemon_msglog_amqp_routing_key_rr;
+  int bmp_daemon_msglog_amqp_persistent_msg;
+  u_int32_t bmp_daemon_msglog_amqp_frame_max;
+  u_int32_t bmp_daemon_msglog_amqp_heartbeat_interval;
+  int bmp_daemon_msglog_amqp_retry;
+  char *bmp_daemon_msglog_kafka_broker_host;
+  char *bmp_daemon_msglog_kafka_topic;
+  int bmp_daemon_msglog_kafka_topic_rr;
+  int bmp_daemon_msglog_kafka_partition;
+  char *bmp_daemon_msglog_kafka_partition_key;
+  int bmp_daemon_msglog_kafka_partition_keylen;
+  int bmp_daemon_msglog_kafka_broker_port;
+  int bmp_daemon_msglog_kafka_retry;
+  char *bmp_daemon_msglog_kafka_config_file;
+  char *bmp_daemon_msglog_kafka_avro_schema_registry;
   int bmp_table_peer_buckets;
   int bmp_table_per_peer_buckets;
   int bmp_table_attr_hash_buckets;
   int bmp_table_per_peer_hash;
   int bmp_dump_output;
+  int bmp_dump_workers;
   char *bmp_dump_file;
   char *bmp_dump_latest_file;
   char *bmp_dump_avro_schema_file;
@@ -534,6 +565,8 @@ struct configuration {
   int nfprobe_hoplimit;
   int nfprobe_maxflows;
   char *nfprobe_receiver;
+  int nfprobe_dtls;
+  char *nfprobe_dtls_verify_cert;
   int nfprobe_version;
   char *nfprobe_engine;
   int nfprobe_peer_as;
@@ -545,6 +578,7 @@ struct configuration {
   int nfprobe_ifindex_override;
   int nfprobe_ifindex_type;
   int nfprobe_dont_cache;
+  int nfprobe_tstamp_usec;
   char *sfprobe_receiver;
   char *sfprobe_agentip;
   int sfprobe_agentsubid;
@@ -560,16 +594,20 @@ struct configuration {
   int uacctd_threshold;
   char *tunnel0;
   int use_ip_next_hop;
-  int decode_arista_trailer;
+  int pcap_arista_trailer_offset;
+  int pcap_arista_trailer_flag_value;
   int dump_max_writers;
   int tmp_asa_bi_flow;
   int tmp_bgp_lookup_compare_ports;
+  int tmp_bgp_daemon_route_refresh;
+  int tmp_bgp_daemon_origin_type_int;
   size_t thread_stack;
   char *rpki_roas_file;
   char *rpki_rtr_cache;
   int rpki_rtr_cache_version;
   int rpki_rtr_cache_pipe_size;
   int rpki_rtr_cache_ipprec;
+  int bmp_daemon_parse_proxy_header;
 };
 
 /* prototypes */ 
