@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -34,12 +34,18 @@ extern int cfg_key_syslog(char *, char *, char *);
 extern int cfg_key_logfile(char *, char *, char *);
 extern int cfg_key_pidfile(char *, char *, char *);
 extern int cfg_key_daemonize(char *, char *, char *);
+extern int cfg_key_propagate_signals(char *, char *, char *);
 extern int cfg_key_proc_name(char *, char *, char *);
 extern int cfg_key_proc_priority(char *, char *, char *);
+extern int cfg_key_cluster_name(char *, char *, char *);
+extern int cfg_key_cluster_id(char *, char *, char *);
+extern int cfg_key_redis_host(char *, char *, char *);
+extern int cfg_key_redis_db(char *, char *, char *);
 extern int cfg_key_aggregate(char *, char *, char *);
 extern int cfg_key_aggregate_primitives(char *, char *, char *);
 extern int cfg_key_snaplen(char *, char *, char *);
 extern int cfg_key_aggregate_filter(char *, char *, char *);
+extern int cfg_key_dtls_path(char *, char *, char *);
 extern int cfg_key_pcap_filter(char *, char *, char *);
 extern int cfg_key_pcap_protocol(char *, char *, char *);
 extern int cfg_key_pcap_savefile(char *, char *, char *);
@@ -49,8 +55,9 @@ extern int cfg_key_pcap_savefile_replay(char *, char *, char *);
 extern int cfg_key_pcap_direction(char *, char *, char *);
 extern int cfg_key_pcap_ifindex(char *, char *, char *);
 extern int cfg_key_pcap_interfaces_map(char *, char *, char *);
+extern int cfg_key_pcap_arista_trailer_offset(char *, char *, char *);
+extern int cfg_key_pcap_arista_trailer_flag_value(char *, char *, char *);
 extern int cfg_key_use_ip_next_hop(char *, char *, char *);
-extern int cfg_key_decode_arista_trailer(char *, char *, char *);
 extern int cfg_key_thread_stack(char *, char *, char *);
 extern int cfg_key_pcap_interface(char *, char *, char *);
 extern int cfg_key_pcap_interface_wait(char *, char *, char *);
@@ -144,6 +151,7 @@ extern int cfg_key_print_output_custom_cfg_file(char *, char *, char *);
 extern int cfg_key_print_latest_file(char *, char *, char *);
 extern int cfg_key_nfacctd_port(char *, char *, char *);
 extern int cfg_key_nfacctd_ip(char *, char *, char *);
+extern int cfg_key_nfacctd_ipv6_only(char *, char *, char *);
 extern int cfg_key_nfacctd_allow_file(char *, char *, char *);
 extern int cfg_key_nfacctd_time_secs(char *, char *, char *);
 extern int cfg_key_nfacctd_time_new(char *, char *, char *);
@@ -155,6 +163,8 @@ extern int cfg_key_nfacctd_mcast_groups(char *, char *, char *);
 extern int cfg_key_nfacctd_pipe_size(char *, char *, char *);
 extern int cfg_key_nfacctd_pro_rating(char *, char *, char *);
 extern int cfg_key_nfacctd_templates_file(char *, char *, char *);
+extern int cfg_key_nfacctd_templates_receiver(char *, char *, char *);
+extern int cfg_key_nfacctd_templates_port(char *, char *, char *);
 extern int cfg_key_nfacctd_account_options(char *, char *, char *);
 extern int cfg_key_nfacctd_stitching(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_broker_host(char *, char *, char *);
@@ -162,6 +172,7 @@ extern int cfg_key_nfacctd_kafka_broker_port(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_topic(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_config_file(char *, char *, char *);
 extern int cfg_key_nfacctd_zmq_address(char *, char *, char *);
+extern int cfg_key_nfacctd_dtls_port(char *, char *, char *);
 extern int cfg_key_pmacctd_force_frag_handling(char *, char *, char *);
 extern int cfg_key_pmacctd_frag_buffer_size(char *, char *, char *);
 extern int cfg_key_pmacctd_flow_buffer_size(char *, char *, char *);
@@ -196,7 +207,15 @@ extern int cfg_key_telemetry_daemon(char *, char *, char *);
 extern int cfg_key_telemetry_port_tcp(char *, char *, char *);
 extern int cfg_key_telemetry_port_udp(char *, char *, char *);
 extern int cfg_key_telemetry_ip(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_port(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_ip(char *, char *, char *);
+extern int cfg_key_telemetry_udp_notif_nmsgs(char *, char *, char *);
+extern int cfg_key_telemetry_ipv6_only(char *, char *, char *);
 extern int cfg_key_telemetry_zmq_address(char *, char *, char *);
+extern int cfg_key_telemetry_kafka_broker_host(char *, char *, char *);
+extern int cfg_key_telemetry_kafka_broker_port(char *, char *, char *);
+extern int cfg_key_telemetry_kafka_topic(char *, char *, char *);
+extern int cfg_key_telemetry_kafka_config_file(char *, char *, char *);
 extern int cfg_key_telemetry_decoder(char *, char *, char *);
 extern int cfg_key_telemetry_max_peers(char *, char *, char *);
 extern int cfg_key_telemetry_peer_timeout(char *, char *, char *);
@@ -247,6 +266,7 @@ extern int cfg_key_telemetry_dump_kafka_topic_rr(char *, char *, char *);
 extern int cfg_key_telemetry_dump_kafka_partition(char *, char *, char *);
 extern int cfg_key_telemetry_dump_kafka_partition_key(char *, char *, char *);
 extern int cfg_key_telemetry_dump_kafka_config_file(char *, char *, char *);
+extern int cfg_key_telemetry_dump_workers(char *, char *, char *);
 extern int cfg_key_maps_refresh(char *, char *, char *);
 extern int cfg_key_maps_index(char *, char *, char *);
 extern int cfg_key_maps_entries(char *, char *, char *);
@@ -275,6 +295,8 @@ extern int cfg_key_nfprobe_timeouts(char *, char *, char *);
 extern int cfg_key_nfprobe_hoplimit(char *, char *, char *);
 extern int cfg_key_nfprobe_maxflows(char *, char *, char *);
 extern int cfg_key_nfprobe_receiver(char *, char *, char *);
+extern int cfg_key_nfprobe_dtls(char *, char *, char *);
+extern int cfg_key_nfprobe_dtls_verify_cert(char *, char *, char *);
 extern int cfg_key_nfprobe_version(char *, char *, char *);
 extern int cfg_key_nfprobe_engine(char *, char *, char *);
 extern int cfg_key_nfprobe_peer_as(char *, char *, char *);
@@ -283,6 +305,7 @@ extern int cfg_key_nfprobe_ip_precedence(char *, char *, char *);
 extern int cfg_key_nfprobe_direction(char *, char *, char *);
 extern int cfg_key_nfprobe_ifindex(char *, char *, char *);
 extern int cfg_key_nfprobe_ifindex_override(char *, char *, char *);
+extern int cfg_key_nfprobe_tstamp_usec(char *, char *, char *);
 extern int cfg_key_nfprobe_dont_cache(char *, char *, char *);
 extern int cfg_key_sfprobe_receiver(char *, char *, char *);
 extern int cfg_key_sfprobe_agentip(char *, char *, char *);
@@ -294,95 +317,97 @@ extern int cfg_key_tee_max_receivers(char *, char *, char *);
 extern int cfg_key_tee_max_receiver_pools(char *, char *, char *);
 extern int cfg_key_tee_pipe_size(char *, char *, char *);
 extern int cfg_key_tee_kafka_config_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_output(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_avro_schema_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_vhost(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_user(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_passwd(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_exchange(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_exchange_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_routing_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_routing_key_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_persistent_msg(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_frame_max(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_heartbeat_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_amqp_retry(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_broker_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_broker_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_topic(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_topic_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_partition(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_partition_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_retry(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_config_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_msglog_kafka_avro_schema_registry(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_max_peers(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_ip(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_id(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_as(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_ip_precedence(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_allow_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_aspath_radius(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_stdcomm_pattern(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_extcomm_pattern(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_lrgcomm_pattern(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_stdcomm_pattern_to_asn(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_lrgcomm_pattern_to_asn(char *, char *, char *);
+extern int cfg_key_bgp_daemon(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_output(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_avro_schema_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_host(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_vhost(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_user(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_passwd(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_exchange(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_exchange_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_routing_key(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_routing_key_rr(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_persistent_msg(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_frame_max(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_heartbeat_interval(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_amqp_retry(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_broker_host(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_broker_port(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_topic(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_topic_rr(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_partition(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_partition_key(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_retry(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_config_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_msglog_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_bgp_daemon_max_peers(char *, char *, char *);
+extern int cfg_key_bgp_daemon_ip(char *, char *, char *);
+extern int cfg_key_bgp_daemon_ipv6_only(char *, char *, char *);
+extern int cfg_key_bgp_daemon_id(char *, char *, char *);
+extern int cfg_key_bgp_daemon_as(char *, char *, char *);
+extern int cfg_key_bgp_daemon_port(char *, char *, char *);
+extern int cfg_key_bgp_daemon_ip_precedence(char *, char *, char *);
+extern int cfg_key_bgp_daemon_allow_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_aspath_radius(char *, char *, char *);
+extern int cfg_key_bgp_daemon_stdcomm_pattern(char *, char *, char *);
+extern int cfg_key_bgp_daemon_extcomm_pattern(char *, char *, char *);
+extern int cfg_key_bgp_daemon_lrgcomm_pattern(char *, char *, char *);
+extern int cfg_key_bgp_daemon_stdcomm_pattern_to_asn(char *, char *, char *);
+extern int cfg_key_bgp_daemon_lrgcomm_pattern_to_asn(char *, char *, char *);
 extern int cfg_key_bgp_blackhole_stdcomm_list(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_peer_src_as_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_std_comm_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_ext_comm_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_lrg_comm_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_as_path_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_local_pref_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_med_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_roa_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_peer_as_skip_subas(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_peer_src_as_map(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_local_pref_map(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_src_med_map(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_to_agent_map(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_follow_default(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_follow_nexthop(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_follow_nexthop_external(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_disable_router_id_check(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_neighbors_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_md5_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_peer_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_per_peer_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_attr_hash_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_per_peer_hash(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_output(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_latest_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_avro_schema_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_refresh_time(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_vhost(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_user(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_passwd(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_exchange(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_exchange_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_routing_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_routing_key_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_persistent_msg(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_frame_max(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_amqp_heartbeat_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_broker_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_broker_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_topic(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_partition(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_partition_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_config_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_table_dump_kafka_avro_schema_registry(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_batch(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_batch_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bgp_pipe_size(char *, char *, char *);
+extern int cfg_key_bgp_daemon_peer_src_as_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_std_comm_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_ext_comm_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_lrg_comm_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_as_path_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_local_pref_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_med_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_roa_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_peer_as_skip_subas(char *, char *, char *);
+extern int cfg_key_bgp_daemon_peer_src_as_map(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_local_pref_map(char *, char *, char *);
+extern int cfg_key_bgp_daemon_src_med_map(char *, char *, char *);
+extern int cfg_key_bgp_daemon_to_xflow_agent_map(char *, char *, char *);
+extern int cfg_key_bgp_daemon_follow_default(char *, char *, char *);
+extern int cfg_key_bgp_daemon_follow_nexthop(char *, char *, char *);
+extern int cfg_key_bgp_daemon_follow_nexthop_external(char *, char *, char *);
+extern int cfg_key_bgp_daemon_disable_router_id_check(char *, char *, char *);
+extern int cfg_key_bgp_daemon_neighbors_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_md5_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_peer_buckets(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_per_peer_buckets(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_attr_hash_buckets(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_per_peer_hash(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_output(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_latest_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_avro_schema_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_refresh_time(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_host(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_vhost(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_user(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_passwd(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_exchange(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_exchange_type(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_routing_key(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_routing_key_rr(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_persistent_msg(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_frame_max(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_amqp_heartbeat_interval(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_broker_host(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_broker_port(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_topic(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_topic_rr(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_partition(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_partition_key(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_config_file(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_bgp_daemon_table_dump_workers(char *, char *, char *);
+extern int cfg_key_bgp_daemon_batch(char *, char *, char *);
+extern int cfg_key_bgp_daemon_batch_interval(char *, char *, char *);
+extern int cfg_key_bgp_daemon_pipe_size(char *, char *, char *);
 extern int cfg_key_bgp_lg(char *, char *, char *);
 extern int cfg_key_bgp_lg_ip(char *, char *, char *);
 extern int cfg_key_bgp_lg_port(char *, char *, char *);
@@ -390,67 +415,70 @@ extern int cfg_key_bgp_lg_threads(char *, char *, char *);
 extern int cfg_key_bgp_lg_user(char *, char *, char *);
 extern int cfg_key_bgp_lg_passwd(char *, char *, char *);
 extern int cfg_key_bgp_xconnect_map(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_ip(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_pipe_size(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_max_peers(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_allow_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_ip_precedence(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_batch(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_batch_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_output(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_avro_schema_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_vhost(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_user(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_passwd(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_exchange(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_exchange_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_routing_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_routing_key_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_persistent_msg(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_frame_max(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_heartbeat_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_amqp_retry(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_broker_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_broker_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_topic(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_topic_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_partition(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_partition_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_retry(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_config_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_msglog_kafka_avro_schema_registry(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_table_peer_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_table_per_peer_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_table_attr_hash_buckets(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_table_per_peer_hash(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_output(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_avro_schema_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_latest_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_refresh_time(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_vhost(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_user(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_passwd(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_exchange(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_exchange_type(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_routing_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_routing_key_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_persistent_msg(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_frame_max(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_amqp_heartbeat_interval(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_broker_host(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_broker_port(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_topic(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_topic_rr(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_partition(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_partition_key(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_config_file(char *, char *, char *);
-extern int cfg_key_nfacctd_bmp_dump_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_bmp_daemon(char *, char *, char *);
+extern int cfg_key_bmp_daemon_ip(char *, char *, char *);
+extern int cfg_key_bmp_daemon_ipv6_only(char *, char *, char *);
+extern int cfg_key_bmp_daemon_port(char *, char *, char *);
+extern int cfg_key_bmp_daemon_pipe_size(char *, char *, char *);
+extern int cfg_key_bmp_daemon_max_peers(char *, char *, char *);
+extern int cfg_key_bmp_daemon_allow_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_ip_precedence(char *, char *, char *);
+extern int cfg_key_bmp_daemon_batch(char *, char *, char *);
+extern int cfg_key_bmp_daemon_batch_interval(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_output(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_avro_schema_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_host(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_vhost(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_user(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_passwd(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_exchange(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_exchange_type(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_routing_key(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_routing_key_rr(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_persistent_msg(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_frame_max(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_heartbeat_interval(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_amqp_retry(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_broker_host(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_broker_port(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_topic(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_topic_rr(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_partition(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_partition_key(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_retry(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_config_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_msglog_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_bmp_daemon_table_peer_buckets(char *, char *, char *);
+extern int cfg_key_bmp_daemon_table_per_peer_buckets(char *, char *, char *);
+extern int cfg_key_bmp_daemon_table_attr_hash_buckets(char *, char *, char *);
+extern int cfg_key_bmp_daemon_table_per_peer_hash(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_output(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_workers(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_avro_schema_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_latest_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_refresh_time(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_host(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_vhost(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_user(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_passwd(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_exchange(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_exchange_type(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_routing_key(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_routing_key_rr(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_persistent_msg(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_frame_max(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_amqp_heartbeat_interval(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_broker_host(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_broker_port(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_topic(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_topic_rr(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_partition(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_partition_key(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_config_file(char *, char *, char *);
+extern int cfg_key_bmp_daemon_dump_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_nfacctd_bmp_daemon_parse_proxy_header(char *, char *, char *);
 extern int cfg_key_nfacctd_flow_to_rd_map(char *, char *, char *);
 extern int cfg_key_nfacctd_isis(char *, char *, char *);
 extern int cfg_key_nfacctd_isis_ip(char *, char *, char *);
@@ -475,6 +503,8 @@ extern int cfg_key_tunnel_0(char *, char *, char *);
 extern int cfg_key_dump_max_writers(char *, char *, char *);
 extern int cfg_key_tmp_asa_bi_flow(char *, char *, char *);
 extern int cfg_key_tmp_bgp_lookup_compare_ports(char *, char *, char *);
+extern int cfg_key_tmp_bgp_daemon_route_refresh(char *, char *, char *);
+extern int cfg_key_tmp_bgp_daemon_origin_type_int(char *, char *, char *);
 
 extern void parse_time(char *, char *, int *, int *);
 extern void cfg_set_aggregate(char *, u_int64_t [], u_int64_t, char *);
