@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,16 +19,22 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __IP_FLOW_C
-
 /* includes */
 #include "pmacct.h"
+#include "addr.h"
 #include "pmacct-data.h"
 #include "plugin_hooks.h"
 #include "ip_flow.h"
 #include "classifier.h"
 #include "jhash.h"
 
+/* Global variables */
+struct ip_flow **ip_flow_table;
+struct flow_lru_l flow_lru_list;
+
+struct ip_flow6 **ip_flow_table6;
+struct flow_lru_l6 flow_lru_list6;
+
 u_int32_t flt_total_nodes;  
 time_t flt_prune_deadline;
 time_t flt_emergency_prune;
@@ -36,18 +42,14 @@ time_t flow_generic_lifetime;
 time_t flow_tcpest_lifetime;
 u_int32_t flt_trivial_hash_rnd = 140281; /* ummmh */
 
-#if defined ENABLE_IPV6
 u_int32_t flt6_total_nodes;
 time_t flt6_prune_deadline;
 time_t flt6_emergency_prune;
-#endif
 
 void init_ip_flow_handler()
 {
   init_ip4_flow_handler();
-#if defined ENABLE_IPV6
   init_ip6_flow_handler();
-#endif
 }
 
 void init_ip4_flow_handler()
@@ -189,7 +191,7 @@ void create_flow(struct timeval *now, struct ip_flow *fp, u_int8_t is_candidate,
 
   if (!flt_total_nodes) {
     if (now->tv_sec > flt_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-      Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer full. Skipping flows.\n", config.name); 
+      Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer full. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname); 
       flt_emergency_prune = now->tv_sec;
       prune_old_flows(now);
     }
@@ -204,7 +206,7 @@ void create_flow(struct timeval *now, struct ip_flow *fp, u_int8_t is_candidate,
       newf = (struct ip_flow *) malloc(sizeof(struct ip_flow));
       if (!newf) { 
 	if (now->tv_sec > flt_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-	  Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer finished memory. Skipping flows.\n", config.name);
+	  Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer finished memory. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname);
 	  flt_emergency_prune = now->tv_sec;
 	  prune_old_flows(now);
 	}
@@ -241,7 +243,7 @@ void create_flow(struct timeval *now, struct ip_flow *fp, u_int8_t is_candidate,
     fp = (struct ip_flow *) malloc(sizeof(struct ip_flow));  
     if (!fp) {
       if (now->tv_sec > flt_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-        Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer finished memory. Skipping flows.\n", config.name);
+        Log(LOG_INFO, "INFO ( %s/core ): Flow/4 buffer finished memory. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname);
         flt_emergency_prune = now->tv_sec;
         prune_old_flows(now);
       }
@@ -393,7 +395,6 @@ unsigned int is_expired_uni(struct timeval *now, struct ip_flow_common *fp, unsi
   return FALSE;
 }
 
-#if defined ENABLE_IPV6
 void init_ip6_flow_handler()
 {
   int size;
@@ -537,7 +538,6 @@ void find_flow6(struct timeval *now, struct packet_ptrs *pptrs)
     last_seen = fp;
   }
 
-  create:
   if (candidate) create_flow6(now, candidate, TRUE, bucket, pptrs, iphp, tlhp, idx);
   else create_flow6(now, last_seen, FALSE, bucket, pptrs, iphp, tlhp, idx);
 }
@@ -549,7 +549,7 @@ void create_flow6(struct timeval *now, struct ip_flow6 *fp, u_int8_t is_candidat
 
   if (!flt6_total_nodes) {
     if (now->tv_sec > flt6_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-      Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows.\n", config.name);
+      Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname);
       flt6_emergency_prune = now->tv_sec;
       prune_old_flows6(now);
     }
@@ -564,7 +564,7 @@ void create_flow6(struct timeval *now, struct ip_flow6 *fp, u_int8_t is_candidat
       newf = (struct ip_flow6 *) malloc(sizeof(struct ip_flow6));
       if (!newf) {
 	if (now->tv_sec > flt6_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-	  Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows.\n", config.name);
+	  Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname);
 	  flt6_emergency_prune = now->tv_sec;
 	  prune_old_flows6(now);
 	}
@@ -600,7 +600,7 @@ void create_flow6(struct timeval *now, struct ip_flow6 *fp, u_int8_t is_candidat
     fp = (struct ip_flow6 *) malloc(sizeof(struct ip_flow6));
     if (!fp) {
       if (now->tv_sec > flt6_emergency_prune+FLOW_TABLE_EMER_PRUNE_INTERVAL) {
-        Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows.\n", config.name);
+        Log(LOG_INFO, "INFO ( %s/core ): Flow/6 buffer full. Skipping flows. Increase %s_flow_buffer_size\n", config.name, config.progname);
         flt6_emergency_prune = now->tv_sec;
         prune_old_flows6(now);
       }
@@ -675,4 +675,3 @@ void prune_old_flows6(struct timeval *now)
 
   flow_lru_list6.last = last_seen;
 }
-#endif
