@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -26,7 +26,6 @@
 #include "isis.h"
 #include "thread_pool.h"
 
-#include "linklist.h"
 #include "stream.h"
 #include "hash.h"
 #include "prefix.h"
@@ -68,11 +67,11 @@ void nfacctd_isis_wrapper()
   send_to_pool(isis_pool, skinny_isis_daemon, NULL);
 }
 
-void skinny_isis_daemon()
+int skinny_isis_daemon()
 {
   char errbuf[PCAP_ERRBUF_SIZE];
-  struct pcap_device device;
-  struct pcap_isis_callback_data cb_data;
+  struct pm_pcap_device device;
+  struct pm_pcap_isis_callback_data cb_data;
   struct host_addr addr;
   struct prefix_ipv4 *ipv4;
   struct plugin_requests req;
@@ -83,7 +82,7 @@ void skinny_isis_daemon()
   struct isis_circuit *circuit;
   struct interface interface;
 
-  memset(&device, 0, sizeof(struct pcap_device));
+  memset(&device, 0, sizeof(struct pm_pcap_device));
   memset(&cb_data, 0, sizeof(cb_data));
   memset(&interface, 0, sizeof(interface));
   memset(&isis_spf_deadline, 0, sizeof(isis_spf_deadline));
@@ -124,7 +123,7 @@ void skinny_isis_daemon()
 
     if (device.data == NULL) {
       Log(LOG_ERR, "ERROR ( %s/core/ISIS ): data link not supported: %d\n", config.name, device.link_type);
-      return;
+      return ERR;
     }
     else {
       Log(LOG_INFO, "OK ( %s/core/ISIS ): link type is: %d\n", config.name, device.link_type);
@@ -145,7 +144,7 @@ void skinny_isis_daemon()
   area->area_tag = area_tag;
   area->is_type = IS_LEVEL_2;
   area->newmetric = TRUE;
-  isis_listnode_add(isis->area_list, area);
+  pm_listnode_add(isis->area_list, area);
   Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): New IS-IS area instance %s\n", config.name, area->area_tag);
   if (config.nfacctd_isis_net) area_net_title(area, config.nfacctd_isis_net);
   else {
@@ -183,8 +182,8 @@ void skinny_isis_daemon()
   ipv4 = isis_prefix_ipv4_new();
   ipv4->prefixlen = 32;
   ipv4->prefix.s_addr = addr.address.ipv4.s_addr;
-  circuit->ip_addrs = isis_list_new();
-  isis_listnode_add(circuit->ip_addrs, ipv4);
+  circuit->ip_addrs = pm_list_new();
+  pm_listnode_add(circuit->ip_addrs, ipv4);
 
   circuit_update_nlpids(circuit);
   isis_circuit_configure(circuit, area);
@@ -225,18 +224,20 @@ void skinny_isis_daemon()
       }
     }
   }
+
+  return SUCCESS;
 }
 
 void isis_pdu_runner(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
 {
-  struct pcap_isis_callback_data *cb_data = (struct pcap_isis_callback_data *) user;
-  struct pcap_device *device = cb_data->device;
+  struct pm_pcap_isis_callback_data *cb_data = (struct pm_pcap_isis_callback_data *) user;
+  struct pm_pcap_device *device = cb_data->device;
   struct isis_circuit *circuit = cb_data->circuit;
   struct packet_ptrs pptrs;
   int ret;
 
   struct stream stm;
-  char *ssnpa;
+  u_char *ssnpa;
 
   /* Let's export a time reference */
   memcpy(&isis_now, &pkthdr->ts, sizeof(struct timeval));
@@ -623,8 +624,8 @@ void igp_daemon_map_validate(char *filename, struct plugin_requests *req)
 
   if (entry) {
     if (entry->node.family && entry->area_id && (entry->adj_metric_num || entry->reach_metric_num || entry->reach6_metric_num)) {
-      char isis_dgram[RECEIVE_LSP_BUFFER_SIZE+sizeof(struct chdlc_header)+sizeof(struct isis_fixed_hdr)];
-      char *isis_dgram_ptr = isis_dgram;
+      u_char isis_dgram[RECEIVE_LSP_BUFFER_SIZE+sizeof(struct chdlc_header)+sizeof(struct isis_fixed_hdr)];
+      u_char *isis_dgram_ptr = isis_dgram;
       struct chdlc_header *chdlc_hdr;
       struct isis_fixed_hdr *isis_hdr;
       struct isis_link_state_hdr *lsp_hdr;
@@ -790,7 +791,7 @@ void igp_daemon_map_validate(char *filename, struct plugin_requests *req)
       if (config.debug && config.igp_daemon_map_msglog) {
 	memset(&phdr, 0, sizeof(phdr));
 	phdr.len = phdr.caplen = sizeof(isis_dgram)-rem_len;
-	pcap_dump((char *) idmm_fd, &phdr, isis_dgram);
+	pcap_dump((u_char *) idmm_fd, &phdr, isis_dgram);
       }
     }
     else {
@@ -834,7 +835,7 @@ int igp_daemon_map_handle_len(int *rem_len, int len, struct plugin_requests *req
   return FALSE;
 }
 
-int igp_daemon_map_handle_lsp_id(char *lsp_id, struct host_addr *addr)
+int igp_daemon_map_handle_lsp_id(u_char *lsp_id, struct host_addr *addr)
 {
   u_char sysid[ISIS_SYS_ID_LEN];
   int idx;
