@@ -24,7 +24,6 @@
 #include "pmacct.h"
 #include "isis.h"
 
-#include "linklist.h"
 #include "prefix.h"
 #include "hash.h"
 #include "table.h"
@@ -123,17 +122,17 @@ vid2string (struct isis_vertex *vertex, u_char * buff)
 
 /* 7.2.7 */
 static void
-remove_excess_adjs (struct list *adjs)
+remove_excess_adjs (struct pm_list *adjs)
 {
-  struct listnode *node, *excess = NULL;
+  struct pm_listnode *node, *excess = NULL;
   struct isis_adjacency *adj, *candidate = NULL;
   int comp;
 
-  for (ALL_LIST_ELEMENTS_RO (adjs, node, adj)) 
+  for (PM_ALL_LIST_ELEMENTS_RO (adjs, node, adj)) 
     {
       if (excess == NULL)
 	excess = node;
-      candidate = listgetdata (excess);
+      candidate = pm_listgetdata (excess);
 
       if (candidate->sys_type < adj->sys_type)
 	{
@@ -173,7 +172,7 @@ remove_excess_adjs (struct list *adjs)
 	}
     }
 
-  isis_list_delete_node (adjs, excess);
+  pm_list_delete_node (adjs, excess);
 
   return;
 }
@@ -190,15 +189,15 @@ isis_spftree_new ()
       return NULL;
     }
 
-  tree->tents = isis_list_new ();
-  tree->paths = isis_list_new ();
+  tree->tents = pm_list_new ();
+  tree->paths = pm_list_new ();
   return tree;
 }
 
 static void
 isis_vertex_del (struct isis_vertex *vertex)
 {
-  isis_list_delete (vertex->Adj_N);
+  pm_list_delete (vertex->Adj_N);
 
   free(vertex);
 
@@ -210,10 +209,10 @@ static void
 isis_spftree_del (struct isis_spftree *spftree)
 {
   spftree->tents->del = (void (*)(void *)) isis_vertex_del;
-  isis_list_delete (spftree->tents);
+  pm_list_delete (spftree->tents);
 
   spftree->paths->del = (void (*)(void *)) isis_vertex_del;
-  isis_list_delete (spftree->paths);
+  pm_list_delete (spftree->paths);
 
   free(spftree);
 
@@ -280,7 +279,7 @@ isis_vertex_new (void *id, enum vertextype vtype)
       Log(LOG_ERR, "ERROR ( %s/core/ISIS ): WTF!\n", config.name);
     }
 
-  vertex->Adj_N = isis_list_new ();
+  vertex->Adj_N = pm_list_new ();
 
   return vertex;
 }
@@ -311,19 +310,19 @@ isis_spf_add_self (struct isis_spftree *spftree, struct isis_area *area,
 
   vertex->lsp = lsp;
 
-  isis_listnode_add (spftree->paths, vertex);
+  pm_listnode_add (spftree->paths, vertex);
 
   return;
 }
 
 static struct isis_vertex *
-isis_find_vertex (struct list *list, void *id, enum vertextype vtype)
+isis_find_vertex (struct pm_list *list, void *id, enum vertextype vtype)
 {
-  struct listnode *node;
+  struct pm_listnode *node;
   struct isis_vertex *vertex;
   struct isis_prefix *p1, *p2;
 
-  for (ALL_LIST_ELEMENTS_RO (list, node, vertex))
+  for (PM_ALL_LIST_ELEMENTS_RO (list, node, vertex))
     {
       if (vertex->type != vtype)
 	continue;
@@ -367,7 +366,7 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
 		   int depth, int family)
 {
   struct isis_vertex *vertex, *v;
-  struct listnode *node;
+  struct pm_listnode *node;
 
   u_char buff[BUFSIZ];
 
@@ -376,27 +375,27 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
   vertex->depth = depth;
 
   if (adj)
-    isis_listnode_add (vertex->Adj_N, adj);
+    pm_listnode_add (vertex->Adj_N, adj);
 
   if (config.nfacctd_isis_msglog) 
     Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Spf: add to TENT  %s %s depth %d dist %d\n",
               config.name, vtype2string (vertex->type), vid2string (vertex, buff),
               vertex->depth, vertex->d_N);
 
-  isis_listnode_add (spftree->tents, vertex);
-  if (isis_list_isempty (spftree->tents))
+  pm_listnode_add (spftree->tents, vertex);
+  if (pm_list_isempty (spftree->tents))
     {
-      isis_listnode_add (spftree->tents, vertex);
+      pm_listnode_add (spftree->tents, vertex);
       return vertex;
     }
   
   /* XXX: This cant use the standard ALL_LIST_ELEMENT macro */
-  for (node = listhead (spftree->tents); node; node = listnextnode (node))
+  for (node = pm_listhead (spftree->tents); node; node = pm_listnextnode (node))
     {
-      v = listgetdata (node);
+      v = pm_listgetdata (node);
       if (v->d_N > vertex->d_N)
 	{
-	  isis_list_add_node_prev (spftree->tents, node, vertex);
+	  pm_list_add_node_prev (spftree->tents, node, vertex);
 	  break;
 	}
       else if (v->d_N == vertex->d_N)
@@ -409,15 +408,15 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
 		  break;
 		}
               /* XXX: this seems dubious, node is the loop iterator */
-	      node = listnextnode (node);
-	      (node) ? (v = listgetdata (node)) : (v = NULL);
+	      node = pm_listnextnode (node);
+	      (node) ? (v = pm_listgetdata (node)) : (v = NULL);
 	    }
-	  isis_list_add_node_prev (spftree->tents, node, vertex);
+	  pm_list_add_node_prev (spftree->tents, node, vertex);
 	  break;
 	}
       else if (node->next == NULL)
 	{
-	  isis_list_add_node_next (spftree->tents, node, vertex);
+	  pm_list_add_node_next (spftree->tents, node, vertex);
 	  break;
 	}
     }
@@ -439,15 +438,15 @@ isis_spf_add_local (struct isis_spftree *spftree, enum vertextype vtype,
       if (vertex->d_N == cost)
 	{
 	  if (adj)
-	    isis_listnode_add (vertex->Adj_N, adj);
+	    pm_listnode_add (vertex->Adj_N, adj);
 	  /*       d) */
-	  if (listcount (vertex->Adj_N) > ISIS_MAX_PATH_SPLITS)
+	  if (pm_listcount (vertex->Adj_N) > ISIS_MAX_PATH_SPLITS)
 	    remove_excess_adjs (vertex->Adj_N);
 	}
       /*         f) */
       else if (vertex->d_N > cost)
 	{
-	  isis_listnode_delete (spftree->tents, vertex);
+	  pm_listnode_delete (spftree->tents, vertex);
 	  goto add2tent;
 	}
       /*       e) do nothing */
@@ -484,9 +483,9 @@ process_N (struct isis_spftree *spftree, enum vertextype vtype, void *id,
       if (vertex->d_N == dist)
 	{
 	  if (adj)
-	    isis_listnode_add (vertex->Adj_N, adj);
+	    pm_listnode_add (vertex->Adj_N, adj);
 	  /*      2) */
-	  if (listcount (vertex->Adj_N) > ISIS_MAX_PATH_SPLITS)
+	  if (pm_listcount (vertex->Adj_N) > ISIS_MAX_PATH_SPLITS)
 	    remove_excess_adjs (vertex->Adj_N);
 	  /*      3) */
 	  return;
@@ -498,7 +497,7 @@ process_N (struct isis_spftree *spftree, enum vertextype vtype, void *id,
 	}
       else
 	{
-	  isis_listnode_delete (spftree->tents, vertex);
+	  pm_listnode_delete (spftree->tents, vertex);
 	}
     }
 
@@ -513,7 +512,7 @@ static int
 isis_spf_process_lsp (struct isis_spftree *spftree, struct isis_lsp *lsp,
 		      uint32_t cost, uint16_t depth, int family)
 {
-  struct listnode *node, *fragnode = NULL;
+  struct pm_listnode *node, *fragnode = NULL;
   u_int16_t dist;
   struct is_neigh *is_neigh;
   struct te_is_neigh *te_is_neigh;
@@ -540,7 +539,7 @@ lspfragloop:
     {
       if (lsp->tlv_data.is_neighs)
 	{
-          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
+          for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
 	    {
 	      /* C.2.6 a) */
 	      /* Two way connectivity */
@@ -555,7 +554,7 @@ lspfragloop:
 	}
       if (lsp->tlv_data.te_is_neighs)
 	{
-	  for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node,
+	  for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node,
 				     te_is_neigh))
 	    {
 	      uint32_t metric;
@@ -572,7 +571,7 @@ lspfragloop:
       if (family == AF_INET && lsp->tlv_data.ipv4_int_reachs)
 	{
 	  prefix.family = AF_INET;
-          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_int_reachs, 
+          for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_int_reachs, 
                                      node, ipreach))
 	    {
 	      dist = cost + ipreach->metrics.metric_default;
@@ -587,7 +586,7 @@ lspfragloop:
       if (family == AF_INET && lsp->tlv_data.ipv4_ext_reachs)
 	{
 	  prefix.family = AF_INET;
-          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_ext_reachs,
+          for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_ext_reachs,
                                      node, ipreach))
 	    {
 	      dist = cost + ipreach->metrics.metric_default;
@@ -595,7 +594,7 @@ lspfragloop:
 
               /* Set advertising router to the first IP address */
               if (lsp->tlv_data.ipv4_addrs) {
-                memcpy(&prefix.adv_router, isis_listnode_head(lsp->tlv_data.ipv4_addrs), sizeof(struct in_addr));
+                memcpy(&prefix.adv_router, pm_listnode_head(lsp->tlv_data.ipv4_addrs), sizeof(struct in_addr));
               }
 	      prefix.u.prefix4 = ipreach->prefix;
 	      prefix.prefixlen = isis_ip_masklen (ipreach->mask);
@@ -606,7 +605,7 @@ lspfragloop:
       if (family == AF_INET && lsp->tlv_data.te_ipv4_reachs)
 	{
 	  prefix.family = AF_INET;
-	  for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_ipv4_reachs,
+	  for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_ipv4_reachs,
 				     node, te_ipv4_reach))
 	    {
 	      dist = cost + ntohl (te_ipv4_reach->te_metric);
@@ -614,7 +613,7 @@ lspfragloop:
 
 	      /* Set advertising router to the first IP address */
 	      if (lsp->tlv_data.ipv4_addrs) {
-		memcpy(&prefix.adv_router, isis_listnode_head(lsp->tlv_data.ipv4_addrs), sizeof(struct in_addr));
+		memcpy(&prefix.adv_router, pm_listnode_head(lsp->tlv_data.ipv4_addrs), sizeof(struct in_addr));
 	      }
 	      prefix.u.prefix4 = newprefix2inaddr (&te_ipv4_reach->prefix_start,
 						   te_ipv4_reach->control);
@@ -626,7 +625,7 @@ lspfragloop:
       if (family == AF_INET6 && lsp->tlv_data.ipv6_reachs)
 	{
 	  prefix.family = AF_INET6;
-          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv6_reachs, 
+          for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv6_reachs, 
                                      node, ip6reach))
 	    {
 	      dist = cost + ip6reach->metric;
@@ -643,13 +642,13 @@ lspfragloop:
     }
 
   if (fragnode == NULL)
-    fragnode = listhead (lsp->lspu.frags);
+    fragnode = pm_listhead (lsp->lspu.frags);
   else
-    fragnode = listnextnode (fragnode);
+    fragnode = pm_listnextnode (fragnode);
 
   if (fragnode)
     {
-      lsp = listgetdata (fragnode);
+      lsp = pm_listgetdata (fragnode);
       goto lspfragloop;
     }
 
@@ -661,7 +660,7 @@ isis_spf_process_pseudo_lsp (struct isis_spftree *spftree,
 			     struct isis_lsp *lsp, uint16_t cost,
 			     uint16_t depth, int family)
 {
-  struct listnode *node, *fragnode = NULL;
+  struct pm_listnode *node, *fragnode = NULL;
   struct is_neigh *is_neigh;
   struct te_is_neigh *te_is_neigh;
   enum vertextype vtype;
@@ -675,7 +674,7 @@ pseudofragloop:
     }
 
   if (lsp->tlv_data.is_neighs)
-    for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
+    for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
       {
 	vtype = LSP_PSEUDO_ID (is_neigh->neigh_id) ? VTYPE_PSEUDO_IS
 	  : VTYPE_NONPSEUDO_IS;
@@ -693,7 +692,7 @@ pseudofragloop:
 	  }
       }
   if (lsp->tlv_data.te_is_neighs)
-    for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node, te_is_neigh))
+    for (PM_ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node, te_is_neigh))
       {
 	vtype = LSP_PSEUDO_ID (te_is_neigh->neigh_id) ? VTYPE_PSEUDO_TE_IS
 	  : VTYPE_NONPSEUDO_TE_IS;
@@ -712,13 +711,13 @@ pseudofragloop:
       }
 
   if (fragnode == NULL)
-    fragnode = listhead (lsp->lspu.frags);
+    fragnode = pm_listhead (lsp->lspu.frags);
   else
-    fragnode = listnextnode (fragnode);
+    fragnode = pm_listnextnode (fragnode);
 
   if (fragnode)
     {
-      lsp = listgetdata (fragnode);
+      lsp = pm_listgetdata (fragnode);
       goto pseudofragloop;
     }
 
@@ -730,14 +729,14 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 		       struct isis_area *area, int level, int family)
 {
   struct isis_circuit *circuit;
-  struct listnode *cnode, *ipnode;
+  struct pm_listnode *cnode, *ipnode;
   struct isis_adjacency *adj;
   struct prefix_ipv4 *ipv4;
   struct isis_prefix prefix;
   int retval = ISIS_OK;
   struct prefix_ipv6 *ipv6;
 
-  for (ALL_LIST_ELEMENTS_RO (area->circuit_list, cnode, circuit))
+  for (PM_ALL_LIST_ELEMENTS_RO (area->circuit_list, cnode, circuit))
     {
       if (circuit->state != C_STATE_UP)
 	continue;
@@ -753,7 +752,7 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
       if (family == AF_INET)
 	{
 	  prefix.family = AF_INET;
-          for (ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
+          for (PM_ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
 	    {
 	      prefix.u.prefix4 = ipv4->prefix;
 	      prefix.prefixlen = ipv4->prefixlen;
@@ -764,7 +763,7 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
       if (family == AF_INET6)
 	{
 	  prefix.family = AF_INET6;
-	  for (ALL_LIST_ELEMENTS_RO (circuit->ipv6_non_link, ipnode, ipv6))
+	  for (PM_ALL_LIST_ELEMENTS_RO (circuit->ipv6_non_link, ipnode, ipv6))
 	    {
 	      prefix.prefixlen = ipv6->prefixlen;
 	      prefix.u.prefix6 = ipv6->prefix;
@@ -815,16 +814,16 @@ static void
 add_to_paths (struct isis_spftree *spftree, struct isis_vertex *vertex,
 	      struct isis_area *area, int level)
 {
-  isis_listnode_add (spftree->paths, vertex);
+  pm_listnode_add (spftree->paths, vertex);
 
   if (vertex->type > VTYPE_ES)
     {
-      if (listcount (vertex->Adj_N) > 0) {
+      if (pm_listcount (vertex->Adj_N) > 0) {
 	isis_route_create ((struct isis_prefix *) &vertex->N.prefix, vertex->d_N,
 			   vertex->depth, vertex->Adj_N, area, level);
 
 	if (config.nfacctd_isis_msglog) {
-	  u_char prefix[BUFSIZ];
+	  char prefix[BUFSIZ];
 
 	  isis_prefix2str (&vertex->N.prefix, prefix, BUFSIZ);
 	  if (config.nfacctd_isis_msglog)
@@ -834,7 +833,7 @@ add_to_paths (struct isis_spftree *spftree, struct isis_vertex *vertex,
       }
       else {
 	if (config.nfacctd_isis_msglog) {
-	  u_char prefix[BUFSIZ];
+	  char prefix[BUFSIZ];
 
 	  isis_prefix2str (&vertex->N.prefix, prefix, BUFSIZ);
 	  if (config.nfacctd_isis_msglog)
@@ -851,8 +850,8 @@ static void
 init_spt (struct isis_spftree *spftree)
 {
   spftree->tents->del = spftree->paths->del = (void (*)(void *)) isis_vertex_del;
-  isis_list_delete_all_node (spftree->tents);
-  isis_list_delete_all_node (spftree->paths);
+  pm_list_delete_all_node (spftree->tents);
+  pm_list_delete_all_node (spftree->paths);
   spftree->tents->del = spftree->paths->del = NULL;
 
   return;
@@ -862,7 +861,7 @@ int
 isis_run_spf (struct isis_area *area, int level, int family)
 {
   int retval = ISIS_OK;
-  struct listnode *node;
+  struct pm_listnode *node;
   struct isis_vertex *vertex;
   struct isis_spftree *spftree = NULL;
   u_char lsp_id[ISIS_SYS_ID_LEN + 2];
@@ -905,19 +904,19 @@ isis_run_spf (struct isis_area *area, int level, int family)
   /*
    * C.2.7 Step 2
    */
-  if (listcount (spftree->tents) == 0)
+  if (pm_listcount (spftree->tents) == 0)
     {
       Log(LOG_WARNING, "WARN ( %s/core/ISIS ): ISIS-Spf: TENT is empty\n", config.name);
       goto out;
     }
 
-  while (listcount (spftree->tents) > 0)
+  while (pm_listcount (spftree->tents) > 0)
     {
-      node = listhead (spftree->tents);
-      vertex = listgetdata (node);
+      node = pm_listhead (spftree->tents);
+      vertex = pm_listgetdata (node);
 
       /* Remove from tent list */
-      isis_list_delete_node (spftree->tents, node);
+      pm_list_delete_node (spftree->tents, node);
 
       if (isis_find_vertex (spftree->paths, vertex->N.id, vertex->type))
 	continue;
