@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -28,12 +28,10 @@
 
 struct p_kafka_host kafkap_kafka_host;
 struct p_kafka_host bgp_daemon_msglog_kafka_host;
-struct p_kafka_host bgp_table_dump_kafka_host;
 struct p_kafka_host bmp_daemon_msglog_kafka_host;
-struct p_kafka_host bmp_dump_kafka_host;
 struct p_kafka_host sfacctd_counter_kafka_host;
+struct p_kafka_host telemetry_kafka_host;
 struct p_kafka_host telemetry_daemon_msglog_kafka_host;
-struct p_kafka_host telemetry_dump_kafka_host;
 struct p_kafka_host nfacctd_kafka_host;
 
 int kafkap_ret_err_cb;
@@ -83,7 +81,7 @@ void p_kafka_unset_topic(struct p_kafka_host *kafka_host)
 
 void p_kafka_set_topic(struct p_kafka_host *kafka_host, char *topic)
 {
-  if (kafka_host) {
+  if (kafka_host && kafka_host->rk) {
     kafka_host->topic_cfg = rd_kafka_topic_conf_new();
     p_kafka_apply_topic_config(kafka_host);
 
@@ -106,7 +104,7 @@ void p_kafka_set_topic(struct p_kafka_host *kafka_host, char *topic)
     /* destroy current allocation before making a new one */
     if (kafka_host->topic) p_kafka_unset_topic(kafka_host);
 
-    if (kafka_host->rk && kafka_host->topic_cfg) {
+    if (kafka_host->topic_cfg) {
       kafka_host->topic = rd_kafka_topic_new(kafka_host->rk, topic, kafka_host->topic_cfg);
       kafka_host->topic_cfg = NULL; /* rd_kafka_topic_new() destroys conf as per rdkafka.h */
     }
@@ -347,10 +345,6 @@ void p_kafka_apply_topic_config(struct p_kafka_host *kafka_host)
 
 void p_kafka_logger(const rd_kafka_t *rk, int level, const char *fac, const char *buf)
 {
-  struct timeval tv;
-
-  gettimeofday(&tv, NULL);
-
   Log(LOG_DEBUG, "DEBUG ( %s/%s ): RDKAFKA-%i-%s: %s: %s\n", config.name, config.type, level, fac, rd_kafka_name(rk), buf);
 }
 
@@ -365,11 +359,11 @@ void p_kafka_msg_delivered(rd_kafka_t *rk, void *payload, size_t len, int error_
     if (config.debug) {
       if (p_kafka_get_content_type(kafka_host) == PM_KAFKA_CNT_TYPE_STR) {
         char *payload_str = (char *) payload;
-	char saved = payload_str[len];
+	char saved = payload_str[len - 1];
 
-	payload_str[len] = '\0';
+	payload_str[len - 1] = '\0';
         Log(LOG_DEBUG, "DEBUG ( %s/%s ): Kafka message delivery successful (%zd bytes): %p\n", config.name, config.type, len, payload);
-	payload_str[len] = saved;
+	payload_str[len - 1] = saved;
       }
       else {
 	size_t base64_data_len = 0;
@@ -585,7 +579,7 @@ int p_kafka_check_outq_len(struct p_kafka_host *kafka_host)
       if (outq_len == old_outq_len) {
 	if (retries < PM_KAFKA_OUTQ_LEN_RETRIES) retries++;
 	else {
-	  Log(LOG_ERR, "ERROR ( %s/%s ): Connection failed to Kafka: p_kafka_check_outq_len()\n", config.name, config.type);
+	  Log(LOG_ERR, "ERROR ( %s/%s ): Connection failed to Kafka: p_kafka_check_outq_len() (%u)\n", config.name, config.type, outq_len);
           p_kafka_close(kafka_host, TRUE);
 	  return outq_len; 
 	}
