@@ -29,7 +29,8 @@ struct pm_ndpi_workflow *pm_ndpi_workflow_init()
     
   NDPI_PROTOCOL_BITMASK all;
     
-  struct ndpi_detection_module_struct *module = ndpi_init_detection_module();
+  ndpi_init_prefs pm_ndpi_init_prefs = ndpi_no_prefs;
+  struct ndpi_detection_module_struct *module = ndpi_init_detection_module(pm_ndpi_init_prefs);
   struct pm_ndpi_workflow *workflow = ndpi_calloc(1, sizeof(struct pm_ndpi_workflow));
 
   log_notification_init(&log_notifications.ndpi_cache_full);
@@ -77,6 +78,14 @@ struct pm_ndpi_workflow *pm_ndpi_workflow_init()
   NDPI_BITMASK_SET_ALL(all);
   ndpi_set_protocol_detection_bitmask2(workflow->ndpi_struct, &all);
 
+#if NDPI_MAJOR >= 3 && NDPI_MINOR >= 5
+  ndpi_finalize_initialization(workflow->ndpi_struct);
+#endif
+
+#if NDPI_MAJOR >= 3 && NDPI_MINOR <= 4
+  ndpi_finalize_initalization(workflow->ndpi_struct);
+#endif
+
   return workflow;
 }
 
