@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -198,6 +198,9 @@ struct data_hdr_v9 {
 #define NF9_FLOW_PACKETS		86 
 
 #define NF9_FORWARDING_STATUS           89
+#define NF9_MPLS_VPN_RD			90
+/* ... */
+#define NF9_LAYER2_PKT_SECTION_DATA	104
 /* ... */
 #define NF9_PEER_DST_AS			128
 #define NF9_PEER_SRC_AS			129
@@ -210,6 +213,8 @@ struct data_hdr_v9 {
 #define NF9_LAST_SWITCHED_SEC		151
 #define NF9_FIRST_SWITCHED_MSEC		152
 #define NF9_LAST_SWITCHED_MSEC		153
+#define NF9_FIRST_SWITCHED_USEC		154
+#define NF9_LAST_SWITCHED_USEC		155
 /* ... */
 #define NF9_FIRST_SWITCHED_DELTA_MICRO	158
 #define NF9_LAST_SWITCHED_DELTA_MICRO	159
@@ -312,7 +317,7 @@ struct data_hdr_v9 {
 #define NF9_CUST_LABEL			3
 /* CUSTOM TYPES END HERE */
 
-#define MAX_TPL_DESC_LIST 89
+#define MAX_TPL_DESC_LIST 90
 static char __attribute__((unused)) *tpl_desc_list[] = {
   "",
   "in bytes",
@@ -382,6 +387,7 @@ static char __attribute__((unused)) *tpl_desc_list[] = {
   "", "", "", "",
   "", "", "",
   "forwarding status",
+  "mpls vpn rd"
 };
 
 #define MAX_OPT_TPL_DESC_LIST 100
@@ -486,10 +492,11 @@ struct NF_dissect {
 
 /* functions */
 extern void process_v5_packet(unsigned char *, u_int16_t, struct packet_ptrs *, struct plugin_requests *, u_int16_t, struct NF_dissect *);
-extern void process_v9_packet(unsigned char *, u_int16_t, struct packet_ptrs_vector *, struct plugin_requests *, u_int16_t, struct NF_dissect *);
+extern void process_v9_packet(unsigned char *, u_int16_t, struct packet_ptrs_vector *, struct plugin_requests *, u_int16_t, struct NF_dissect *, int *);
 extern void process_raw_packet(unsigned char *, u_int16_t, struct packet_ptrs_vector *, struct plugin_requests *);
-extern u_int8_t NF_evaluate_flow_type(struct template_cache_entry *, struct packet_ptrs *);
+extern void NF_evaluate_flow_type(struct flow_chars *, struct template_cache_entry *, struct packet_ptrs *);
 extern u_int16_t NF_evaluate_direction(struct template_cache_entry *, struct packet_ptrs *);
+extern void NF_process_classifiers(struct packet_ptrs *, struct packet_ptrs *, unsigned char *, struct template_cache_entry *);
 extern pm_class_t NF_evaluate_classifiers(struct xflow_status_entry_class *, pm_class_t *, struct xflow_status_entry *);
 extern void reset_mac(struct packet_ptrs *);
 extern void reset_mac_vlan(struct packet_ptrs *);
