@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -36,16 +36,16 @@
 
 #ifdef WITH_AVRO
 /* global variables */
-avro_schema_t avro_acct_schema, avro_acct_init_schema, avro_acct_close_schema;
+avro_schema_t p_avro_acct_schema, p_avro_acct_init_schema, p_avro_acct_close_schema;
 
 /* functions */
-avro_schema_t avro_schema_build_acct_data(u_int64_t wtc, u_int64_t wtc_2)
+avro_schema_t p_avro_schema_build_acct_data(u_int64_t wtc, u_int64_t wtc_2)
 {
   avro_schema_t schema = avro_schema_record("acct_data", NULL);
   avro_schema_t optlong_s = avro_schema_union();
   avro_schema_t optstr_s = avro_schema_union();
 
-  Log(LOG_INFO, "INFO ( %s/%s ): avro_schema_build_acct_data(): building acct schema.\n", config.name, config.type);
+  Log(LOG_INFO, "INFO ( %s/%s ): p_avro_schema_build_acct_data(): building acct schema.\n", config.name, config.type);
 
   avro_schema_union_append(optlong_s, avro_schema_null());
   avro_schema_union_append(optlong_s, avro_schema_long());
@@ -299,6 +299,9 @@ avro_schema_t avro_schema_build_acct_data(u_int64_t wtc, u_int64_t wtc_2)
   if (wtc_2 & COUNT_EXPORT_PROTO_SYSID)
     avro_schema_record_field_append(schema, "export_proto_sysid", avro_schema_long());
 
+  if (wtc_2 & COUNT_EXPORT_PROTO_TIME)
+    avro_schema_record_field_append(schema, "timestamp_export", avro_schema_string());
+
   if (config.cpptrs.num > 0) {
     avro_schema_record_field_append(
         schema, "custom_primitives", avro_schema_map(avro_schema_string()));
@@ -319,11 +322,11 @@ avro_schema_t avro_schema_build_acct_data(u_int64_t wtc, u_int64_t wtc_2)
   return schema;
 }
 
-avro_schema_t avro_schema_build_acct_init()
+avro_schema_t p_avro_schema_build_acct_init()
 {
   avro_schema_t schema = avro_schema_record("acct_init", NULL);
 
-  Log(LOG_INFO, "INFO ( %s/%s ): avro_schema_build_acct_init(): building acct_init schema.\n", config.name, config.type);
+  Log(LOG_INFO, "INFO ( %s/%s ): p_avro_schema_build_acct_init(): building acct_init schema.\n", config.name, config.type);
 
   avro_schema_record_field_append(schema, "event_type", avro_schema_string());
   avro_schema_record_field_append(schema, "writer_id", avro_schema_string());
@@ -331,11 +334,11 @@ avro_schema_t avro_schema_build_acct_init()
   return schema;
 }
 
-avro_schema_t avro_schema_build_acct_close()
+avro_schema_t p_avro_schema_build_acct_close()
 {
   avro_schema_t schema = avro_schema_record("acct_close", NULL);
 
-  Log(LOG_INFO, "INFO ( %s/%s ): avro_schema_build_acct_close(): building acct_close schema.\n", config.name, config.type);
+  Log(LOG_INFO, "INFO ( %s/%s ): p_avro_schema_build_acct_close(): building acct_close schema.\n", config.name, config.type);
 
   avro_schema_record_field_append(schema, "event_type", avro_schema_string());
   avro_schema_record_field_append(schema, "writer_id", avro_schema_string());
@@ -347,7 +350,7 @@ avro_schema_t avro_schema_build_acct_close()
   return schema;
 }
 
-void avro_schema_add_writer_id(avro_schema_t schema)
+void p_avro_schema_add_writer_id(avro_schema_t schema)
 {
   avro_schema_record_field_append(schema, "writer_id", avro_schema_string());
 }
@@ -581,7 +584,7 @@ avro_value_t compose_avro_acct_data(u_int64_t wtc, u_int64_t wtc_2, u_int8_t flo
 
   if (wtc & COUNT_PEER_DST_IP) {
     pm_avro_check(avro_value_get_by_name(&value, "peer_ip_dst", &field, NULL));
-    addr_to_str(ip_address, &pbgp->peer_dst_ip);
+    addr_to_str2(ip_address, &pbgp->peer_dst_ip, ft2af(flow_type));
     pm_avro_check(avro_value_set_string(&field, ip_address));
   }
 
@@ -974,6 +977,14 @@ avro_value_t compose_avro_acct_data(u_int64_t wtc, u_int64_t wtc_2, u_int8_t flo
     pm_avro_check(avro_value_set_long(&field, pbase->export_proto_sysid));
   }
 
+  if (wtc_2 & COUNT_EXPORT_PROTO_TIME) {
+    compose_timestamp(tstamp_str, SRVBUFLEN, &pnat->timestamp_export, TRUE,
+		      config.timestamps_since_epoch, config.timestamps_rfc3339,
+		      config.timestamps_utc);
+    pm_avro_check(avro_value_get_by_name(&value, "timestamp_export", &field, NULL));
+    pm_avro_check(avro_value_set_string(&field, tstamp_str));
+  }
+
   /* all custom primitives printed here */
   {
     if (config.cpptrs.num > 0)
@@ -1069,15 +1080,15 @@ void add_writer_name_and_pid_avro(avro_value_t value, char *name, pid_t writer_p
 void write_avro_schema_to_file(char *filename, avro_schema_t schema)
 {
   FILE *avro_fp;
-  avro_writer_t avro_schema_writer;
+  avro_writer_t p_avro_schema_writer;
 
   avro_fp = open_output_file(filename, "w", TRUE);
 
   if (avro_fp) {
-    avro_schema_writer = avro_writer_file(avro_fp);
+    p_avro_schema_writer = avro_writer_file(avro_fp);
 
-    if (avro_schema_writer) {
-      if (avro_schema_to_json(schema, avro_schema_writer)) {
+    if (p_avro_schema_writer) {
+      if (avro_schema_to_json(schema, p_avro_schema_writer)) {
 	goto exit_lane;
       }
     }
@@ -1091,7 +1102,7 @@ void write_avro_schema_to_file(char *filename, avro_schema_t schema)
 
   exit_lane:
   Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_schema_to_file(): unable to dump Avro schema: %s\n", config.name, config.type, avro_strerror());
-  exit_gracefully(1);
+  pm_avro_exit_gracefully(1);
 }
 
 void write_avro_schema_to_file_with_suffix(char *filename, char *suffix, char *buf, avro_schema_t schema)
@@ -1103,45 +1114,45 @@ void write_avro_schema_to_file_with_suffix(char *filename, char *suffix, char *b
 
 char *write_avro_schema_to_memory(avro_schema_t avro_schema)
 {
-  avro_writer_t avro_writer;
-  char *avro_buf = NULL;
+  avro_writer_t p_avro_writer;
+  char *p_avro_buf = NULL;
 
   if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
 
-  avro_buf = malloc(config.avro_buffer_size);
+  p_avro_buf = malloc(config.avro_buffer_size);
 
-  if (!avro_buf) {
+  if (!p_avro_buf) {
     Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_schema_to_memory(): malloc() failed. Exiting.\n", config.name, config.type);
-    exit_gracefully(1);
+    pm_avro_exit_gracefully(1);
   }
-  else memset(avro_buf, 0, config.avro_buffer_size);
+  else memset(p_avro_buf, 0, config.avro_buffer_size);
 
-  avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+  p_avro_writer = avro_writer_memory(p_avro_buf, config.avro_buffer_size);
 
-  if (avro_schema_to_json(avro_schema, avro_writer)) {
+  if (avro_schema_to_json(avro_schema, p_avro_writer)) {
     Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_schema_to_memory(): unable to dump Avro schema: %s\n", config.name, config.type, avro_strerror());
-    free(avro_buf);
-    avro_buf = NULL;
+    free(p_avro_buf);
+    p_avro_buf = NULL;
   }
 
-  if (!avro_writer_tell(avro_writer)) {
+  if (!avro_writer_tell(p_avro_writer)) {
     Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_schema_to_memory(): unable to tell Avro schema: %s\n", config.name, config.type, avro_strerror());
-    free(avro_buf);
-    avro_buf = NULL;
+    free(p_avro_buf);
+    p_avro_buf = NULL;
   }
 
-  avro_writer_free(avro_writer);
+  avro_writer_free(p_avro_writer);
 
-  return avro_buf;
+  return p_avro_buf;
 }
 
 char *compose_avro_purge_schema(avro_schema_t avro_schema, char *writer_name)
 {
-  char *avro_buf = NULL, *json_str = NULL;
+  char *p_avro_buf = NULL, *json_str = NULL;
 
-  avro_buf = write_avro_schema_to_memory(avro_schema);
+  p_avro_buf = write_avro_schema_to_memory(avro_schema);
 
-  if (avro_buf) {
+  if (p_avro_buf) {
     char event_type[] = "purge_schema", wid[SHORTSHORTBUFLEN];
     json_t *obj = json_object();
 
@@ -1150,15 +1161,15 @@ char *compose_avro_purge_schema(avro_schema_t avro_schema, char *writer_name)
     snprintf(wid, SHORTSHORTBUFLEN, "%s/%u", writer_name, 0);
     json_object_set_new_nocheck(obj, "writer_id", json_string(wid));
 
-    json_object_set_new_nocheck(obj, "schema", json_string(avro_buf));
+    json_object_set_new_nocheck(obj, "schema", json_string(p_avro_buf));
 
-    free(avro_buf);
+    free(p_avro_buf);
 
     json_str = compose_json_str(obj);
   }
   else {
-    Log(LOG_ERR, "ERROR ( %s/%s ): compose_avro_purge_schema(): no avro_buf. Exiting.\n", config.name, config.type);
-    exit_gracefully(1);
+    Log(LOG_ERR, "ERROR ( %s/%s ): compose_avro_purge_schema(): no p_avro_buf. Exiting.\n", config.name, config.type);
+    pm_avro_exit_gracefully(1);
   }
 
   return json_str;
@@ -1183,7 +1194,7 @@ char *compose_avro_schema_name(char *extra1, char *extra2)
   schema_name = malloc(len_total);  
   if (!schema_name) {
     Log(LOG_ERR, "ERROR ( %s/%s ): compose_avro_schema_name(): malloc() failed. Exiting.\n", config.name, config.type);
-    exit_gracefully(1);
+    pm_avro_exit_gracefully(1);
   }
   else memset(schema_name, 0, len_total);
 
@@ -1205,6 +1216,11 @@ char *compose_avro_schema_name(char *extra1, char *extra2)
 }
 
 #ifdef WITH_SERDES
+void p_avro_serdes_logger(serdes_t *sd_desc, int level, const char *fac, const char *buf, void *opaque)
+{
+  Log(LOG_DEBUG, "DEBUG ( %s/%s ): SERDES-%i-%s: %s\n", config.name, config.type, level, fac, buf);
+}
+
 serdes_schema_t *compose_avro_schema_registry_name_2(char *topic, int is_topic_dyn,
 		avro_schema_t avro_schema, char *type, char *name, char *schema_registry)
 {
@@ -1235,41 +1251,52 @@ serdes_schema_t *compose_avro_schema_registry_name(char *topic, int is_topic_dyn
 {
   serdes_conf_t *sd_conf;
   serdes_t *sd_desc;
-  serdes_schema_t *sd_schema = NULL;
+  serdes_schema_t *loc_schema = NULL;
   char sd_errstr[LONGSRVBUFLEN];
 
-  char *avro_schema_str = write_avro_schema_to_memory(avro_schema);
-  char *avro_schema_name;
+  char *p_avro_schema_str = write_avro_schema_to_memory(avro_schema);
+  char *p_avro_schema_name;
 
   if (!is_topic_dyn) {
-    avro_schema_name = malloc(strlen(topic) + strlen("-value") + 1);
+    p_avro_schema_name = malloc(strlen(topic) + strlen("-value") + 1);
 
-    strcpy(avro_schema_name, topic);
-    strcat(avro_schema_name, "-value");
+    strcpy(p_avro_schema_name, topic);
+    strcat(p_avro_schema_name, "-value");
   }
   else {
-    avro_schema_name = compose_avro_schema_name(type, name);
+    p_avro_schema_name = compose_avro_schema_name(type, name);
   }
 
   sd_conf = serdes_conf_new(NULL, 0, "schema.registry.url", schema_registry, NULL);
+  if (!sd_conf) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): serdes_conf_new() failed: %s. Exiting.\n", config.name, config.type, sd_errstr);
+    pm_avro_exit_gracefully(1);
+  }
+
+  serdes_conf_set_log_cb(sd_conf, p_avro_serdes_logger);
 
   sd_desc = serdes_new(sd_conf, sd_errstr, sizeof(sd_errstr));
   if (!sd_desc) {
     Log(LOG_ERR, "ERROR ( %s/%s ): serdes_new() failed: %s. Exiting.\n", config.name, config.type, sd_errstr);
-    exit_gracefully(1);
+    pm_avro_exit_gracefully(1);
   }
 
-  sd_schema = serdes_schema_add(sd_desc, avro_schema_name, -1, avro_schema_str, -1, sd_errstr, sizeof(sd_errstr));
-  if (!sd_schema) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): serdes_schema_add() failed: %s. Exiting.\n", config.name, config.type, sd_errstr);
-    exit_gracefully(1);
+  loc_schema = serdes_schema_add(sd_desc, p_avro_schema_name, -1, p_avro_schema_str, -1, sd_errstr, sizeof(sd_errstr));
+  if (!loc_schema) {
+    Log(LOG_WARNING, "WARN ( %s/%s ): serdes_schema_add() failed: %s\n", config.name, config.type, sd_errstr);
   }
   else {
-    Log(LOG_DEBUG, "DEBUG ( %s/%s ): serdes_schema_add(): name=%s id=%d definition=%s\n", config.name, config.type,
-	serdes_schema_name(sd_schema), serdes_schema_id(sd_schema), serdes_schema_definition(sd_schema));
+    if (!config.debug) {
+      Log(LOG_INFO, "INFO ( %s/%s ): serdes_schema_add(): name=%s id=%d\n", config.name, config.type,
+	  serdes_schema_name(loc_schema), serdes_schema_id(loc_schema));
+    }
+    else {
+      Log(LOG_DEBUG, "DEBUG ( %s/%s ): serdes_schema_add(): name=%s id=%d definition=%s\n", config.name, config.type,
+	  serdes_schema_name(loc_schema), serdes_schema_id(loc_schema), serdes_schema_definition(loc_schema));
+    }
   }
 
-  return sd_schema;
+  return loc_schema;
 }
 #endif
 
@@ -1279,7 +1306,7 @@ void write_avro_json_record_to_file(FILE *fp, avro_value_t value)
 
   if (avro_value_to_json(&value, TRUE, &json_str)) {
     Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_json_record_to_file() unable to value to JSON: %s\n", config.name, config.type, avro_strerror());
-    exit_gracefully(1);
+    pm_avro_exit_gracefully(1);
   }
 
   fprintf(fp, "%s\n", json_str);
@@ -1292,9 +1319,20 @@ char *write_avro_json_record_to_buf(avro_value_t value)
 
   if (avro_value_to_json(&value, TRUE, &json_str)) {
     Log(LOG_ERR, "ERROR ( %s/%s ): write_avro_json_record_to_buf() unable to value to JSON: %s\n", config.name, config.type, avro_strerror());
-    exit_gracefully(1);
+    pm_avro_exit_gracefully(1);
   }
 
   return json_str;
 }
 #endif
+
+void pm_avro_exit_gracefully(int status)
+{
+  if (!config.debug) {
+    exit_gracefully(status);
+  }
+  else {
+    /* gather additional info on exit */
+    assert(1 == 0);
+  }
+}
