@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -453,6 +453,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_EXPORT_PROTO_TIME) {
+    cjhandler[idx] = compose_json_timestamp_export;
+    idx++;
+  }
+
   if (config.cpptrs.num) {
     cjhandler[idx] = compose_json_custom_primitives;
     idx++;
@@ -670,7 +675,7 @@ void compose_json_peer_dst_ip(json_t *obj, struct chained_cache *cc)
 {
   char ip_address[INET6_ADDRSTRLEN];
 
-  addr_to_str(ip_address, &cc->pbgp->peer_dst_ip);
+  addr_to_str2(ip_address, &cc->pbgp->peer_dst_ip, ft2af(cc->flow_type));
   json_object_set_new_nocheck(obj, "peer_ip_dst", json_string(ip_address));
 }
 
@@ -1072,6 +1077,16 @@ void compose_json_timestamp_arrival(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "timestamp_arrival", json_string(tstamp_str));
 }
 
+void compose_json_timestamp_export(json_t *obj, struct chained_cache *cc)
+{
+  char tstamp_str[VERYSHORTBUFLEN];
+
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_export, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
+  json_object_set_new_nocheck(obj, "timestamp_export", json_string(tstamp_str));
+}
+
 void compose_json_timestamp_stitching(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
