@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -253,7 +253,7 @@ void load_plugins(struct plugin_requests *req)
       /* ZMQ inits, if required */
 #ifdef WITH_ZMQ
       if (list->cfg.pipe_zmq) {
-	char log_id[SHORTBUFLEN];
+	char log_id[LARGEBUFLEN];
 
 	p_zmq_plugin_pipe_init_core(&chptr->zmq_host, list->id, username, password);
 	snprintf(log_id, sizeof(log_id), "%s/%s", list->name, list->type.string);
@@ -334,6 +334,27 @@ void load_plugins(struct plugin_requests *req)
 	}
       }
 
+      if (list->cfg.type_id != PLUGIN_ID_TEE) {
+	if (list->cfg.acct_type == ACCT_NF && list->cfg.nfacctd_account_options) {
+	  int have_sample_type = FALSE;
+
+	  if (list->cfg.ptm.index_num) {
+	    int idx;
+
+	    for (idx = 0; idx < list->cfg.ptm.index_num; idx++) {
+	      if (list->cfg.ptm.index[idx].bitmap & PRETAG_SAMPLE_TYPE) {
+		have_sample_type = TRUE;
+	      }
+	    }
+	  }
+
+	  if ((!have_sample_type && !list->cfg.ptm.num) || (!have_sample_type && list->cfg.ptm.index_num)) {
+	    Log(LOG_WARNING, "WARN ( %s/%s ): nfacctd_account_options is enabled but pre_tag_map does not contain a sample_type rule.\n",
+		list->name, list->type.string);
+	  }
+	}
+      }
+
       list = list->next;
       ptm_index++;
     }
@@ -389,7 +410,7 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
       if (p->cfg.ptm_global && got_tags) {
         pptrs->tag = saved_tag;
         pptrs->tag2 = saved_tag2;
-	    pretag_copy_label(&pptrs->label, saved_label);
+	pretag_copy_label(&pptrs->label, saved_label);
 
         pptrs->have_tag = saved_have_tag;
         pptrs->have_tag2 = saved_have_tag2;
@@ -964,7 +985,7 @@ void P_zmq_pipe_init(void *zh, int *pipe_fd, u_int32_t *seq)
 #ifdef WITH_ZMQ
   if (zh) {
     struct p_zmq_host *zmq_host = zh;
-    char log_id[SHORTBUFLEN];
+    char log_id[LARGEBUFLEN];
 
     p_zmq_plugin_pipe_init_plugin(zmq_host);
 
