@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -129,6 +129,7 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"timestamp_start", 0, 0, 1, 0, 0, 0, 0, "Flow start time or observation time at the exporter"},
   {"timestamp_end", 0, 0, 1, 0, 0, 0, 0, "Flow end time"},
   {"timestamp_arrival", 1, 1, 1, 1, 0, 0, 0, "Observation time at the collector"},
+  {"timestamp_export", 0, 0, 1, 0, 0, 0, 0, "Observation time at the exporter"},
   {"", 0, 0, 0, 0, 0, 0, 0, ""}
 };
 
@@ -183,8 +184,8 @@ static const struct _protocols_struct _protocols[] = {
   {"gre", 47},
   {"48", 48},
   {"49", 49},
-  {"ipv6-crypt", 50}, 
-  {"ipv6-auth", 51},
+  {"esp", 50}, 
+  {"ah", 51},
   {"52", 52}, 
   {"53", 53},  
   {"54", 54}, 
@@ -300,6 +301,20 @@ static const char __attribute__((unused)) *bgp_origin[] = {
   ""
 };
 
+static const char __attribute__((unused)) *bgp_rd_origin[] = {
+  "unknown",
+  "bgp",
+  "bmp",
+  "flow",
+  ""
+};
+
+static const u_int16_t __attribute__((unused)) lookup_type_to_bgp_rd_origin[] = {
+  RD_ORIGIN_UNKNOWN,
+  RD_ORIGIN_BGP,
+  RD_ORIGIN_BMP
+};
+
 static const char __attribute__((unused)) *rpki_roa[] = {
   "u",
   "i",
