@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -26,7 +26,7 @@
 #define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:MuRg:L:VaAE:"
 #define ARGS_PMTELEMETRYD "hVL:u:t:Z:f:dDS:F:o:O:i:"
 #define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:gm:"
-#define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:"
+#define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:I:Z:Y:T:"
 #define ARGS_PMACCT "hSsc:Cetm:p:P:M:arN:n:lT:O:E:uVUiI0"
 #define N_PRIMITIVES 128
 #define N_FUNCS 10 
@@ -39,7 +39,7 @@
 #define FOLLOW_BGP_NH_ENTRIES 32 
 #define MAX_PROTOCOL_LEN 32 
 #define DEFAULT_AVRO_SCHEMA_REFRESH_TIME 60
-#define MAX_AVRO_SCHEMA 16
+#define MAX_AVRO_SCHEMA 65535 
 #define DEFAULT_IMT_PLUGIN_POLL_TIMEOUT 5
 #define DEFAULT_SLOTH_SLEEP_TIME 5
 #define UINT32T_THRESHOLD 4290000000UL
@@ -95,7 +95,6 @@
 #define PRIMITIVE_LEN 		32
 #define PRIMITIVE_DESC_LEN	64
 
-#define PMACCT_VERSION "1.7.4-git"
 #define MANTAINER "Paolo Lucente <paolo@pmacct.net>"
 #define GET_IN_TOUCH_MSG "If you see this message, please get in touch"
 #define PMACCTD_USAGE_HEADER "Promiscuous Mode Accounting Daemon, pmacctd"
@@ -138,6 +137,7 @@
 #endif
 
 /* Daemon identificator */ 
+#define ACCT_UNKNOWN		0	/* Undefined / unknown daemon */
 #define ACCT_PM			1	/* promiscuous mode */
 #define ACCT_NF			2	/* NetFlow */
 #define ACCT_SF			3	/* sFlow */
@@ -253,6 +253,8 @@
 #define COUNT_INT_TUNNEL_DST_MAC	0x0002000800000000ULL
 #define COUNT_INT_TUNNEL_SRC_PORT	0x0002001000000000ULL
 #define COUNT_INT_TUNNEL_DST_PORT	0x0002002000000000ULL
+#define COUNT_INT_EXPORT_PROTO_TIME	0x0002004000000000ULL
+#define COUNT_INT_TIMESTAMP_EXPORT	0x0002004000000000ULL /* alias of COUNT_EXPORT_PROTO_TIME */
 #define COUNT_INT_CUSTOM_PRIMITIVES	0x0002800000000000ULL
 
 #define COUNT_INDEX_MASK	0xFFFF
@@ -349,6 +351,8 @@
 #define COUNT_TUNNEL_DST_MAC		(COUNT_INT_TUNNEL_DST_MAC & COUNT_REGISTRY_MASK)
 #define COUNT_TUNNEL_SRC_PORT		(COUNT_INT_TUNNEL_SRC_PORT & COUNT_REGISTRY_MASK)
 #define COUNT_TUNNEL_DST_PORT		(COUNT_INT_TUNNEL_DST_PORT & COUNT_REGISTRY_MASK)
+#define COUNT_EXPORT_PROTO_TIME		(COUNT_INT_EXPORT_PROTO_TIME & COUNT_REGISTRY_MASK)
+#define COUNT_TIMESTAMP_EXPORT		(COUNT_INT_TIMESTAMP_EXPORT & COUNT_REGISTRY_MASK) /* alias of COUNT_EXPORT_PROTO_TIME */
 #define COUNT_CUSTOM_PRIMITIVES		(COUNT_INT_CUSTOM_PRIMITIVES & COUNT_REGISTRY_MASK)
 /* PRIMITIVES DEFINITION: END */
 
@@ -430,12 +434,14 @@
 #define IFINDEX_TAG		0x00000002
 #define IFINDEX_TAG2		0x00000004
 
+#define CUSTOM_PRIMITIVE_TYPE_UNKNOWN	0
 #define CUSTOM_PRIMITIVE_TYPE_UINT	1
 #define CUSTOM_PRIMITIVE_TYPE_HEX	2
 #define CUSTOM_PRIMITIVE_TYPE_STRING	3
 #define CUSTOM_PRIMITIVE_TYPE_IP	4
 #define CUSTOM_PRIMITIVE_TYPE_MAC	5
 #define CUSTOM_PRIMITIVE_TYPE_RAW	6
+#define CUSTOM_PRIMITIVE_TYPE_COMPLEX	7
 
 #define FUNC_TYPE_NULL			0
 #define FUNC_TYPE_BGP			1
@@ -503,11 +509,7 @@ typedef struct {
   u_int16_t n;
 } s_uint16_t;
 
-#if defined HAVE_64BIT_COUNTERS
 typedef u_int64_t pm_counter_t;
-#else
-typedef u_int32_t pm_counter_t;
-#endif
 
 /* Keep common NF_AS and NF_NET values aligned, ie. NF_[NET|AS]_KEEP == 0x00000001 */
 #define NF_AS_COMPAT    0x00000000 /* Unused */
@@ -525,21 +527,22 @@ typedef u_int32_t pm_counter_t;
 #define NF_NET_FALLBACK	0x80000000 /* Fallback flag */
 
 /* flow type */
-#define NF9_FTYPE_TRAFFIC		1  /* temporary: re-coding needed */
-#define NF9_FTYPE_TRAFFIC_IPV6		1
-#define NF9_FTYPE_IPV4                  1
-#define NF9_FTYPE_IPV6                  2
-#define NF9_FTYPE_DLFS			3
-#define NF9_FTYPE_VLAN                  5
-#define NF9_FTYPE_VLAN_IPV4             6
-#define NF9_FTYPE_VLAN_IPV6             7 
-#define NF9_FTYPE_MPLS                  10
-#define NF9_FTYPE_MPLS_IPV4             11
-#define NF9_FTYPE_MPLS_IPV6             12
-#define NF9_FTYPE_VLAN_MPLS             15      
-#define NF9_FTYPE_VLAN_MPLS_IPV4        16
-#define NF9_FTYPE_VLAN_MPLS_IPV6        17
-#define NF9_FTYPE_TRAFFIC_MAX		99  /* temporary: re-coding needed */
+#define PM_FTYPE_TRAFFIC		1
+#define PM_FTYPE_IPV4			2
+#define PM_FTYPE_IPV6			3
+#define PM_FTYPE_VLAN			5
+#define PM_FTYPE_VLAN_IPV4		7  /* PM_FTYPE_VLAN + PM_FTYPE_IPV4 */
+#define PM_FTYPE_VLAN_IPV6		8  /* PM_FTYPE_VLAN + PM_FTYPE_IPV6 */
+#define PM_FTYPE_MPLS			10
+#define PM_FTYPE_MPLS_IPV4		12 /* PM_FTYPE_MPLS + PM_FTYPE_IPV4 */
+#define PM_FTYPE_MPLS_IPV6		13 /* PM_FTYPE_MPLS + PM_FTYPE_IPV6 */
+#define PM_FTYPE_VLAN_MPLS		15 /* PM_FTYPE_MPLS + PM_FTYPE_VLAN */
+#define PM_FTYPE_VLAN_MPLS_IPV4		17 /* PM_FTYPE_VLAN_MPLS + PM_FTYPE_IPV4 */
+#define PM_FTYPE_VLAN_MPLS_IPV6		18 /* PM_FTYPE_VLAN_MPLS + PM_FTYPE_IPV6 */
+#define PM_FTYPE_TRAFFIC_MAX		99  /* temporary: re-coding needed */
+
+/* flow type: NetFlow/IPFIX extended code-points */
+#define NF9_FTYPE_DLFS			4
 #define NF9_FTYPE_EVENT			100 /* temporary: re-coding needed */
 #define NF9_FTYPE_NAT_EVENT             100
 #define NF9_FTYPE_OPTION		200
@@ -553,3 +556,10 @@ typedef u_int32_t pm_counter_t;
 #define CUSTOM_PRIMITIVE_L4_PTR		5
 #define CUSTOM_PRIMITIVE_PAYLOAD_PTR	6
 #define CUSTOM_PRIMITIVE_MAX_PPTRS_IDX	7
+
+/* libpcap supports pkg-config only since recently,
+   so fixing this little >= 1.1.0 definition here for
+   OS's coming with vintage versions installed */
+#ifndef PCAP_NETMASK_UNKNOWN
+#define PCAP_NETMASK_UNKNOWN		0xffffffff
+#endif
