@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -18,6 +18,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
+#ifndef PORTS_AGGR_H
+#define PORTS_AGGR_H
 
 /* defines */
 #define PORTS_TABLE_ENTRIES 65536 
@@ -29,11 +31,6 @@ struct ports_table {
 };
 
 /* prototypes */
-#if (!defined __NET_AGGR_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT void load_ports(char *, struct ports_table *); 
-#undef EXT
+extern void load_ports(char *, struct ports_table *); 
 
+#endif //PORTS_AGGR_H
