@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -346,9 +346,9 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
                   key = NULL; value = NULL;
 		}
                 else if (acct_type == MAP_PCAP_INTERFACES) {
-                  for (dindex = 0; strcmp(pcap_interfaces_map_dictionary[dindex].key, ""); dindex++) {
-                    if (!strcmp(pcap_interfaces_map_dictionary[dindex].key, key)) {
-                      err = (*pcap_interfaces_map_dictionary[dindex].func)(filename, NULL, value, req, acct_type);
+                  for (dindex = 0; strcmp(pm_pcap_interfaces_map_dictionary[dindex].key, ""); dindex++) {
+                    if (!strcmp(pm_pcap_interfaces_map_dictionary[dindex].key, key)) {
+                      err = (*pm_pcap_interfaces_map_dictionary[dindex].func)(filename, NULL, value, req, acct_type);
                       break;
                     }
                     else err = E_NOTFOUND; /* key not found */
@@ -527,7 +527,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	      else if (acct_type == MAP_BGP_XCS) bgp_xcs_map_validate(filename, req); 
 	      else if (acct_type == MAP_IGP) igp_daemon_map_validate(filename, req); 
 	      else if (acct_type == MAP_CUSTOM_PRIMITIVES) custom_primitives_map_validate(filename, req); 
-	      else if (acct_type == MAP_PCAP_INTERFACES) pcap_interfaces_map_validate(filename, req); 
+	      else if (acct_type == MAP_PCAP_INTERFACES) pm_pcap_interfaces_map_validate(filename, req); 
 	    }
           }
           else Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] malformed line. Ignored.\n",
@@ -786,15 +786,63 @@ int pretag_copy_label(pt_label_t *dst, pt_label_t *src)
         Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pretag_copy_label).\n", config.name, config.type);
         return ERR;
       }
-      dst->len = src->len;
+
       strncpy(dst->val, src->val, src->len);
-      dst->val[dst->len] = '\0';
+      dst->val[dst->len - 1] = '\0';
     }
   }
   
   return SUCCESS;
 }
 
+int pretag_move_label(pt_label_t *dst, pt_label_t *src)
+{
+  if (!src || !dst) return ERR;
+
+  if (dst->val) {
+    Log(LOG_WARNING, "WARN ( %s/%s ): pretag_move_label failed: dst->val not null\n", config.name, config.type);
+    return ERR;
+  }
+  else {
+    if (src->len) {
+      dst->val = src->val;
+      dst->len = src->len;
+      
+      src->val = NULL;
+      src->len = 0;
+    }
+  }
+
+  return SUCCESS;
+}
+
+int pretag_append_label(pt_label_t *dst, pt_label_t *src)
+{
+  char default_sep[] = ",";
+
+  if (!src || !dst) return ERR;
+
+  if (!dst->val) {
+    Log(LOG_WARNING, "WARN ( %s/%s ): pretag_append_label failed: dst->val null\n", config.name, config.type);
+    return ERR;
+  }
+  else {
+    if (src->len) {
+      pretag_realloc_label(dst, (dst->len + src->len + 1 /* sep */ + 1 /* null */));
+      if (!dst->val) {
+        Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pretag_append_label).\n", config.name, config.type);
+        return ERR;
+      }
+
+      strncat(dst->val, default_sep, 1);
+      strncat(dst->val, src->val, src->len);
+      dst->val[dst->len] = '\0';
+    }
+  }
+
+  return SUCCESS;
+}
+
 void pretag_free_label(pt_label_t *label)
 {
   if (label && label->val) {
@@ -838,19 +886,11 @@ int pretag_entry_process(struct id_entry *e, struct packet_ptrs *pptrs, pm_id_t
       pptrs->have_tag2 = TRUE;
     } 
     else if (stop & PRETAG_MAP_RCODE_LABEL) {
-      /* auto-stacking if value exists */
       if (pptrs->label.len) {
-	char default_sep[] = ",";
-
-        if (pretag_realloc_label(&pptrs->label, label_local->len + pptrs->label.len + 1 /* sep */ + 1 /* null */)) return TRUE;
-	strncat(pptrs->label.val, default_sep, 1);
-        strncat(pptrs->label.val, label_local->val, label_local->len);
-        pptrs->label.val[pptrs->label.len] = '\0';
+	pretag_append_label(&pptrs->label, label_local);
       }
       else {
-	if (pretag_malloc_label(&pptrs->label, label_local->len + 1 /* null */)) return TRUE;
-	strncpy(pptrs->label.val, label_local->val, label_local->len);
-	pptrs->label.val[pptrs->label.len] = '\0';
+	pretag_copy_label(&pptrs->label, label_local);
       }
 
       pptrs->have_label = TRUE;
@@ -966,44 +1006,45 @@ int pretag_index_set_handlers(struct id_table *t)
 
 int pretag_index_allocate(struct id_table *t)
 {
-  pt_bitmap_t idx_t_size = 0;
-  u_int32_t iterator = 0, j = 0;
-  int ret, destroy = FALSE;
+  u_int32_t iterator = 0, i = 0, j = 0, z = 0;
+  int destroy = FALSE;
 
   if (!t) return TRUE;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
     if (t->index[iterator].entries) {
-      Log(LOG_INFO, "INFO ( %s/%s ): [%s] maps_index: created index %llx (%u entries).\n", config.name,
+      u_int32_t idx_entry_size = 0;
+
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] maps_index: creating index %llx (%u entries).\n", config.name,
     		config.type, t->filename, (unsigned long long)t->index[iterator].bitmap, t->index[iterator].entries);
 
-      assert(!t->index[iterator].idx_t);
-      
-      t->index[iterator].modulo = next_prime(t->index[iterator].entries * 2);
-      if (!t->index[iterator].modulo) t->index[iterator].modulo = (t->index[iterator].entries * 2);
+      assert(!t->index[iterator].idx_map);
 
-      idx_t_size = t->index[iterator].modulo * sizeof(struct id_index_entry);
-      t->index[iterator].idx_t = malloc(idx_t_size);
-
-      if (!t->index[iterator].idx_t) {
-        Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: unable to allocate index %llx.\n", config.name,
-		config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
-	t->index[iterator].bitmap = 0;
-	t->index[iterator].entries = 0;
-	destroy = TRUE;
-	break;
-      }
-      else {
-	memset(t->index[iterator].idx_t, 0, idx_t_size); 
-
-	for (j = 0; j < t->index[iterator].modulo; j++) {
-	  t->index[iterator].idx_t[j].depth = ID_TABLE_INDEX_DEPTH;
+      for (idx_entry_size = 0, i = 0; i < MAX_BITMAP_ENTRIES; i++) {
+	if (!t->index[iterator].idt_handler[i]) {
+	  break;
 	}
 
-	ret = hash_init_serial(&t->index[iterator].hash_serializer, 16 /* dummy len for init sake */);
-	if (ret == ERR) {
-	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: unable to allocate hash serializer for index %llx.\n", config.name,
-		config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
+	for (j = 0; tag_map_index_entries_dictionary[j].key; j++) {
+	  if (t->index[iterator].idt_handler[i] == tag_map_index_entries_dictionary[j].func) {
+	    for (z = 0; tag_map_index_entries_size_dictionary[z].key; z++) {
+	      if (tag_map_index_entries_size_dictionary[z].key == tag_map_index_entries_dictionary[j].key) {
+		idx_entry_size += tag_map_index_entries_size_dictionary[z].size;
+		break;
+	      }
+	    }
+	    break;
+	  }
+	}
+      }
+
+      {
+	char pm_cdada_map_container[idx_entry_size];
+
+        t->index[iterator].idx_map = cdada_map_create(pm_cdada_map_container);
+        if (!t->index[iterator].idx_map) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: unable to allocate index %llx. Destroying.\n", config.name,
+	      config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
 	  destroy = TRUE;
 	  break;
 	}
@@ -1021,98 +1062,81 @@ int pretag_index_allocate(struct id_table *t)
 
 int pretag_index_fill(struct id_table *t, pt_bitmap_t idx_bmap, struct id_entry *ptr)
 {
-  u_int32_t index = 0, iterator = 0, handler_index = 0;
+  u_int32_t iterator = 0, handler_index = 0;
+  int ret;
 
   if (!t) return ERR;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
     if (t->index[iterator].entries && t->index[iterator].bitmap == idx_bmap) {
       struct id_entry e;
-      struct id_index_entry *idie;
       pm_hash_serial_t *hash_serializer;
       pm_hash_key_t *hash_key;
-      int modulo, buckets;
 
-      /* fill serializer in and compute modulo */
+      /* fill serializer in */
       memset(&e, 0, sizeof(struct id_entry));
       hash_serializer = &t->index[iterator].hash_serializer;
       hash_serial_set_off(hash_serializer, 0);
       hash_key = hash_serial_get_key(hash_serializer);
-      buckets = t->index[iterator].modulo;
 
       if (!hash_key) return ERR;
 
       for (handler_index = 0; t->index[iterator].idt_handler[handler_index]; handler_index++) {
 	(*t->index[iterator].idt_handler[handler_index])(&e, hash_serializer, ptr);
       }
-      modulo = cache_crc32(hash_key_get_val(hash_key), hash_key_get_len(hash_key)) % buckets;
-      idie = &t->index[iterator].idx_t[modulo];
 
-      for (index = 0; index < idie->depth; index++) {
-        if (!idie->result[index]) {
-	  hash_dup_key(&idie->hash_key[index], hash_key);
-          idie->result[index] = ptr;
-          break;
-        }
-        /* removing duplicates */
-        else {
-	  pm_id_t saved_pos_idie, saved_pos_ptr;
-	  int match = FALSE;
+      ret = cdada_map_insert(t->index[iterator].idx_map, hash_key_get_val(hash_key), ptr);
+      if (ret == CDADA_E_EXISTS) {
+	u_char key_hexdump[hash_key_get_len(hash_key) * 3];
+	serialize_hex(hash_key_get_val(hash_key), key_hexdump, hash_key_get_len(hash_key));
 
-	  saved_pos_idie = idie->result[index]->pos; idie->result[index]->pos = 0;
-	  saved_pos_ptr = ptr->pos; ptr->pos = 0;
-
-          if (!memcmp(idie->result[index], ptr, sizeof(struct id_entry))) match = TRUE;
-
-          idie->result[index]->pos = saved_pos_idie;
-          ptr->pos = saved_pos_ptr;
-
-	  if (match) {
-	    hash_destroy_key(hash_key);
-	    break;
-	  }
-        }
+	Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] pretag_index_fill(): index=%llx key=%s (dup!)\n",
+	    config.name, config.type, t->filename, (unsigned long long)idx_bmap, key_hexdump);
       }
 
-      if (index == idie->depth) {
-        Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: out of index space %llx. Indexing disabled.\n",
-		config.name, config.type, t->filename, (unsigned long long)idx_bmap);
-	pretag_index_report(t);
-	pretag_index_destroy(t);
-	break;
-      }
+      break;
     }
   }
 
   return SUCCESS;
 }
 
+void pretag_index_print_key(const cdada_map_t *map, const void *key, void *value, void *opaque)
+{
+  struct id_table_index *index = opaque;
+  pm_hash_serial_t *hash_serializer;
+  pm_hash_key_t *hash_key;
+  u_int16_t hash_keylen;
+
+  hash_serializer = &index->hash_serializer;
+  hash_key = hash_serial_get_key(hash_serializer);
+  hash_keylen = hash_key_get_len(hash_key);
+
+  {
+    u_char key_hexdump[hash_keylen * 3];
+    serialize_hex(key, key_hexdump, hash_keylen);
+
+    Log(LOG_DEBUG, "DEBUG ( %s/%s ): pretag_index_print_key(): index=%llx key=%s\n",
+	config.name, config.type, (unsigned long long)index->bitmap, key_hexdump);
+  }
+}
+
 void pretag_index_report(struct id_table *t)
 {
-  u_int32_t iterator = 0, buckets = 0, index = 0;
+  u_int32_t iterator = 0;
 
   if (!t) return;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
     if (t->index[iterator].entries) {
-      u_int32_t bucket_depths[ID_TABLE_INDEX_DEPTH];
-
-      buckets = t->index[iterator].modulo;
-      memset(&bucket_depths, 0, sizeof(bucket_depths));
-
-      for (index = 0; index < buckets; index++) {
-	struct id_index_entry *idie = &t->index[iterator].idx_t[index]; 
-	u_int32_t depth = 0;
-
-	for (depth = 0; idie->result[depth] && depth < idie->depth; depth++) bucket_depths[depth]++;
-      }
-
-      Log(LOG_DEBUG, "DEBUG ( %s/%s ): [%s] pretag_index_report(): index=%llx buckets=%u depths=[0:%u 1:%u 2:%u 3:%u 4:%u 5:%u 6:%u 7:%u] size=%lu\n",
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] pretag_index_report(): index=%llx entries=%u\n",
 	  config.name, config.type, t->filename, (unsigned long long)t->index[iterator].bitmap,
-	  buckets, bucket_depths[0], bucket_depths[1], bucket_depths[2], bucket_depths[3],
-	  bucket_depths[4], bucket_depths[5], bucket_depths[6], bucket_depths[7],
-	  (unsigned long)(buckets * sizeof(struct id_index_entry)));
-    } 
+	  cdada_map_size(t->index[iterator].idx_map));
+
+      if (config.debug) {
+	cdada_map_traverse(t->index[iterator].idx_map, &pretag_index_print_key, &t->index[iterator]);
+      }
+    }
   }
 }
 
@@ -1120,28 +1144,34 @@ void pretag_index_destroy(struct id_table *t)
 {
   pm_hash_serial_t *hash_serializer;
   pm_hash_key_t *hash_key;
-  u_int32_t iterator = 0, buckets = 0, bucket_idx = 0, depth_idx = 0;
+  u_int32_t iterator = 0;
 
   if (!t) return;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
-    if (t->index[iterator].idx_t) {
-      buckets = t->index[iterator].modulo;
-
-      for (bucket_idx = 0; bucket_idx < buckets; bucket_idx++) {
-        for (depth_idx = 0; depth_idx < ID_TABLE_INDEX_DEPTH; depth_idx++) {
-          hash_destroy_key(&t->index[iterator].idx_t[bucket_idx].hash_key[depth_idx]);
-        }
-      }
-
-      free(t->index[iterator].idx_t);
-      Log(LOG_INFO, "INFO ( %s/%s ): [%s] maps_index: destroyed index %llx.\n",
-                config.name, config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
-    }
-
     hash_serializer = &t->index[iterator].hash_serializer;
     hash_key = hash_serial_get_key(hash_serializer);
+
+    /* destroy the hash */
+    if (t->index[iterator].idx_map) {
+      char pm_cdada_map_container[hash_key_get_len(hash_key)];
+      void *idx_map_val;
+
+      Log(LOG_INFO, "INFO ( %s/%s ): [%s] maps_index: destroying index %llx.\n", config.name,
+	  config.type, t->filename, (unsigned long long)t->index[iterator].bitmap);
+
+      while (cdada_map_first(t->index[iterator].idx_map, pm_cdada_map_container, &idx_map_val) == CDADA_SUCCESS) {
+	cdada_map_erase(t->index[iterator].idx_map, pm_cdada_map_container);
+      }
+
+      cdada_map_destroy(t->index[iterator].idx_map);
+      t->index[iterator].idx_map = NULL;
+    }
+
+    /* destroy the serializer */
     hash_destroy_key(hash_key);
+
+    /* cleanup */
     memset(&t->index[iterator], 0, sizeof(struct id_table_index));
   }
 
@@ -1151,11 +1181,11 @@ void pretag_index_destroy(struct id_table *t)
 u_int32_t pretag_index_lookup(struct id_table *t, struct packet_ptrs *pptrs, struct id_entry **index_results, int ir_entries)
 {
   struct id_entry res_fdata;
-  struct id_index_entry *idie;
   pm_hash_serial_t *hash_serializer;
   pm_hash_key_t *hash_key;
-  u_int32_t iterator, index_cc, index_hdlr;
-  int modulo, buckets, iterator_ir;
+  u_int32_t iterator, index_hdlr;
+  int iterator_ir;
+  void *idx_value;
 
   if (!t || !pptrs || !index_results) return 0;
 
@@ -1168,33 +1198,25 @@ u_int32_t pretag_index_lookup(struct id_table *t, struct packet_ptrs *pptrs, str
       hash_serializer = &t->index[iterator].hash_serializer;
       hash_serial_set_off(hash_serializer, 0);
       hash_key = hash_serial_get_key(hash_serializer);
-      buckets = t->index[iterator].modulo;
 
       for (index_hdlr = 0; (*t->index[iterator].fdata_handler[index_hdlr]); index_hdlr++) {
         (*t->index[iterator].fdata_handler[index_hdlr])(&res_fdata, &t->index[iterator].hash_serializer, pptrs);
       }
 
-      modulo = cache_crc32(hash_key_get_val(hash_key), hash_key_get_len(hash_key)) % buckets;
-      idie = &t->index[iterator].idx_t[modulo];
-
-      for (index_cc = 0; idie->result[index_cc] && index_cc < idie->depth; index_cc++) {
-	if (!hash_key_cmp(&idie->hash_key[index_cc], hash_key)) {
-          index_results[iterator_ir] = idie->result[index_cc];
-	  if (iterator_ir < ir_entries) iterator_ir++;
-	  else {
-	    Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: out of index results space. Indexing disabled.\n",
-                config.name, config.type, t->filename);
-	    pretag_index_destroy(t);
-	    memset(index_results, 0, (sizeof(struct id_entry *) * ir_entries));
-	    return 0;
-	  }
-	}
+      cdada_map_find(t->index[iterator].idx_map, hash_key_get_val(hash_key), &idx_value);
+      index_results[iterator_ir] = idx_value;
+      if (iterator_ir < ir_entries) iterator_ir++;
+      else {
+	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] maps_index: out of index results space. Indexing disabled.\n",
+	    config.name, config.type, t->filename);
+	pretag_index_destroy(t);
+	memset(index_results, 0, (sizeof(struct id_entry *) * ir_entries));
+	return 0;
       }
     }
     else break;
   }
 
-  // pretag_index_results_compress(index_results, iterator_ir);
   pretag_index_results_sort(index_results, iterator_ir);
   pretag_index_results_compress_jeqs(index_results, iterator_ir);
 
@@ -1219,26 +1241,6 @@ void pretag_index_results_sort(struct id_entry **index_results, int ir_entries)
   }
 }
 
-void pretag_index_results_compress(struct id_entry **index_results, int ir_entries)
-{
-  u_int32_t j, valid;
-  int i;
-
-  if (!index_results) return;
-
-  for (i = 0; i < ir_entries; i++) {
-    valid = 0;
-    if (!index_results[i]) {
-      for (j = i + 1; j < ir_entries; j++) {
-	if (index_results[j]) valid++;
-        index_results[j-1] = index_results[j];
-      }
-      index_results[ir_entries-1] = NULL;
-      if (!index_results[i] && valid) i--;
-    }
-  }
-}
-
 void pretag_index_results_compress_jeqs(struct id_entry **index_results, int ir_entries)
 {
   u_int32_t i, j, x;
