@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -38,6 +38,7 @@ extern int PT_map_agent_id_handler(char *, struct id_entry *, char *, struct plu
 extern int PT_map_flowset_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_fwdstatus_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_sample_type_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int PT_map_is_bi_flow_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_direction_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_nat_event_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_src_as_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -58,6 +59,7 @@ extern int PT_map_vlan_id_handler(char *, struct id_entry *, char *, struct plug
 extern int PT_map_cvlan_id_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_src_net_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_dst_net_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int PT_map_is_multicast_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_set_tos_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_entry_label_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 extern int PT_map_jeq_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
@@ -82,6 +84,9 @@ extern int PT_map_index_entries_src_mac_handler(struct id_entry *, pm_hash_seria
 extern int PT_map_index_entries_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_entries_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_entries_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *); 
+extern int PT_map_index_entries_src_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_dst_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_entries_is_multicast_handler(struct id_entry *, pm_hash_serial_t *, void *);
 extern int PT_map_index_entries_fwdstatus_handler(struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_ip_handler(struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_input_handler(struct id_entry *, pm_hash_serial_t *, void *); 
@@ -99,6 +104,9 @@ extern int PT_map_index_fdata_src_mac_handler(struct id_entry *, pm_hash_serial_
 extern int PT_map_index_fdata_dst_mac_handler(struct id_entry *, pm_hash_serial_t *, void *); 
 extern int PT_map_index_fdata_vlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
 extern int PT_map_index_fdata_cvlan_id_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_src_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_dst_net_handler(struct id_entry *, pm_hash_serial_t *, void *);
+extern int PT_map_index_fdata_is_multicast_handler(struct id_entry *, pm_hash_serial_t *, void *);
 extern int PT_map_index_fdata_fwdstatus_handler(struct id_entry *, pm_hash_serial_t *, void *);
 
 /* BPAS_*: bgp_peer_as_src map specific handlers */
@@ -147,6 +155,7 @@ extern int pretag_dst_roa_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_src_comms_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_comms_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_sample_type_handler(struct packet_ptrs *, void *, void *);
+extern int pretag_is_bi_flow_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_direction_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_nat_event_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_mpls_vpn_rd_handler(struct packet_ptrs *, void *, void *);
@@ -156,6 +165,7 @@ extern int pretag_dst_mac_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_src_net_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_dst_net_handler(struct packet_ptrs *, void *, void *);
+extern int pretag_is_multicast_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_forwarding_status_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_cvlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int pretag_set_tos_handler(struct packet_ptrs *, void *, void *);
@@ -174,6 +184,7 @@ extern int SF_pretag_vlan_id_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_mpls_pw_id_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_src_net_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_dst_net_handler(struct packet_ptrs *, void *, void *);
+extern int SF_pretag_is_multicast_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_sample_type_handler(struct packet_ptrs *, void *, void *);
 extern int SF_pretag_direction_handler(struct packet_ptrs *, void *, void *);
 
@@ -191,17 +202,17 @@ extern int BTA_lookup_bgp_port_handler(struct packet_ptrs *, void *, void *);
 extern int BITR_mpls_label_bottom_handler(struct packet_ptrs *, void *, void *);
 extern int BITR_mpls_vpn_id_handler(struct packet_ptrs *, void *, void *);
 
-extern void pcap_interfaces_map_validate(char *, struct plugin_requests *);
-extern int pcap_interfaces_map_ifindex_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
-extern int pcap_interfaces_map_ifname_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
-extern int pcap_interfaces_map_direction_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern void pm_pcap_interfaces_map_validate(char *, struct plugin_requests *);
+extern int pm_pcap_interfaces_map_ifindex_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int pm_pcap_interfaces_map_ifname_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
+extern int pm_pcap_interfaces_map_direction_handler(char *, struct id_entry *, char *, struct plugin_requests *, int);
 
-extern void pcap_interfaces_map_initialize(struct pcap_interfaces *);
-extern void pcap_interfaces_map_load(struct pcap_interfaces *);
-extern void pcap_interfaces_map_destroy(struct pcap_interfaces *);
-extern void pcap_interfaces_map_copy(struct pcap_interfaces *, struct pcap_interfaces *);
-extern u_int32_t pcap_interfaces_map_lookup_ifname(struct pcap_interfaces *, char *);
-extern struct pcap_interface *pcap_interfaces_map_getentry_by_ifname(struct pcap_interfaces *, char *);
-extern char *pcap_interfaces_map_getnext_ifname(struct pcap_interfaces *, int *);
+extern void pm_pcap_interfaces_map_initialize(struct pm_pcap_interfaces *);
+extern void pm_pcap_interfaces_map_load(struct pm_pcap_interfaces *);
+extern void pm_pcap_interfaces_map_destroy(struct pm_pcap_interfaces *);
+extern void pm_pcap_interfaces_map_copy(struct pm_pcap_interfaces *, struct pm_pcap_interfaces *);
+extern u_int32_t pm_pcap_interfaces_map_lookup_ifname(struct pm_pcap_interfaces *, char *);
+extern struct pm_pcap_interface *pm_pcap_interfaces_map_getentry_by_ifname(struct pm_pcap_interfaces *, char *);
+extern char *pm_pcap_interfaces_map_getnext_ifname(struct pm_pcap_interfaces *, int *);
 
 #endif //PRETAG_HANDLERS_H
