@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -24,6 +24,9 @@
 
 /* includes */
 #include "base64.h"
+#ifdef WITH_UNYTE_UDP_NOTIF
+#include <unyte-udp-notif/unyte_udp_collector.h>
+#endif
 
 /* defines */
 #define TELEMETRY_TCP_PORT		1620
@@ -33,6 +36,8 @@
 #define TELEMETRY_PEER_TIMEOUT_INTERVAL	60
 #define TELEMETRY_UDP_MAXMSG		65535
 #define TELEMETRY_LOG_STATS_INTERVAL	120	
+#define TELEMETRY_KAFKA_FD		INT_MAX	
+#define TELEMETRY_UDP_NOTIF_FD		INT_MAX
 
 #define TELEMETRY_DECODER_UNKNOWN	0
 #define TELEMETRY_DECODER_JSON		1
@@ -68,6 +73,13 @@
 #define TELEMETRY_LOGDUMP_ET_LOG	BGP_LOGDUMP_ET_LOG
 #define TELEMETRY_LOGDUMP_ET_DUMP	BGP_LOGDUMP_ET_DUMP
 
+#ifdef WITH_UNYTE_UDP_NOTIF
+#define TELEMETRY_DEFAULT_UNYTE_UDP_NOTIF_NMSGS	1
+#define TELEMETRY_UDP_NOTIF_ENC_CBOR		0
+#define TELEMETRY_UDP_NOTIF_ENC_JSON		1
+#define TELEMETRY_UDP_NOTIF_ENC_XML		2
+#endif
+
 struct telemetry_cisco_hdr_v0 {
   u_int32_t type;
   u_int32_t flags;
@@ -90,6 +102,9 @@ struct telemetry_data {
 #if defined WITH_ZMQ
   void *zmq_host;
 #endif
+#if defined WITH_KAFKA
+  void *kafka_msg;
+#endif
 
   telemetry_stats global_stats;
   time_t now;
@@ -136,7 +151,7 @@ typedef struct _telemetry_peer_timeout telemetry_peer_timeout;
 
 /* prototypes */
 extern void telemetry_wrapper();
-extern void telemetry_daemon(void *);
+extern int telemetry_daemon(void *);
 extern void telemetry_prepare_thread(struct telemetry_data *);
 extern void telemetry_prepare_daemon(struct telemetry_data *);
 
@@ -146,4 +161,5 @@ extern telemetry_misc_structs *telemetry_misc_db;
 extern telemetry_peer *telemetry_peers;
 extern void *telemetry_peers_cache;
 extern telemetry_peer_timeout *telemetry_peers_timeout; 
+extern int zmq_input, kafka_input, unyte_udp_notif_input;
 #endif //TELEMETRY_H
