@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -40,10 +40,14 @@
 #ifdef WITH_KAFKA
 #include "kafka_plugin.h"
 #endif
+#ifdef WITH_REDIS
+#include "hiredis/hiredis.h"
+#endif
 #include "pmacct-data.h"
 #include "ip_flow.h"
 #include "classifier.h"
 #include "plugin_hooks.h"
+#include "pkt_handlers.h"
 #include <netdb.h>
 #include <sys/file.h>
 #include <sys/utsname.h>
@@ -484,13 +488,15 @@ void link_latest_output_file(char *link_filename, char *filename_to_link)
 
   if (!link_filename || !filename_to_link) return;
 
+  buf[0] = '\0';
+
   if (config.files_uid) owner = config.files_uid;
   if (config.files_gid) group = config.files_gid;
 
   /* create dir structure to get to file, if needed */
   ret = mkdir_multilevel(link_filename, TRUE, owner, group);
   if (ret) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] link_latest_output_file(): mkdir_multilevel() failed.\n", config.name, config.type, buf);
+    Log(LOG_ERR, "ERROR ( %s/%s ): [%s] link_latest_output_file(): mkdir_multilevel() failed.\n", config.name, config.type, link_filename);
     return;
   }
 
@@ -502,7 +508,7 @@ void link_latest_output_file(char *link_filename, char *filename_to_link)
 
     memset(&s1, 0, sizeof(struct stat));
     memset(&s2, 0, sizeof(struct stat));
-    readlink(link_filename, buf, SRVBUFLEN);
+    ret = readlink(link_filename, buf, SRVBUFLEN);
 
     /* filename_to_link is newer than buf or buf is un-existing */
     stat(buf, &s1);
@@ -513,10 +519,13 @@ void link_latest_output_file(char *link_filename, char *filename_to_link)
 
   if (rewrite_latest) {
     unlink(link_filename);
-    symlink(filename_to_link, link_filename);
+    ret = symlink(filename_to_link, link_filename);
 
-    if (lchown(link_filename, owner, group) == -1)
-      Log(LOG_WARNING, "WARN ( %s/%s ): link_latest_output_file(): unable to chown() '%s'.\n", config.name, config.type, link_filename);
+    if (!ret) {
+      if (lchown(link_filename, owner, group) == -1) {
+	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] link_latest_output_file(): unable to chown().\n", config.name, config.type, link_filename);
+      }
+    }
   }
 }
 
@@ -585,7 +594,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_SQL_TABLE || type == DYN_STR_PRINT_FILE) &&
 	     !strncmp(ptr_var, hst_string, var_len)) {
@@ -607,7 +616,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if (!strncmp(ptr_var, psi_string, var_len)) {
       char empty_peer_src_ip[] = "null";
@@ -633,7 +642,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if (!strncmp(ptr_var, tag_string, var_len)) {
       pm_id_t zero_tag = 0;
@@ -655,7 +664,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if (!strncmp(ptr_var, tag2_string, var_len)) {
       pm_id_t zero_tag = 0;
@@ -677,7 +686,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if (!strncmp(ptr_var, post_tag_string, var_len)) {
       int len;
@@ -697,7 +706,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if (!strncmp(ptr_var, post_tag2_string, var_len)) {
       int len;
@@ -717,7 +726,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, src_host_string, var_len)) {
       char empty_src_host[] = "null";
@@ -743,7 +752,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, dst_host_string, var_len)) {
       char empty_dst_host[] = "null";
@@ -769,7 +778,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, src_port_string, var_len)) {
       u_int16_t zero_port = 0;
@@ -791,7 +800,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, dst_port_string, var_len)) {
       u_int16_t zero_port = 0;
@@ -813,7 +822,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, proto_string, var_len)) {
       int null_proto = -1;
@@ -835,7 +844,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
     else if ((type == DYN_STR_KAFKA_PART) && !strncmp(ptr_var, in_iface_string, var_len)) {
       int null_in_iface = 0;
@@ -857,7 +866,7 @@ int handle_dynname_internal_strings(char *new, int newlen, char *old, struct pri
       *ptr_start = '\0';
 
       if (len >= rem_len) return ERR;
-      strncat(new, buf, len);
+      strncat(new, buf, rem_len);
     }
 
     if (sub_len) ptr_substr = ptr_var + sub_len;
@@ -1415,24 +1424,43 @@ int timeval_cmp(struct timeval *a, struct timeval *b)
   return INT_MIN; /* silence compiler warning */
 }
 
+void signal_kittens(int sig, int amicore_check)
+{
+  struct plugins_list_entry *list = plugins_list;
+  struct channels_list_entry *chptr = channels_list;
+
+  /* round #1: safety check, am i the core process? */
+  if (amicore_check) {
+    if (!chptr || chptr->core_pid != getpid()) {
+      return;
+    }
+  }
+
+  /* round #2: do it */
+  list = plugins_list;
+
+  while (list) {
+    if (memcmp(list->type.string, "core", sizeof("core"))) {
+      kill(list->pid, sig);
+    }
+
+    list = list->next;
+  }
+}
+
 /*
  * exit_all(): Core Process exit lane. Not meant to be a nice shutdown method: it is
  * an exit() replacement that sends kill signals to the plugins.
  */
 void exit_all(int status)
 {
-  struct plugins_list_entry *list = plugins_list;
-
 #if defined (SOLARIS)
   signal(SIGCHLD, SIG_IGN);
 #else
   signal(SIGCHLD, ignore_falling_child);
 #endif
 
-  while (list) {
-    if (memcmp(list->type.string, "core", sizeof("core"))) kill(list->pid, SIGKILL);
-    list = list->next;
-  }
+  signal_kittens(SIGKILL, TRUE);
 
   wait(NULL);
   if (config.pidfile) remove_pid_file(config.pidfile);
@@ -1558,13 +1586,13 @@ void set_default_preferences(struct configuration *cfg)
     if (!cfg->nfacctd_as) cfg->nfacctd_as = NF_AS_KEEP;
     set_truefalse_nonzero(&cfg->nfacctd_disable_checks);
   }
-  if (!cfg->nfacctd_bgp_peer_as_src_type) cfg->nfacctd_bgp_peer_as_src_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_std_comm_type) cfg->nfacctd_bgp_src_std_comm_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_ext_comm_type) cfg->nfacctd_bgp_src_ext_comm_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_lrg_comm_type) cfg->nfacctd_bgp_src_lrg_comm_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_as_path_type) cfg->nfacctd_bgp_src_as_path_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_local_pref_type) cfg->nfacctd_bgp_src_local_pref_type = BGP_SRC_PRIMITIVES_KEEP;
-  if (!cfg->nfacctd_bgp_src_med_type) cfg->nfacctd_bgp_src_med_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_peer_as_src_type) cfg->bgp_daemon_peer_as_src_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_std_comm_type) cfg->bgp_daemon_src_std_comm_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_ext_comm_type) cfg->bgp_daemon_src_ext_comm_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_lrg_comm_type) cfg->bgp_daemon_src_lrg_comm_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_as_path_type) cfg->bgp_daemon_src_as_path_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_local_pref_type) cfg->bgp_daemon_src_local_pref_type = BGP_SRC_PRIMITIVES_KEEP;
+  if (!cfg->bgp_daemon_src_med_type) cfg->bgp_daemon_src_med_type = BGP_SRC_PRIMITIVES_KEEP;
 }
 
 void set_shadow_status(struct packet_ptrs *pptrs)
@@ -1591,7 +1619,7 @@ void *pm_malloc(size_t size)
 
   obj = (unsigned char *) malloc(size);
   if (!obj) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): Unable to grab enough memory (requested: %lu bytes). Exiting ...\n",
+    Log(LOG_ERR, "ERROR ( %s/%s ): Unable to grab enough memory (requested: %zu bytes). Exiting ...\n",
     config.name, config.type, size);
     exit_gracefully(1);
   }
@@ -1848,17 +1876,18 @@ char *write_sep(char *sep, int *count)
   }
 }
 
-void version_daemon(char *header)
+void version_daemon(int acct_type, char *header)
 {
   struct utsname utsbuf;
 
-  printf("%s %s (%s)\n\n", header, PMACCT_VERSION, PMACCT_BUILD);
+  printf("%s %s [%s]\n\n", header, PMACCT_VERSION, PMACCT_BUILD);
 
   printf("Arguments:\n");
   printf("%s\n", PMACCT_COMPILE_ARGS);
   printf("\n");
 
   printf("Libs:\n");
+  printf("cdada %s\n", cdada_get_ver());
   printf("%s\n", pcap_lib_version());
 #ifdef WITH_MYSQL
   MY_mysql_get_version();
@@ -1884,18 +1913,54 @@ void version_daemon(char *header)
 #ifdef WITH_ZMQ
   printf("ZeroMQ %u.%u.%u\n", ZMQ_VERSION_MAJOR, ZMQ_VERSION_MINOR, ZMQ_VERSION_PATCH); 
 #endif
+#ifdef WITH_REDIS
+  printf("Redis %u.%u.%u\n", HIREDIS_MAJOR, HIREDIS_MINOR, HIREDIS_PATCH);
+#endif
+#ifdef WITH_GNUTLS
+  printf("GnuTLS %u.%u.%u\n", GNUTLS_VERSION_MAJOR, GNUTLS_VERSION_MINOR, GNUTLS_VERSION_PATCH);
+#endif
 #ifdef WITH_AVRO
   printf("avro-c\n");
 #endif
 #ifdef WITH_SERDES
   printf("serdes\n");
 #endif
+#ifdef WITH_UNYTE_UDP_NOTIF
+  printf("unyte-udp-notif\n");
+#endif
 #ifdef WITH_NDPI
   printf("nDPI %s\n", ndpi_revision());
 #endif
 #ifdef WITH_NFLOG
   printf("netfilter_log\n");
 #endif
+
+  if (acct_type == ACCT_NF || acct_type == ACCT_SF || acct_type == ACCT_PM) {
+    printf("\n");
+
+    printf("Plugins:\n");
+    printf("memory\n");
+    printf("print\n");
+    printf("nfprobe\n");
+    printf("sfprobe\n");
+    printf("tee\n");
+#ifdef WITH_MYSQL
+    printf("mysql\n");
+#endif
+#ifdef WITH_PGSQL
+    printf("postgresql\n");
+#endif
+#ifdef WITH_SQLITE3
+    printf("sqlite\n");
+#endif
+#ifdef WITH_RABBITMQ
+    printf("amqp\n");
+#endif
+#ifdef WITH_KAFKA
+    printf("kafka\n");
+#endif
+  }
+
   printf("\n");
 
   if (!uname(&utsbuf)) {
@@ -2012,8 +2077,14 @@ void compose_timestamp(char *buf, int buflen, struct timeval *tv, int usec, int
     if (!utc) time2 = localtime(&time1);
     else time2 = gmtime(&time1);
     
-    if (!rfc3339) slen = strftime(buf, buflen, "%Y-%m-%d %H:%M:%S", time2);
-    else slen = strftime(buf, buflen, "%Y-%m-%dT%H:%M:%S", time2);
+    if (tv->tv_sec) {
+      if (!rfc3339) slen = strftime(buf, buflen, "%Y-%m-%d %H:%M:%S", time2);
+      else slen = strftime(buf, buflen, "%Y-%m-%dT%H:%M:%S", time2);
+    }
+    else {
+      if (!rfc3339) slen = snprintf(buf, buflen, "0000-00-00 00:00:00");
+      else slen = snprintf(buf, buflen, "0000-00-00T00:00:00");
+    }
 
     if (usec) snprintf((buf + slen), (buflen - slen), ".%.6ld", (long)tv->tv_usec);
     if (rfc3339) append_rfc3339_timezone(buf, buflen, time2);
@@ -2641,35 +2712,6 @@ int vlen_prims_delete(struct pkt_vlen_hdr_primitives *hdr, pm_cfgreg_t wtc /*, o
   return ret;
 }
 
-void replace_string(char *str, int string_len, char *var, char *value)
-{
-  char *ptr_start, *ptr_end;
-  char buf[string_len];
-  int ptr_len, len;
-
-  if (!str || !var || !value) return;
-
-  if (!strchr(str, '$')) return;
-
-  if (string_len < ((strlen(str) + strlen(value)) - strlen(var))) return;
-
-  ptr_start = strstr(str, var);
-  if (ptr_start) {
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(var);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    snprintf(buf, string_len, "%s", value);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(str, buf, len);
-  }
-}
-
 int delete_line_from_file(int index, char *path)
 {
   int len = strlen(path) + 5;
@@ -2986,7 +3028,12 @@ void generate_random_string(char *s, const int len)
   s[len] = '\0';
 }
 
-void open_pcap_savefile(struct pcap_device *dev_ptr, char *file)
+void pm_pcap_device_initialize(struct pm_pcap_devices *map)
+{
+  memset(map, 0, sizeof(struct pm_pcap_devices));
+}
+
+void open_pcap_savefile(struct pm_pcap_device *dev_ptr, char *file)
 {
   char errbuf[PCAP_ERRBUF_SIZE];
   int idx;
@@ -3002,7 +3049,7 @@ void open_pcap_savefile(struct pcap_device *dev_ptr, char *file)
       dev_ptr->data = &_devices[idx];
   }
 
-  if (!dev_ptr->data->handler) {
+  if (!dev_ptr->data || !dev_ptr->data->handler) {
     Log(LOG_ERR, "ERROR ( %s/core ): pcap_savefile: unsupported link layer.\n", config.name);
     exit_gracefully(1);
   }
@@ -3144,6 +3191,62 @@ char *null_terminate(char *str, int len)
   return loc;
 }
 
+char *uint_print(void *value, int len, int flip)
+{
+  char *buf = NULL;
+  ssize_t buflen = 0;
+
+  switch(len) {
+  case 1:
+    {
+      u_int8_t *u8 = (u_int8_t *) value;
+
+      buflen = snprintf(NULL, 0, "%u", (*u8)); 
+      buf = malloc(buflen + 1);
+      snprintf(buf, (buflen + 1), "%u", (*u8));
+    }
+    break;
+  case 2:
+    {
+      u_int16_t u16h, *u16 = (u_int16_t *) value;
+
+      if (flip) u16h = ntohs((*u16));
+      else u16h = (*u16);
+
+      buflen = snprintf(NULL, 0, "%u", u16h);
+      buf = malloc(buflen + 1);
+      snprintf(buf, (buflen + 1), "%u", u16h);
+    }
+    break;
+  case 4:
+    {
+      u_int32_t u32h, *u32 = (u_int32_t *) value;
+
+      if (flip) u32h = ntohl((*u32));
+      else u32h = (*u32);
+
+      buflen = snprintf(NULL, 0, "%u", u32h);
+      buf = malloc(buflen + 1);
+      snprintf(buf, (buflen + 1), "%u", u32h);
+    }
+    break;
+  case 8:
+    {
+      u_int64_t u64h, *u64 = (u_int64_t *) value;
+
+      if (flip) u64h = pm_ntohll((*u64));
+      else u64h = (*u64);
+
+      buflen = snprintf(NULL, 0, "%"PRIu64, u64h);
+      buf = malloc(buflen + 1);
+      snprintf(buf, (buflen + 1), "%"PRIu64, u64h);
+    }
+    break;
+  }
+
+  return buf;
+}
+
 void reload_logs()
 {
   int logf;
@@ -3164,3 +3267,88 @@ void reload_logs()
     config.logfile_fd = open_output_file(config.logfile, "a", FALSE);
   }
 }
+
+int is_empty_256b(void *area, int len)
+{
+  if (len <= SRVBUFLEN) {
+    if (!memcmp(area, empty_mem_area_256b, len)) {
+      return TRUE;
+    }
+    else {
+      return FALSE;
+    } 
+  }
+
+  return ERR;
+}
+
+ssize_t pm_recv(int sockfd, void *buf, size_t len, int flags, unsigned int seconds)
+{
+  ssize_t ret;
+
+  if (flags == MSG_WAITALL) {
+    alarm(seconds);
+  }
+
+  ret = recv(sockfd, buf, len, flags);
+
+  alarm(0);
+
+  return ret;
+}
+
+/* flow type to address family */
+int ft2af(u_int8_t ft)
+{
+  if (ft == PM_FTYPE_IPV4 || ft == PM_FTYPE_VLAN_IPV4 ||
+      ft == PM_FTYPE_MPLS_IPV4 || ft == PM_FTYPE_VLAN_MPLS_IPV4) {
+    return AF_INET;
+  }
+  else if (ft == PM_FTYPE_IPV6 || ft == PM_FTYPE_VLAN_IPV6 ||
+           ft == PM_FTYPE_MPLS_IPV6 || ft == PM_FTYPE_VLAN_MPLS_IPV6) {
+    return AF_INET6;
+  }
+
+  return ERR;
+}
+
+void distribute_work(struct pm_dump_runner *pdr, u_int64_t seqno, int workers, u_int64_t last_elem) 
+{
+  int id, idx;
+
+  if (!pdr) return;
+
+  memset(pdr, 0, (workers * sizeof(struct pm_dump_runner)));
+
+  for (idx = 0, id = 1; idx < workers; idx++, id++) {
+    struct pm_dump_runner *worker = &pdr[idx];
+
+    worker->id = id;
+    worker->seq = seqno;
+    worker->noop = FALSE;
+
+    if (workers <= last_elem) {
+      int ratio = last_elem / workers;
+
+      if (idx < (workers - 1)) {
+	worker->first = idx * ratio;
+	worker->last = (((idx * ratio) + ratio) - 1);
+      }
+      else {
+	worker->first = idx * ratio;
+	worker->last = (last_elem - 1);
+      }
+    }
+    else {
+      if (idx < last_elem) {
+	worker->first = idx;
+	worker->last = idx;
+      }
+      else {
+	worker->noop = TRUE;
+	worker->first = FALSE;
+	worker->last = FALSE;
+      }
+    }
+  }
+}
