@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2021 by Paolo Lucente
 */
 
 /*
@@ -70,6 +70,7 @@ extern int check_bosbit(u_char *);
 extern u_int32_t decode_mpls_label(u_char *);
 extern void encode_mpls_label(char *, u_int32_t);
 extern int timeval_cmp(struct timeval *, struct timeval *);
+extern void signal_kittens(int, int);
 extern void exit_all(int);
 extern void exit_plugin(int);
 extern void exit_gracefully(int);
@@ -82,7 +83,8 @@ extern void set_sampling_table(struct packet_ptrs_vector *, u_char *);
 extern void set_shadow_status(struct packet_ptrs *);
 extern void set_default_preferences(struct configuration *);
 extern FILE *open_output_file(char *, char *, int);
-extern void open_pcap_savefile(struct pcap_device *, char *);
+extern void open_pcap_savefile(struct pm_pcap_device *, char *);
+extern void pm_pcap_device_initialize(struct pm_pcap_devices *);
 extern void link_latest_output_file(char *, char *);
 extern void close_output_file(FILE *);
 extern int handle_dynname_internal_strings(char *, int, char *, struct primitives_ptrs *, int);
@@ -101,14 +103,18 @@ extern int load_labels(char *, struct pretag_label_filter *, char *);
 extern int evaluate_tags(struct pretag_filter *, pm_id_t);
 extern int evaluate_labels(struct pretag_label_filter *, pt_label_t *);
 extern char *write_sep(char *, int *);
-extern void version_daemon(char *);
+extern void version_daemon(int, char *);
 extern void set_truefalse_nonzero(int *);
 extern char *ip_proto_print(u_int8_t, char *, int);
 extern void parse_hostport(const char *, struct sockaddr *, socklen_t *);
 extern bool is_prime(u_int32_t);
 extern u_int32_t next_prime(u_int32_t);
 extern char *null_terminate(char *, int);
+extern char *uint_print(void *, int, int);
 extern void reload_logs();
+extern int is_empty_256b(void *, int);
+extern ssize_t pm_recv(int, void *, size_t, int, unsigned int);
+extern int ft2af(u_int8_t);
 
 extern char *compose_json_str(void *);
 extern void write_and_free_json(FILE *, void *);
@@ -177,7 +183,6 @@ extern int pm_alphasort(const void *, const void *);
 extern void *pm_tsearch(const void *, void **, int (*compar)(const void *, const void *), size_t);
 extern void pm_tdestroy(void **, void (*free_node)(void *));
 
-extern void replace_string(char *, int, char *, char *);
 extern int delete_line_from_file(int, char *);
 
 extern void generate_random_string(char *, const int);
@@ -191,4 +196,5 @@ extern time_t P_broker_timers_get_last_fail(struct p_broker_timers *);
 
 extern primptrs_func primptrs_funcs[PRIMPTRS_FUNCS_N];
 
+extern void distribute_work(struct pm_dump_runner *, u_int64_t, int, u_int64_t);
 #endif //UTIL_H
